<?php

use Cake\ORM\TableRegistry;
use Migrations\AbstractMigration;
use Refae\Model\Entity\Document;

class AddIdentifiantToDocuments extends AbstractMigration
{
    /**
     * Migrate Up
     *
     * @return void
     * @throws Exception
     */
    public function up()
    {
        $Documents = TableRegistry::getTableLocator()->get("Documents");
        $conn = $Documents->getConnection();
        $conn->begin();

        $table = $this->table('documents');
        $table->addColumn('identifiant', 'string', [
            'default' => null,
            'limit' => 255,
            'null' => true,
        ]);
        $table->addColumn('version', 'integer', [
            'default' => null,
            'limit' => 11,
            'null' => true,
        ]);
        $table->addColumn('note_version', 'text', [
            'default' => null,
            'null' => true,
        ]);
        $table->update();

        $documents = $Documents->find()->all()->toArray();
        /** @var Document $document */
        foreach ($documents as $document) {
            $data = [
                'identifiant' => preg_replace('/[^\w]/', '_', iconv("utf-8", "ascii//TRANSLIT", $document->get('name'))),
                'version' => 1
            ];
            $i = 1;
            while ($exists = $Documents->exists($data)) {
                $i++;
                $data['version'] = $i;
            }
            $Documents->patchEntity($document, $data);
            $Documents->save($document);
        }

        $table->addIndex([
            'identifiant',
            'version',
        ], [
            'name' => 'UNIQUE_IDENTIFIANT_VERSION',
            'unique' => true,
        ]);
        $table->update();

        $table->changeColumn('identifiant', 'string', [
            'default' => null,
            'limit' => 255,
            'null' => false,
        ]);
        $table->changeColumn('version', 'integer', [
            'default' => null,
            'limit' => 11,
            'null' => false,
        ]);
        $table->update();

        $conn->commit();
    }

    /**
     * Migrate Down
     *
     * @return void
     */
    public function down()
    {
        $table = $this->table('documents');
        $table->removeIndex(['UNIQUE_IDENTIFIANT_VERSION']);
        $table->removeColumn('identifiant');
        $table->removeColumn('version');
        $table->removeColumn('note_version');
        $table->update();
    }
}
