<?php

use Cake\ORM\TableRegistry;
use Migrations\AbstractMigration;
use Refae\Model\Entity\Document;

class AddUidToDocuments extends AbstractMigration
{
    /**
     * Migrate Up
     *
     * @return void
     * @throws Exception
     */
    public function up()
    {
        $Documents = TableRegistry::getTableLocator()->get("Documents");
        $conn = $Documents->getConnection();
        $conn->begin();

        $table = $this->table('documents');
        $table->addColumn('uid', 'string', [
            'default' => null,
            'limit' => 255,
            'null' => true,
        ]);
        $table->update();

        $documents = $Documents->find();
        /** @var Document $document */
        foreach ($documents as $document) {
            if ($document->get('uid')) {
                continue;
            }
            $data = [
                'uid' => uniqid('', true)
            ];
            $Documents->patchEntity($document, $data);
            $Documents->save($document, ['version-upgrade' => false]);
        }

        $table->addIndex([
            'uid',
        ], [
            'name' => 'UNIQUE_UID',
            'unique' => true,
        ]);
        $table->update();

        $table->changeColumn('uid', 'string', [
            'default' => null,
            'limit' => 255,
            'null' => false,
        ]);

        $table->update();

        $conn->commit();
    }

    /**
     * Migrate Down
     *
     * @return void
     */
    public function down()
    {
        $table = $this->table('documents');
        $table->removeIndex(['UNIQUE_UID']);
        $table->removeColumn('uid');
        $table->update();
    }
}
