<?php

use Cake\Core\Configure;
use Cake\ORM\TableRegistry;
use Libriciel\Filesystem\Utility\Filesystem;
use Migrations\AbstractMigration;
use Refae\Model\Entity\Document;
use Refae\Model\Table\DocumentsTable;
use Refae\Model\Table\FilesTable;

class AlterDocumentsAlterColumnSchemaProfilArchivage extends AbstractMigration
{
    /**
     * Migrate Up
     *
     * @return void
     * @throws Exception
     */
    public function up()
    {
        /** @var DocumentsTable $Documents */
        $Documents = TableRegistry::getTableLocator()->get("Documents");
        /** @var FilesTable $Files */
        $Files = TableRegistry::getTableLocator()->get("Files", ['className' => 'Refae\\Model\\Table\\FilesTable']);
        $conn = $Documents->getConnection();
        $conn->begin();

        $table = $this->table('files');
        $table->changeColumn('user_id', 'integer', [
            'default' => null,
            'limit' => 11,
            'null' => true,
        ]);
        $table->update();

        $table = $this->table('documents');
        $table->addColumn('schema_file_id', 'integer', [
            'default' => null,
            'limit' => 11,
            'null' => true,
        ]);
        $table->update();
        $table
            ->addForeignKey(
                'schema_file_id',
                'files',
                'id',
                [
                    'update' => 'CASCADE',
                    'delete' => 'SET NULL'
                ]
            )
            ->update();

        $results = $Documents->find()
            ->where(['schema_profil_archivage IS NOT' => null])
            ->all()
            ->toArray()
        ;
        /** @var Document $document */
        foreach ($results as $document) {
            $path = $document->get('schema_profil_archivage');
            if (is_file($path)) {
                $file = $Files->newEntityFromUploadedFile($path);
                $Files->save($file);
                $document->set('schema_file_id', $file->get('id'));
                $Documents->save($document, ['version-upgrade' => false]);
            }
        }

        $table->removeColumn('schema_profil_archivage');

        $conn->commit();
        $Files->filesystemCommit();
    }

    /**
     * Migrate Down
     *
     * @return void
     */
    public function down()
    {
        /** @var DocumentsTable $Documents */
        $Documents = TableRegistry::getTableLocator()->get("Documents");
        /** @var FilesTable $Files */
        $Files = TableRegistry::getTableLocator()->get("Files");
        $conn = $Files->getConnection();
        Filesystem::begin('down');
        $conn->begin();
        $table = $this->table('documents');
        $Documents->getSchema()->addColumn('schema_profil_archivage', [
            'type' => 'string',
            'length' => 255,
        ]);
        $table->addColumn('schema_profil_archivage', 'string', [
            'default' => null,
            'limit' => 255,
            'null' => true,
        ]);
        $table->update();
        $results = $Documents->find()
            ->where(['schema_file_id IS NOT' => null])
            ->contain(['Files'])
            ->all()
            ->toArray()
        ;
        $targetdir = Configure::read('Volumes.profil_archivage')
            ?: ROOT . DS . 'data' . DS . 'profil_archivage'
        ;
        /** @var Document $document */
        foreach ($results as $document) {
            $file = $document->get('file');
            $filename = $Files->buildFile($file, $targetdir);
            $document->set('schema_profil_archivage', $filename);
            $Documents->save($document, ['version-upgrade' => false]);
        }

        $table->removeColumn('schema_file_id');
        $table->update();

        $Files->deleteAll([]);

        $table = $this->table('files');
        $table->changeColumn('user_id', 'integer', [
            'default' => null,
            'limit' => 11,
            'null' => false,
        ]);
        $table->update();

        $conn->commit();
        Filesystem::commit('down');
    }
}
