<?php
/**
 * Refae\Controller\DocumentsController
 */
namespace Refae\Controller;

use AsalaeCore\Controller\RenderDataTrait;
use Cake\Event\Event;
use Exception;
use Refae\Model\Entity\Document;
use Refae\Model\Table\CategoriesTable;

/**
 * Controller de la table users
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017-2018, Libriciel SCOP
 * @license   https://opensource.org/licenses/AGPL-3.0
 * @property CategoriesTable $Categories
 * @property bool|object CategoriesDocuments
 */
class CategoriesController extends AppController
{
    /**
     * Traits
     */
    use RenderDataTrait;

    /**
     * Options de pagination par défaut
     *
     * @var array
     */
    public $paginate = [
        'limit' => 10,
        'order' => [
            'Documents.id' => 'desc'
        ],
        'contain' => ['Categories'],
        'sortWhitelist' => [
            'name',
            'support',
            'file_format',
            'dua',
            'communicabilite',
            'sort_final',
            'Categories.name'
        ]
    ];

    /**
     * Called before the controller action. You can use this method to configure and customize components
     * or perform logic that needs to happen before each controller action.
     *
     * @param \Cake\Event\Event $event An Event instance
     * @return \Cake\Http\Response|null
     * @link https://book.cakephp.org/3.0/en/controllers.html#request-life-cycle-callbacks
     */
    public function beforeFilter(Event $event)
    {
        $this->Auth->allow('index');
        return parent::beforeFilter($event);
    }

    /**
     * Ajout d'une categorie
     * @param string $type
     * @param string $id
     * @throws Exception
     */
    public function add(string $type = 'table', string $id = '')
    {
        $this->loadModel('Categories');
        $categorie = $this->Categories->newEntity();
        if ($id) {
            $this->Categories->patchEntity($categorie, ['parent_id' => $id]);
        }

        if ($this->getRequest()->is('post')) {
            $data = $this->getRequest()->getData();
            $data['type'] = $type;
            $this->Categories->patchEntity($categorie, $data);
            $this->loadComponent('AsalaeCore.Modal');
            if ($this->Categories->save($categorie)) {
                $this->Modal->success();
            } else {
                $this->Modal->fail();
            }
        }

        if ($type !== 'table') {
            $categories = $this->Categories->find('treeList')->toArray();
            $this->set('categories', $categories);
        }

        $this->set('categorie', $categorie);
    }


    /**
     * Modification d'un document
     * @param string $type
     * @param string $id
     * @throws Exception
     */
    public function edit(string $type = 'table', string $id)
    {
        $this->loadModel('Categories');
        $categorie = $this->Categories->get($id);

        if ($this->getRequest()->is('put')) {
            $this->Categories->patchEntity($categorie, $this->getRequest()->getData());
            $this->loadComponent('AsalaeCore.Modal');
            if ($this->Categories->save($categorie)) {
                $this->Modal->success();
            } else {
                $this->Modal->fail();
            }
        }

        if ($type !== 'table') {
            $categories = $this->Categories->find('treeList')->toArray();
            $this->set('categories', $categories);
        }

        $this->set('categorie', $categorie);
    }

    /**
     * Permet la suppression d'une catégorie si elle n'a pas d'enfants et que
     * aucuns documents actif n'est lié
     * @param string $id
     * @return \Cake\Http\Response|null
     * @throws Exception
     */
    public function delete(string $id)
    {
        $this->getRequest()->allowMethod('delete');
        $this->loadModel('Categories');
        $categorie = $this->Categories->find()
            ->where(['Categories.id' => $id])
            ->contain(['Documents'])
            ->firstOrFail()
        ;
        if ($categorie->get('rght') !== $categorie->get('lft') +1) {
            throw new Exception(__("Cette catégorie ne peut pas être supprimée car elle possède des catégories enfant."));
        }
        if ($documents = $categorie->get('documents')) {
            /** @var Document $document */
            foreach ($documents as $document) {
                if ($document->get('status') === 'actif') {
                    throw new Exception(__("Cette catégorie ne peut pas être supprimée car elle est liée à une ou plusieurs règles de gestion active."));
                }
            }
        }
        $this->loadModel('CategoriesDocuments');
        $this->CategoriesDocuments->deleteAll(['category_id' => $id]);
        $this->Categories->deleteOrFail($categorie);
        $this->loadComponent('AsalaeCore.Modal');
        $this->Modal->success();
        return $this->renderJson('"done"');
    }
}
