<?php
/**
 * Refae\Controller\FilesController
 */

namespace Refae\Controller;

use AsalaeCore\Controller\RenderDataTrait;
use Cake\Http\Response;

/**
 * Fichiers liés au documents
 *
 * @category Controller
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017-2018, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 * @property \Refae\Model\Table\FilesTable $Files
 */
class FilesController extends AppController
{
    /**
     * Traits
     */
    use RenderDataTrait;

    /**
     * Permet de supprimer un schema_profil_archivage uploadé
     *      Suppression dans le dossier temporaire si disponnible
     *      Suppression en base
     * @param string $id valeur 'add' dans le cas d'un ajout
     * @param string $filename
     * @return \Cake\Http\Response|null
     */
    public function delete(string $id, string $filename)
    {
        $user_id = $this->Auth->user('id');
        $path = TMP.'upload'.DS.'user_'.$user_id;
        if (is_file($path . DS . $filename)) {
            Filesystem::remove($path . DS . $filename);
            return $this->renderJson('"removed from tmp"');
        }
        if ($id === 'add') {
            return $this->renderJson('"not removed"');
        }
        $this->loadModel('Documents');
        $this->loadModel('Files');
        $document = $this->Documents->find()
            ->where(['Documents.id' => $id])
            ->contain(['Files'])
            ->firstOrFail()
        ;
        $file = $document->get('file');
        if ($file && $file->get('filename') === $filename && $this->Files->delete($file)) {
            return $this->renderJson('"removed"');
        }
        return $this->renderJson('"not removed"');
    }

    /**
     * Permet de télécharger le fichier stocké en base
     * @param string $id
     * @return Response
     */
    public function download(string $id)
    {
        $file = $this->loadModel('Files')->get($id);
        $response = $this->getResponse()->withDownload($file->get('filename'));
        $body = $response->getBody();
        $handle = $file->get('content');
        while (!feof($handle)) {
            $body->write(fread($handle, 8192));
        }
        return $response
            ->withBody($body)
            ->withType('xml')
        ;
    }
}
