<?php
/**
 * Asalae\Controller\UploadController
 */

namespace Refae\Controller;

use Exception;
use Flow\Basic;
use Flow\Config;
use Flow\Request;
use Libriciel\Filesystem\Utility\Filesystem;

/**
 * Upload de fichiers
 *
 * @category Controller
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */
class UploadController extends AppController
{
    /**
     * Upload du fichier
     * (appelé par ajax via <b>new Flow({target: '/upload'})</b>)
     *
     * @throws Exception
     */
    public function index()
    {
        $user_id = $this->Auth->user('id');

        $path = TMP.'upload'.DS.'user_'.$user_id;

        if (!file_exists($path)) {
            Filesystem::mkdir($path);
        }

        $config = new Config();
        $config->setTempDir($path);
        $request = new Request();

        $id = $request->getIdentifier();
        $finalPath = $path.DS.$id;

        $report = array(
            'id' => $id,
            'uid' => $id,
            'filename' => $request->getFileName(),
            'path' => $finalPath,
            'completion' => false
        );

        if (empty($report['uid'])) {
            throw new Exception(__("Une erreur inattendue a eu lieu"));
        }

        $report['completed'] = Basic::save($finalPath, $config, $request);
        $report['path'] = urlencode(substr($report['path'], strlen(TMP)));
        if ($report['completed']) {
            $report['completion'] = date("Y-m-d H:i:s");
        }

        $this->viewBuilder()->setClassName('Json');
        $this->set(compact('report'));
        $this->set('_serialize', ['report']);
    }
}
