<?php
/**
 * Refae\Controller\UsersController
 */
namespace Refae\Controller;

use AsalaeCore\Controller\RenderDataTrait;
use Cake\Event\Event;
use Cake\Utility\Security;
use Exception;
use Libriciel\Filesystem\Utility\Filesystem;
use Refae\Model\Table\UsersTable;

/**
 * Controller de la table users
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017-2018, Libriciel SCOP
 * @license   https://opensource.org/licenses/AGPL-3.0
 * @property UsersTable $Users
 */
class UsersController extends AppController
{
    /**
     * Traits
     */
    use RenderDataTrait;

    /**
     * Authorize les méthodes login et logout
     *
     * @param Event $event
     * @return null
     */
    public function beforeFilter(Event $event)
    {
        $this->Auth->allow(['login', 'logout']);
        return parent::beforeFilter($event);
    }

    /**
     * Identification de l'utilisateur
     */
    public function login()
    {
        $this->getRequest()->allowMethod('ajax');
        $this->viewBuilder()->setLayout('ajax');
        $this->loadModel('Users');
        $user = $this->Users->newEntity();
        if ($this->getRequest()->is('post')) {
            $connect = $this->Users->find()
                ->where([
                    'username' => $this->getRequest()->getData('login'),
                    'password' => Security::hash(
                        $this->getRequest()->getData('password'),
                        'sha256',
                        true
                    )
                ])
                ->first();
            $this->loadComponent('AsalaeCore.Modal');
            if ($connect) {
                $connect = $connect->toArray();
                $this->Auth->setUser($connect);
                $this->getRequest()->getSession()->write('User', $connect);
                $path = TMP.'upload'.DS.'user_'.$connect['id'];
                if (file_exists($path)) {
                    Filesystem::remove($path);
                }
                $this->Modal->success();
            } else {
                $this->Modal->fail();
                $user->setErrors(['login' => __('Login ou mot de passe invalide')]);
            }
        }
        $this->set('user', $user);
    }

    /**
     * Déconnexion
     */
    public function logout()
    {
        $user_id = $this->Auth->user('id');
        $path = TMP.'upload'.DS.'user_'.$user_id;
        if (file_exists($path)) {
            try {
                Filesystem::remove($path);
            } catch (Exception $e) {
                $this->log($e->getMessage());
            }
        }
        $this->getRequest()->getSession()->destroy();
        return $this->renderJson('"session destoyed"');
    }

    /**
     * Liste des utilisateurs
     */
    public function index()
    {
        $this->loadModel('Users');
        $this->set('users', $this->Users->find()->order(['name' => 'asc'])->all()->toArray());
    }

    /**
     * Formulaire d'ajout d'un utilisateur
     */
    public function add()
    {
        $this->loadModel('Users');
        $user = $this->Users->newEntity();

        if ($this->getRequest()->is('post')) {
            $this->Users->patchEntity($user, $this->getRequest()->getData());
            $this->loadComponent('AsalaeCore.Modal');
            if ($this->Users->save($user)) {
                $this->Modal->success();
            } else {
                $this->Modal->fail();
            }
        }

        $this->set('user', $user);
    }

    /**
     * Formulaire d'ajout d'un utilisateur
     * @param string $id
     * @throws Exception
     */
    public function edit(string $id)
    {
        $this->loadModel('Users');
        $user = $this->Users->get($id);

        if ($this->getRequest()->is('put')) {
            $data = $this->getRequest()->getData();
            if (empty($data['password'])) {
                unset($data['password'], $data['password-confirm']);
            }
            $this->Users->patchEntity($user, $data);
            $this->loadComponent('AsalaeCore.Modal');
            if ($this->Users->save($user)) {
                $this->Modal->success();
            } else {
                $this->Modal->fail();
            }
        }

        $this->set('user', $user);
    }

    /**
     * Action de suppression d'un utilisateur
     * @param string $id
     * @return \Cake\Http\Response
     * @throws Exception
     */
    public function delete(string $id)
    {
        $this->getRequest()->allowMethod('delete');
        $this->loadModel('Users');
        $user = $this->Users->get($id);
        $this->loadComponent('AsalaeCore.Modal');
        if ($this->Users->delete($user)) {
            $this->Modal->success();
        } else {
            $this->Modal->fail();
        }
        return $this->renderData($user->toArray());
    }
}
