<?php
/**
 * Refae\Model\Entity\Document
 */

namespace Refae\Model\Entity;

use Cake\ORM\Entity;
use Refae\Model\Table\DocumentsTable;

/**
 * Entité de la table documents
 *
 * @category Entity
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017-2018, Libriciel SCOP
 * @license   https://opensource.org/licenses/AGPL-3.0
 */
class Document extends Entity
{
    /**
     * Champs virtuels
     *
     * @var array
     */
    protected $_virtual = [
        'statustrad',
        'sort_finaltrad',
        'communicabilitetrad',
        'communicabilite_an',
    ];

    /**
     * Traductions des status
     *
     * @return string
     * @throws \Aura\Intl\Exception
     */
    protected function _getStatustrad(): string
    {
        if (empty($this->_properties['status'])) {
            return '';
        }
        switch ($this->_properties['status']) {
            case 'actif':
                return __dx('document', 'status', 'Publiée');
            case 'brouillon':
                return __dx('document', 'status', 'Brouillon');
            case 'obsolete':
                return __dx('document', 'status', 'Obsolète');
        }
        return $this->_properties['status'];
    }

    /**
     * Traductions des sort_final
     *
     * @return string
     * @throws \Aura\Intl\Exception
     */
    protected function _getSortFinaltrad(): string
    {
        if (empty($this->_properties['sort_final'])) {
            return '';
        }
        switch ($this->_properties['sort_final']) {
            case 'D':
                return __dx('document', 'sort_final', 'Détruire');
            case 'C':
                return __dx('document', 'sort_final', 'Conserver');
            case 'T':
                return __dx('document', 'sort_final', 'Trier');
            case 'TS':
                return __dx('document', 'sort_final', 'Tri sélectif');
        }
        return $this->_properties['sort_final'];
    }

    /**
     * Traductions des support
     *
     * @return string
     * @throws \Aura\Intl\Exception
     */
    protected function _getSupporttrad(): string
    {
        if (empty($this->_properties['support'])) {
            return '';
        }
        switch ($this->_properties['support']) {
            case 'P':
                return __dx('document', 'P', 'Papier');
            case 'E':
                return __dx('document', 'E', 'Électronique');
            case 'P et E':
                return __dx('document', 'P et E', 'Papier et Électronique');
        }
        return $this->_properties['support'];
    }

    /**
     * Traductions des support
     *
     * @return string
     * @throws \Exception
     */
    protected function _getCommunicabilitetrad(): string
    {
        if (empty($this->_properties['communicabilite'])) {
            return '';
        }
        $options = DocumentsTable::getCommunicabiliteOptions();
        return $options[$this->_properties['communicabilite']]
            ?? $this->_properties['communicabilite']
        ;
    }

    /**
     * Permet d'obtenir la partie "0 an" de "AR038: 0 an"
     * @return string
     * @throws \Exception
     */
    protected function _getCommunicabiliteAn(): string
    {
        if (empty($this->_properties['communicabilite'])
            || !($trad = $this->_getCommunicabilitetrad())
            || strpos($trad, ':') === false
        ) {
            return '';
        }
        list(, $trad) = explode(':', $this->_getCommunicabilitetrad());
        return trim($trad);
    }
}
