<?php
/**
 * Refae\Model\Entity\File
 */

namespace Refae\Model\Entity;

use Cake\ORM\Entity;
use JsonSerializable;

/**
 * Entité de la table files
 *
 * @category Entity
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017-2018, Libriciel SCOP
 * @license   https://opensource.org/licenses/AGPL-3.0
 */
class File extends Entity implements JsonSerializable
{
    /**
     * Encode la ressource en base64 pour permettre la conversion en json
     * @return array|mixed
     */
    public function jsonSerialize()
    {
        $data = $this->toArray();
        if (isset($data['content']) && gettype($data['content']) === 'resource') {
            $data['content'] = $this->getBase64($data['content']);
        }
        return $data;
    }

    /**
     * Converti les ressources en base64
     * @return array
     */
    public function toArray()
    {
        $data = parent::toArray();
        foreach ($data as $key => $value) {
            if (gettype($value) === 'resource') {
                $data[$key] = $this->getBase64($value);
            }
        }
        return $data;
    }

    /**
     * @var string content en base64
     */
    private $base64Content = '';

    /**
     * Converti une ressource en base64
     * @param resource $value
     * @return string
     */
    private function getBase64($value): string
    {
        if (feof($value)) {
            return $this->base64Content;
        }
        $contents = '';
        while (!feof($value)) {
            $contents .= fread($value, 8192);
        }
        $this->base64Content = base64_encode($contents);
        return $this->base64Content;
    }

    /**
     * Getter du content en base64
     * @return string
     */
    protected function _getContent64(): string
    {
        return $this->getBase64($this->_properties['content']);
    }

    /**
     * Permet d'insérer le content directement en base64
     * @param $value
     * @return bool|string
     */
    protected function _setContent($value)
    {
        if (is_string($value)
            && preg_match('/^[a-zA-Z0-9\/\r\n+]*={0,2}$/', $value)
            && $decoded = base64_decode($value, true)
        ) {
            return $decoded;
        }
        return $value;
    }
}
