<?php
/**
 * Refae\Model\Table\CategoriesTable
 */

namespace Refae\Model\Table;

use Cake\ORM\Table;
use Cake\Validation\Validator;
use Exception;

/**
 * Table categories
 *
 * @category Table
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017-2018, Libriciel SCOP
 * @license   https://opensource.org/licenses/AGPL-3.0
 */
class CategoriesTable extends Table
{
    /**
     * Configuration initale de la table
     *
     * @param array $config
     * @throws Exception
     */
    public function initialize(array $config)
    {
        $this->addBehavior('Timestamp');
        $this->addBehavior('Tree');
        $this->addBehavior('AsalaeCore.Options', [
            'type' => $this->enum('type')
        ]);
        $this->belongsToMany('Documents');

        parent::initialize($config);
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     * @throws Exception
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator
            ->notEmpty('name')
            ->notEmpty('type')
            ->allowEmpty('parent_id')
            ->inList('type', $this->enum('type'))
        ;

        $validator->add('name', ['unique' => [
            'rule' => ['validateUnique'],
            'provider' => 'table',
            'message' => __("Ce nom est déjà utilisé")]
        ]);
        $validator->add('parent_id', 'custom', [
            'rule' => function ($value, $context) {
                if ($context['newRecord'] || empty($context['data']['id']) || !$value) {
                    return true;
                }
                /** @var CategoriesTable $table */
                $table = $context['providers']['table'];
                try {
                    if (empty($context['data']['lft']) || empty($context['data']['rght'])) {
                        $entity = $table->find()
                            ->select(['lft', 'rght'])
                            ->where(['id' => $context['data']['id']])
                            ->firstOrFail()
                        ;
                        $context['data']['lft'] = $entity->get('lft');
                        $context['data']['rght'] = $entity->get('rght');
                    }
                    $parent = $table->get($value);
                    return $parent->get('lft') < $context['data']['lft']
                        || $parent->get('rght') > $context['data']['rght']
                        ;
                } catch (\Exception $e) {
                    return false;
                }
            },
            'message' => __("Impossible d'apparenter ce dossier à un de ses enfants")
        ]);

        return $validator;
    }

    /**
     * Renvoi la liste des choix possible pour un champ
     * @param string $field
     * @return array
     * @throws Exception
     */
    public function enum(string $field)
    {
        switch ($field) {
            case 'type':
                return ['table', 'typo', 'dossier'];
            default:
                throw new Exception(__("{0} n'est pas un champ de type enum", $field));
        }
    }
}
