<?php
/**
 * Refae\Model\Table\UsersTable
 */

namespace Refae\Model\Table;

use Cake\Core\Configure;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * Table users
 *
 * @category Table
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017-2018, Libriciel SCOP
 * @license   https://opensource.org/licenses/AGPL-3.0
 */
class UsersTable extends Table
{
    /**
     * Configuration initale de la table
     *
     * @param array $config
     */
    public function initialize(array $config)
    {
        $this->addBehavior('Timestamp');
        $this->hasMany('Files');

        parent::initialize($config);
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     * @throws \Aura\Intl\Exception
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator
            ->notEmpty('name')
            ->notEmpty('username')
            ->notEmpty('mail')
            ->email('mail',
                false,
                __('Veuillez saisir un email valide')
            )
        ;

        $checkPassword = function ($value) {
            $chars = 0;
            if (preg_match('/[a-z]/', $value)) {
                $chars += 26;
            }
            if (preg_match('/[A-Z]/', $value)) {
                $chars += 26;
            }
            if (preg_match('/[0-9]/', $value)) {
                $chars += 10;
            }
            if (preg_match('/[ ! #$*%]/', $value)) {
                $chars += 8;
            }
            if (preg_match('/[^a-zA-Z0-9 ! #$*%]/', $value)) {
                $chars += 20;
            }
            $score = strlen($value) * log($chars, 2);
            return $score >= Configure::read('Password.complexity');
        };

        $validator
            ->add('password', 'custom', [
                'rule' => $checkPassword,
                'message' => __("Le password n'est pas assez complexe")
            ]);
        $validator
            ->sameAs('confirm-password', 'password');
        $validator->allowEmpty('confirm-password');

        $validator->add('username', ['unique' => [
            'rule' => 'validateUnique',
            'provider' => 'table',
            'message' => __("Ce nom d'utilisateur est déjà utilisé")]
        ]);

        return $validator;
    }
}
