<?php
/**
 * Refae\Shell\FileTrait
 */

namespace Refae\Shell;

use Cake\Console\ConsoleIo;
use Cake\Console\Shell;
use Cake\Filesystem\File;

/**
 * Ensemble de méthodes pour la manipulation de mot de passe
 *
 * @category    Shell
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */
trait FileTrait
{

    /**
     * Creates a file at given path
     *
     * @see Shell::createFile()
     * @param string $path Where to put the file.
     * @param string $contents Content to put in the file.
     * @param string $default
     * @return bool Success
     * @throws \Exception
     */
    public function createFileWithDefault($path, $contents, $default = 'n')
    {
        $path = str_replace(DIRECTORY_SEPARATOR . DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR, $path);

        if (!$this->_io instanceof ConsoleIo) {
            throw new \Exception(__("_io must be an instance of ConsoleIo"));
        }
        $this->_io->out();

        if (is_file($path) && empty($this->params['force']) && $this->interactive) {
            $this->_io->out(sprintf('<warning>File `%s` exists</warning>', $path));
            $key = $this->_io->askChoice(__('Do you want to overwrite?'), ['y', 'n', 'a', 'q'], $default);

            if (strtolower($key) === 'q') {
                $this->_io->out(__('<error>Quitting</error>.'), 2);
                $this->_stop();

                return false;
            }
            if (strtolower($key) === 'a') {
                $this->params['force'] = true;
                $key = 'y';
            }
            if (strtolower($key) !== 'y') {
                $this->_io->out(__('Skip `{0}`', $path), 2);

                return false;
            }
        } else {
            $this->out(__('Creating file {0}', $path));
        }

        $File = new File($path, true);

        try {
            if ($File->exists() && $File->writable()) {
                $File->write($contents);
                $this->_io->out(__('<success>Wrote</success> `{0}`', $path));

                return true;
            }

            $this->_io->err(__('<error>Could not write to `{0}`</error>.', $path), 2);

            return false;
        } finally {
            $File->close();
        }
    }
}
