<?php
/**
 * @var \Refae\View\AppView $this
 */
$data = array_map(function($v) {
    $dua = is_numeric($v['dua']) ? (int)$v['dua'] : 0;
    $duc = is_numeric($v['duc']) ? (int)$v['duc'] : 0;
    $v['dua'] = __n('{0} an', '{0} ans', $dua, $dua);
    $v['duc'] = __n('{0} an', '{0} ans', $duc, $duc);
    return $v;
}, $data);

$table = $this->Table
    ->create($tableId, ['class' => 'table table-striped table-hover minifiable-target smart-td-size'])
    ->fields([
        'Documents.identifiant' => ['label' => __('Identifiant'), 'order' => 'identifiant'],
        'Documents.name' => ['label' => __('Nom'), 'order' => 'name'],
        'Documents.categories' => ['label' => __('Associée à'), 'class' => 'puces', 'thead' => ['name' => ['label' => '']]],
        'Documents.support' => ['label' => __('Support'), 'display' => false],
        'Documents.file_format' => ['label' => __('Format de fichier'), 'display' => false],
        'Documents.localisation' => ['label' => __('Localisation du document'), 'display' => false],
        'Documents.dua' => ['label' => __("DUA"), 'title' => __("Durée d'utilité administrative"), 'order' => 'dua'],
        'Documents.duc' => ['label' => __("DUC"), 'title' => __("Durée d'utilité courante"), 'order' => 'duc'],
        'Documents.communicabilitetrad' => ['label' => __('Code de communicabilite'), 'order' => 'communicabilite', 'display' => false],
        'Documents.communicabilite_an' => ['label' => __('Communicabilite')],
        'Documents.sort_finaltrad' => ['label' => __('Sort final'), 'order' => 'sort_final'],
        'Documents.description' => ['label' => __('Description'), 'display' => false],
        'Documents.created' => ['label' => __('Date de création'), 'type' => 'datetime', 'display' => false, 'order' => 'created'],
        'Documents.modified' => ['label' => __('Date de modification'), 'type' => 'datetime', 'display' => false, 'order' => 'created'],
        'Documents.public_date' => ['label' => __('Date de publication'), 'type' => 'datetime', 'display' => false, 'order' => 'public_date'],
        'Documents.obsolete_date' => ['label' => __('Date de fin de publication'), 'type' => 'datetime', 'display' => false, 'order' => 'obsolete_date'],
        'Documents.version' => ['label' => __('Version'), 'display' => false, 'order' => 'version'],
        'Documents.note_version' => ['label' => __('Note sur la version'), 'display' => false],
        'Documents.statustrad' => ['label' => __('Statut'), 'display' => false, 'order' => 'status'],
        'Documents.depart_calcul_dua' => ['label' => __('Date de départ de calcul de la DUA'), 'display' => false],
        'Documents.depart_calcul_duc' => ['label' => __('Date de départ de calcul de la DUC'), 'display' => false],
        'Documents.depart_calcul_com' => ['label' => __('Date de départ de calcul de la communicabilité'), 'display' => false],
        'Documents.profil_archivage' => ['label' => __("Nom du profil d'archivage"), 'display' => false],
    ])
    ->data($data)
    ->params([
        'identifier' => 'Documents.id',
        'favorites' => $this->Paginator->sort('favorite', '&#8203;', ['escape' => false]),
        'checkbox' => false,
    ])
;

$actions = [
    [
        'onclick' => "RefaeHeader.loadViewModal({0})",
        'type' => "button",
        'class' => 'btn-link',
        'label' => $this->Fa->charte('Visualiser'),
        'title' => __('Visualiser {0}', '{1}'),
        'params' => ['Documents.id', 'Documents.name']
    ]
];
if ($user_id) {
    $actions = array_merge($actions, [
        [
            'data-callback' => "loadNewVersionModal({0})",
            'type' => "button",
            'class' => 'btn-link',
            'label' => $this->Fa->i('fa-code-fork'),
            'title' => __('Créer une nouvelle version de {0}', '{1}'),
            'displayEval' => 'data[{index}].Documents.status === "actif"',
            'confirm' => __("Cette action va créer une nouvelle version pour cette règle de gestion, voulez-vous continuer ?"),
            'params' => ['Documents.id', 'Documents.name']
        ],
        [
            'onclick' => "loadEditDocumentModal({0})",
            'type' => "button",
            'class' => 'btn-link',
            'label' => $this->Fa->charte('Modifier'),
            'title' => __('Modifier {0}', '{1}'),
            'displayEval' => 'data[{index}].Documents.status === "brouillon"',
            'params' => ['Documents.id', 'Documents.name']
        ],
        [
            'data-callback' => "publishAction({0})",
            'type' => "button",
            'class' => 'btn-link',
            'label' => $this->Fa->i('fa-globe text-success'),
            'title' => __('Publier {0}', '{1}'),
            'displayEval' => 'data[{index}].Documents.status === "brouillon"',
            'confirm' => __("Êtes-vous certain de vouloir publier cette règle de gestion ?"),
            'params' => ['Documents.id', 'Documents.name']
        ],
        [
            'data-callback' => "obsoleteAction({0})",
            'type' => "button",
            'class' => 'btn-link',
            'label' => $this->Fa->i('fa-calendar-times-o text-danger'),
            'title' => __('Marquer {0} comme obsolète', '{1}'),
            'displayEval' => 'data[{index}].Documents.status === "actif"',
            'confirm' => __("Êtes-vous certain de vouloir marquer cette règle de gestion comme obsolète ?"),
            'params' => ['Documents.id', 'Documents.name']
        ],
        [
            'data-callback' => "deleteAction({0})",
            'type' => "button",
            'class' => 'btn-link',
            'label' => $this->Fa->charte('Supprimer', 'text-danger'),
            'title' => __('Supprimer {0}', '{1}'),
            'displayEval' => 'data[{index}].Documents.status === "brouillon"',
            'confirm' => __("Êtes-vous certain de vouloir supprimer cette règle de gestion ?"),
            'params' => ['Documents.id', 'Documents.name']
        ],
    ]);
}
$table->actions($actions);

// @see src/Template/Documents/index.ctp
$jsTable = $table->tableObject;

echo $this->element('section_table', [
    'id' => 'documents-section',
    'title' => __('Résultats'),
    'table' => $table
]);