<?php
/**
 * @var \Refae\View\AppView $this
 */

use Cake\Core\Configure;
use Cake\View\AjaxView;
use Refae\View\AppView;

echo $this->element('modal', ['idTable' => $tableId]);

$breadcrumbs = '';
if ($table_id) {
    $this->Breadcrumbs->add(__('Accueil'), '/');
    $tableUrl = '/?table_id='.$table_id;
    if ($subcat_id) {
        foreach ($parents as $id => $name) {
            $url = $id === $table_id ? $tableUrl : $tableUrl.'&subcat='.$id;
            $this->Breadcrumbs->add($name, $url);
        }
    }
    $this->Breadcrumbs->add($title);
    $breadcrumbs = $this->Breadcrumbs->render();
}

if ($user_id) {
    echo $this->ModalForm->create('edit-document', ['size' => 'large'])
        ->modal(__("Modifier la règle de gestion"))
        ->javascriptCallback("afterEditDocument")
        ->output('function',
            'loadEditDocumentModal',
            '/Documents/edit'
        )->generate();
    echo $this->ModalForm->create('edit-tableau')
        ->modal(__("Modification d'une entité gestionnaire"))
        ->javascriptCallback("RefaeHeader.refreshPage")
        ->output('function',
            'loadEditCategorieModal',
            '/Categories/edit/table'
        )->generate();
    echo $this->ModalForm->create('edit-dossier')
        ->modal(__("Modification d'un dossier"))
        ->javascriptCallback("RefaeHeader.refreshPage")
        ->output('function',
            'loadEditCategorieDossierModal',
            '/Categories/edit/dossier'
        )->generate();
    echo $this->ModalForm->create('add-version')
        ->modal(__("Nouvelle version d'une règle de gestion"))
        ->javascriptCallback("afterAddVersion")
        ->output('function',
            'loadNewVersionModal',
            '/Documents/new-version'
        )->generate();
}
$wildcard = __("? = n'importe quel caractère ; * = n'importe quelle chaîne de caractères");
$filters = $this->Filter->create('documents-filter')
    ->saves($savedFilters)
    ->filter('created', [
        'label' => __('Date de création'),
        'placeholder' => 'jj/mm/aaaa',
        'type' => 'date',
    ])

    ->filter('categories', [
        'label' => __('Associée à'),
        'options' => $categories,
        'multiple' => true
    ])
    ->filter('favoris', [
        'label' => __('Favoris seulement'),
        'type' => 'checkbox',
        'onclick' => 'return false',
        'style' => 'cursor: not-allowed',
        'checked'
    ])
    ->filter('identifiant', [
        'label' => __('Identifiant'),
        'placeholder' => 'ident?fi*',
        'help' => $wildcard,
        'type' => 'text'
    ])
    ->filter('name', [
        'label' => __('Nom'),
        'placeholder' => 'N?*',
        'help' => $wildcard,
        'type' => 'text'
    ])
;
if ($user_id ?? false) {
    $filters->filter('status', [
        'label' => __('Statut'),
        'options' => $status,
        'multiple' => true
    ]);
    $filters->filter('last-version', [
        'label' => __('Dernière version uniquement'),
        'type' => 'checkbox',
        'onclick' => 'return false',
        'style' => 'cursor: not-allowed',
        'checked'
    ]);
}
echo $filters->generate();

/**
 * @param AppView|AjaxView $View
 * @param int $table_id
 * @param array $sub
 * @param int $user_id
 */
function recursiveSubcats($View, $table_id, $sub, $user_id)
{
    echo $View->Html->tag('ul');
    $actual_id = (int)$View->request->getParam('?.subcat');
    if ($actual_id === 0) {
        $actual_id = (int)$View->request->getParam('?.searchcat');
    }
    foreach ($sub as $data) {
        $name = $View->Fa->i('fa-folder-open-o').' '.h($data['name']);
        $id = $data['id'];
        if ($id !== $actual_id) {
            $link = $View->Html->link($name, '/?table_id='.$table_id.'&subcat='.$id, ['escape' => false]);
            $params = ['escape' => false];
        } else {
            $link = $View->Html->tag('span', $name, ['class' => 'active', 'escape' => false]);
            $params = ['class' => 'active', 'escape' => false];
        }

        if ($data['children']) {
            echo "\t".$View->Html->tag('li', null, $params)
                .$link.PHP_EOL;
            recursiveSubcats($View, $table_id, $data['children'], $user_id);
            echo "\t".$View->Html->tag('/li');
        } else {
            echo "\t".$View->Html->tag('li', $link, $params).PHP_EOL;
        }
    }
    echo $View->Html->tag('/ul');
}
?>
<nav class="navbar sidebar left-panel">
    <div class="h4"><i class="fa fa-list"></i> Entités gestionnaires</div>
    <ul>
        <?php foreach ($tableauxDeGestion as $id => $name) {
            $name = $this->Fa->i('fa-table').' '.h($name);
            if ($id === $table_id) {
                if ($subcat_id) {
                    $link = $this->Html->link($name, '/?table_id='.$id, ['escape' => false]);
                } else {
                    $link = $this->Html->tag('span', $name, ['class' => 'active']);
                }
                $params = ['class' => 'active', 'escape' => false];
                echo "\t".$this->Html->tag('li', null, $params).$link;
                if (!empty($subcats)) {
                    recursiveSubcats($this, $table_id, $subcats, $user_id);
                }
                echo $this->Html->tag('/li').PHP_EOL;
            } else {
                $link = $this->Html->link($name, '/?table_id='.$id, ['escape' => false]);
                $params = ['escape' => false];
                echo "\t".$this->Html->tag('li', $link, $params).PHP_EOL;
            }
        } ?>
    </ul>
</nav>
<div>
<?php
if ($subcat_id) {
    echo $this->Html->tag('h1', $this->Fa->i('fa-table').' '.h($tableauxDeGestion[$table_id]))
        .$this->Html->tag('h2', $this->Fa->i('fa-folder-open').' '.$title)
        .$breadcrumbs;
    ;
} elseif ($table_id) {
    echo $this->Html->tag('h1', $this->Fa->i('fa-table').' '.$title)
        .$breadcrumbs
    ;
} else {
    echo $this->Html->tag('h1', $this->Fa->i('fa-gavel').' '.$title);
}
if (!empty($catDesc)) {
    echo $this->Html->tag('p', $catDesc);
}
if ($user_id && ($table_id || $subcat_id)) {
    $onclick = $subcat_id
        ? "loadEditCategorieDossierModal($subcat_id)"
        : "loadEditCategorieModal($table_id)"
    ;
    $sujet = $subcat_id ? __("le dossier") : __("l'entité gestionnaire");
    $buttons = $this->Html->tag('button',
            '<i class="fa fa-plus-circle" aria-hidden="true"></i> '.__("Ajouter une règle de gestion"),
            ['class' => 'btn btn-success', 'type' => 'button', 'onclick' => 'loadAddDocumentModal('.($subcat_id ?: $table_id).')']
        )
        .$this->Html->tag('button',
            '<i class="fa fa-plus-circle" aria-hidden="true"></i> '.__("Ajouter un dossier"),
            ['class' => 'btn btn-success', 'type' => 'button', 'onclick' => 'loadAddCategorieDossierModal('.($subcat_id ?: $table_id).')']
        )
        .$this->Html->tag('button',
            $this->Fa->charte('Modifier').' '.__("Modifier {0}", $sujet),
        ['class' => 'btn btn-warning', 'type' => 'button', 'onclick' => $onclick]
    );
    if ($categoryDeletable) {
        $buttons .= $this->Html->tag('button',
            $this->Fa->charte('Supprimer').' '.__("Supprimer {0}", $sujet),
            ['class' => 'btn btn-danger', 'type' => 'button', 'onclick' => 'deleteCategory('.($subcat_id ?: $table_id).')']
        );
    }

    echo $this->Html->tag('div', $buttons, ['class' => 'actions container']);
}
if ($msg = Configure::read('Messages.accueil')) {
    echo $this->Html->tag('div', $msg, ['class' => 'configured-message']);
}
include 'ajax_index.ctp';
?>
</div>
<script>
function actionDelete(id) {
    var table;
    table = <?=$jsTable?>;
    if (confirm("Cette action supprimera de façon définitive la règle de gestion et sera consigné dans le registre. Voulez-vous continuer ?")) {
        $('html').addClass('ajax-loading');
        $('.main-content').fadeOut();
        $('#loading-screen').fadeIn(100);
        // Résout un bug sur ie
        setTimeout(function() {
            $('#loading-screen').removeClass('paused');
        }, 10);
        $.ajax({
            url: "<?=$this->Url->build('/api/documents')?>/"+id,
            method: 'DELETE',
            complete: function() {
                AsalaeGlobal.interceptedLinkToAjax(''+location);
            }
        });
    }
}

function deleteCategorieAction(id, name) {
    if (confirm("Cette action supprimera de façon définitive '"+name+"', et sera consigné dans le registre. Voulez-vous continuer ?")) {
        $('html').addClass('ajax-loading');
        $('.main-content').fadeOut();
        $('#loading-screen').fadeIn(100);
        // Résout un bug sur ie
        setTimeout(function() {
            $('#loading-screen').removeClass('paused');
        }, 10);
        $.ajax({
            url: "<?=$this->Url->build('/api/categories')?>/"+id,
            method: 'DELETE',
            complete: function() {
                window.location.href = "<?=$this->Url->build('/')?>";
            }
        });
    }
}

function publishAction(id) {
    var line = $('#documents-table').find('tr[data-id="'+id+'"]');
    $('html').addClass('ajax-loading');
    $('.main-content').fadeOut();
    $('#loading-screen').fadeIn(100);
    // Résout un bug sur ie
    setTimeout(function() {
        $('#loading-screen').removeClass('paused');
    }, 10);
    line.addClass('warning');
    $.ajax({
        url: "<?=$this->Url->build('/Documents/publish')?>/"+id,
        method: 'UPDATE',
        data: {status: 'actif'},
        success: function(content, textStatus, jqXHR) {
            if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
                line.addClass('success').removeClass('warning');
                searchData = content;
                AsalaeGlobal.interceptedLinkToAjax(''+location);
            } else {
                this.error();
            }
        },
        error: function() {
            line.addClass('danger').removeClass('warning');
            alert(PHP.messages.genericError);
        },
        complete: function() {
            $('html').removeClass('ajax-loading');
            $('#loading-screen').fadeOut(100, function() { $(this).addClass('paused'); });
            $('.main-content').fadeIn();
        }
    });
}
function obsoleteAction(id) {
    var line = $('#documents-table').find('tr[data-id="'+id+'"]');
    $('html').addClass('ajax-loading');
    $('.main-content').fadeOut();
    $('#loading-screen').fadeIn(100);
    // Résout un bug sur ie
    setTimeout(function() {
        $('#loading-screen').removeClass('paused');
    }, 10);
    line.addClass('warning');
    $.ajax({
        url: "<?=$this->Url->build('/Documents/obsolete')?>/"+id,
        method: 'UPDATE',
        data: {status: 'obsolete'},
        success: function(content, textStatus, jqXHR) {
            if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
                line.addClass('success').removeClass('warning');
                searchData = content;
                AsalaeGlobal.interceptedLinkToAjax(''+location);
            } else {
                this.error();
            }
        },
        error: function() {
            line.addClass('danger').removeClass('warning');
            alert(PHP.messages.genericError);
        },
        complete: function() {
            $('html').removeClass('ajax-loading');
            $('#loading-screen').fadeOut(100, function() { $(this).addClass('paused'); });
            $('.main-content').fadeIn();
        }
    });
}

function afterAddVersion(content, textStatus, jqXHR) {
    if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
        AsalaeGlobal.interceptedLinkToAjax('<?=$this->Url->build('/Documents/index/brouillon')?>?edit='+content.id);
    }
}

function afterEditDocument(content, textStatus, jqXHR) {
    if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
        AsalaeGlobal.interceptedLinkToAjax('<?=$this->Url->build('/Documents/index/brouillon')?>');
    }
}

function deleteCategory(id) {
    if (!confirm("<?=__("La suppression peut entraîner une perte d'information (retirera le 'associé à' des règles). Êtes-vous certain de le vouloir ?")?>")) {
        return;
    }
    $('html').addClass('ajax-loading');
    $('.main-content').fadeOut();
    $('#loading-screen').fadeIn(100);
    // Résout un bug sur ie
    setTimeout(function() {
        $('#loading-screen').removeClass('paused');
    }, 10);
    $.ajax({
        url: "<?=$this->Url->build('/Categories/delete')?>/"+id,
        method: 'DELETE',
        success: function(content, textStatus, jqXHR) {
            if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
                AsalaeGlobal.interceptedLinkToAjax('<?=$this->Url->build('/')?>');
            } else {
                this.error();
            }
        },
        error: function() {
            alert(PHP.messages.genericError);
        },
        complete: function() {
            $('html').removeClass('ajax-loading');
            $('#loading-screen').fadeOut(100, function() { $(this).addClass('paused'); });
            $('.main-content').fadeIn();
        }
    });
}

function deleteAction(id) {
    $('html').addClass('ajax-loading');
    $('.main-content').fadeOut();
    $('#loading-screen').fadeIn(100);
    // Résout un bug sur ie
    setTimeout(function() {
        $('#loading-screen').removeClass('paused');
    }, 10);
    $.ajax({
        url: "<?=$this->Url->build('/Documents/delete')?>/"+id,
        method: 'DELETE',
        success: function(content, textStatus, jqXHR) {
            if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
                AsalaeGlobal.interceptedLinkToAjax(''+location);
            } else {
                this.error();
            }
        },
        error: function() {
            alert(PHP.messages.genericError);
        },
        complete: function() {
            $('html').removeClass('ajax-loading');
            $('#loading-screen').fadeOut(100, function() { $(this).addClass('paused'); });
            $('.main-content').fadeIn();
        }
    });
}
$(function() {
    $('#documents-section').find('ul.pagination a, table > thead > tr > th a').attr('data-mode', '');
});
<?php
    if ($user_id && $editId = $this->request->getParam('?.edit')) {
        echo 'setTimeout(function() { loadEditDocumentModal('.$editId.'); }, 500);';
    }
?>
</script>
