<?php
/**
 * @var \Refae\View\AppView $this
 */

use Refae\Model\Entity\Category;
use Refae\Model\Entity\Document;

$id = $this->request->getParam('pass.0');
$actualVersion = 0;
$tabsLinks = [];
$tabsContent = [];
$View = $this;

/** @var Document $document */
foreach ($documents as $document) {
    $version = $document->get('version');
    $tId = 'tab-view-version-'.$version;
    if ((string)$document->get('id') === $id) {
        $actualVersion = $version;
    }
    $tabsLinks[] = $this->Html->tag('li',
        $this->Html->tag('a',
            $this->Html->tag('span', '', ['class' => 'fa fa-code-fork'])
            .' '. __("V. {0}", $version),
            ['href' => '#'.$tId]
        )
    );
    $small = $this->Html->tag('span', __("Version {0}", $version), ['class' => 'small']);
    $categories = array_map(function($v) use ($View) {
            /** @var Category $v */
            $name = h($v->get('name'));
            if ($v->get('type') === 'table') {
                $link = $View->Html->link($name, ['controller' => 'Documents', 'action' => 'index', '?' => ['table_id' => $v->get('id')]]);
            } else {
                $link = $View->Html->link($name, ['controller' => 'Documents', 'action' => 'index', '?' => ['subcat' => $v->get('id')]]);
            }
            return $link;
        },
        (array)$document->get('categories')
    );
    if ($categories && count($categories) > 1) {
        $categories = '<ul><li>'.implode('</li><li>', $categories).'</li></ul>';
    } elseif ($categories) {
        $categories = current($categories);
    }
    $fichier = '';
    if ($file = $document->get('file')) {
        $fichier = $this->Html->link($file->get('filename'), '/files/download/'.$file->get('id'));
    }


    $table = $this->Html->tag('table',

        $this->Html->tag('tbody',
            $this->Html->tag('tr',
                $this->Html->tag('th', __('Identifiant'))
                .$this->Html->tag('td', $document->get('identifiant'))
            )
            .$this->Html->tag('tr',
                    $this->Html->tag('th', __('Statut'))
                    .$this->Html->tag('td', $document->get('statustrad'))
                , ['class' => $document->get('status') === 'actif' ? '' : 'warning']
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __('Associée à'))
                .$this->Html->tag('td', $categories)
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __('Support'))
                .$this->Html->tag('td', $document->get('supporttrad'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __('Format de fichier'))
                .$this->Html->tag('td', $document->get('file_format'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __('Localisation du document'))
                .$this->Html->tag('td', $document->get('localisation'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Durée d'utilité courante"))
                .$this->Html->tag('td', is_numeric($duc = $document->get('duc')) ? __n("{0} an", "{0} ans", $duc, $duc) : '')
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Date de départ de calcul de la DUC"))
                .$this->Html->tag('td', $document->get('depart_calcul_duc'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Sort final"))
                .$this->Html->tag('td', $document->get('sort_finaltrad'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Durée d'utilité administrative"))
                .$this->Html->tag('td', is_numeric($dua = $document->get('dua')) ? __n("{0} an", "{0} ans", $dua, $dua) : '')
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Date de départ de calcul de la DUA"))
                .$this->Html->tag('td', $document->get('depart_calcul_dua'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Communicabilité"))
                .$this->Html->tag('td', $document->get('communicabilite'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Date de départ de calcul de la communicabilité"))
                .$this->Html->tag('td', $document->get('depart_calcul_com'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Nom du profil d'archivage"))
                .$this->Html->tag('td', $document->get('profil_archivage'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Schéma du profil d'archivage"))
                .$this->Html->tag('td', $fichier)
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Date de création"))
                .$this->Html->tag('td', $document->get('created'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Date de modification"))
                .$this->Html->tag('td', $document->get('modified'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Date de publication"))
                .$this->Html->tag('td', $document->get('public_date'))
            )
            .$this->Html->tag('tr',
                $this->Html->tag('th', __("Date de fin de publication"))
                .$this->Html->tag('td', $document->get('obsolete_date'))
            )
        )

        , ['class' => 'table table-striped table-hover fixed-left-th']
    );
    $tabsContent[] = $this->Html->tag('div',

        $this->Html->tag('div',
                $this->Html->tag('h3', h($document->get('name')) . ' ' . $small, ['class' => 'h2'])
                .$this->Html->tag('p', h($document->get('description')))
                .(($note = $document->get('note_version'))
                    ? '<hr>'.$this->Html->tag('span', __('Motif de la nouvelle version').':', ['class' => 'fake-label']).$this->Html->tag('p', h($note))
                    : ''
                )
            , ['class' => 'ui-tabs-panel']
        )
        .$table

        , ['id' => $tId, 'class' => 'no-padding']
    );
}

// Tabulation et contenu
echo $this->Html->tag('div', null, ['id' => 'tab-container', 'class' => 'row no-padding'])
    .$this->Html->tag('ul', implode("\n\t\t", $tabsLinks), ($user_id ?? false) ? [] : ['class' => 'hide'])
    .implode("\n\t", $tabsContent)
    .$this->Html->tag('/div')
;
?>
<script>
    var actualId = 'tab-view-version-<?=$actualVersion?>';
    $('#tab-container').tabs({active: $('#'+actualId).index() -1});
</script>
