/* globals $, __, AsalaeGlobal */

if (typeof __ === 'undefined') { var __ = function(str) { return str; }; }

var AsalaeModal = {
    loading:'<div class="text-center loading-container">'
        +'<i class="fa fa-4x fa-spinner faa-spin animated" aria-hidden="true"></i>'
        +'<span class="sr-only">Loading...</span></div>'
};

/**
 * Ajoute une modale à
 * @param {jQuery|Array} modal
 */
AsalaeModal.append = function(modal) {
    var body = $('body');
    if (Array.isArray(modal)) {
        for (var i = 0; i < modal.length; i++) {
            AsalaeModal.append(modal[i]);
        }
        return;
    }
    // Supprime les eventuels doublons
    $('[id="'+modal.attr('id')+'"]').each(function() {
        if (this !== modal.get(0)) {
            $(this).remove();
        }
    });
    AsalaeModal.modalList().append(modal);
    modal.find('div.modal-header, .bottom-handle').off('.draggableCustom');
    modal.find('> .modal-dialog').draggableCustom({handle: '> .modal-content > .modal-header, > .modal-content > .bottom-handle'})
        .find('div.modal-header, .bottom-handle').css('cursor', 'move')
    ;
    // Remet la class modal-open lors de la fermeture d'une modal si il en reste une d'ouverte
    modal.off('hidden.bs.modal').on('hidden.bs.modal', function() {
        if ($('.modal:visible').length) {
            $('body').addClass('modal-open');
        }
    });
    // Fermeture de la modale si on presse plusieurs fois "Esc"
    modal.off('keydown.modal').on('keydown.modal', function(event) {
        var button = $(this).find('button.cancel');
        if (event.keyCode === 27) {
            let count = $(this).data('escape-count');
            if (!count) {
                count = 0;
            }
            count++;
            if (count >= 3) {
                $(this).removeClass('escaping')
                    .data('escape-count', count = 0);
                button.removeClass('active');
                $(this);
                if (!button.click().length) {
                    $(this).modal('hide');
                }
            } else {
                $(this).addClass('escaping')
                    .data('escape-count', count);
            }
            if (count > 1) {
                button.addClass('active');
            }
        } else {
            $(this).removeClass('escaping')
                .data('escape-count', 0);
            button.removeClass('active');
        }
    });
};

/**
 * Renvoi le container de modals, le crée au besoin
 * @return {jQuery}
 */
AsalaeModal.modalList = function() {
    var modalList = $('body > div.modal-list');
    if (modalList.length === 0) {
        modalList = $('<div class="modal-list"></div>');
        $('body').append(modalList);
    }
    return modalList;
};

/**
 * Action d'un bouton d'ouverture d'une modale avec appel ajax
 * @param button
 * @param modal
 * @param url
 */
AsalaeModal.button = function(button, modal, url) {
    $(button).off().clickEnter(function () {
        console.log($(modal));
        AsalaeModal.append($(modal).modal('show'));
        $(modal).find('div.modal-body').html(AsalaeModal.loading);
        AsalaeGlobal.addWaitingAjaxResponse(modal);
        $.ajax({
            url: url,
            success: function (content) {
                $(modal).find('div.modal-body').html(content);
            },
            error: function (error) {
                AsalaeModal.error(error, modal);
            },
            complete: function () {
                AsalaeGlobal.removeWaitingAjaxResponse(modal);
            }
        });
    });
};

/**
 * Erreur générique d'un appel ajax pour une modal
 * @param error
 * @param modal
 */
AsalaeModal.error = function(error, modal) {
    console.error(error);
    var message = '',
        alertBox = $('<div class="alert alert-danger"></div>')
    ;
    if (!error) {
        alertBox.append(__("Une erreur inattendue a eu lieu"));
    } else if (typeof error === 'string') {
        alertBox.append(error);
    } else if (error.responseText) {
        message = $('<div></div>').append($(error.responseText).find('.main-content').html());
    } else if (error.status === 403 && error.getResponseHeader('X-Connected') === 'false') {
        message = alertBox.append(__("Vous devez être connecté pour accéder à ce contenu"));
    } else if (error.status === 403) {
        message = alertBox.append(__("Vous n'avez pas l'autorisation d'accéder à ce contenu"));
    } else {
        message = alertBox.append(error.statusText);
    }
    $(modal).find('div.modal-body').empty().append(message);
};