/*global Flow, TableHelper, TableGenerator*/

/**
 * Permet de faire le pont entre asalae2 et Flow.js
 * Est utilisé par \App\View\Helper\UploadHelper
 *
 * @category    Helper
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */

/**
 * Une nouvelle instance de cette classe permet une nouvelle instance de Flow
 * Selon "config", l'objet sera couplé avec une "dropbox" et remplira une table.
 */
class AsalaeUploader
{
    constructor(config) {
        var dropbox = $('#'+config.id+'.dropbox');
        this.config = config;
        this.uploader = new Flow({
            target: config.target,
            chunkSize: config.chunkSize,
            testChunks: false
        });
        this.uploader.config = config;
        this.uploader.asalae = this;

        if (!this.uploader.support) {
            dropbox.css('background-color', '#f2dede').find('*').attr('disabled', 'disabled');
            dropbox.find('p').text('Votre navigateur ne supporte pas la fragmentation de fichiers, merci de le mettre à jour.');
            dropbox.find('label').text('Ou contactez un administrateur.');
            return;
        }

        this.uploader.assignDrop(dropbox.get(0));

        let isDirectory = typeof config.isDirectory === 'undefined'
            ? false
            : config.isDirectory;
        let singleFile = typeof config.singleFile === 'undefined'
            ? false
            : config.singleFile;
        let attributes = typeof config.attributes === 'undefined'
            ? {}
            : config.attributes;
        let button = dropbox.find('div.btn').off('.upload').on('keydown', function(event) {
            if (event.originalEvent.keyCode === 13) {
                $(this).click();
            }
        });
        this.uploader.assignBrowse(
            button.get(0),
            isDirectory,
            singleFile,
            attributes);

        let events = [
            'fileAdded',
            'filesSubmitted',
            'complete',
            'fileSuccess',
            'fileError',
            'fileProgress',
            'uploadStart',
            'catchAll'];

        for (let key in events) {
            // noinspection JSCheckFunctionSignatures
            this.uploader.on(events[key], eval('this.'+events[key]));
        }

        var input = dropbox.find('input[type="file"]'),
            label = $('<label class="sr-only"></label>')
                .text('File') // TODO Traduction (le fichier entier)
        ;
        label.insertBefore(input);
        label.append(input);
    }

    fileAdded(file) {
        $('#'+this.config.id+'.dropbox').trigger('fileAdded', [file]);
        $('#'+this.config.id+'.dropbox.dragover').removeClass('dragover');
    }

    filesSubmitted(file) {
        var progressBar, pause, resume, cancel, prev,
            table = eval(this.config.table);

        $('#'+this.config.id+'.dropbox').trigger('filesSubmitted', [file]);

        for (let i = 0; i < file.length; i++) {
            AsalaeUploader[file[i].uniqueIdentifier] = {
                resume: function(element) {
                    $(element).parent().parent()
                        .find('.progress.uploading .progress-bar')
                        .removeClass('progress-bar-warning');
                    file[i].resume();
                    $(element).hide();
                    $(element).parent().find('button.pause').show();},
                cancel: function(element) {
                    file[i].cancel();
                    table.removeDataId($(element).attr('data-target'));
                    $(element).closest('tr')
                        .fadeOut(200, function() {
                            $(this).remove();
                            $(table.table).trigger('change');
                        });},
                pause: function(element) {
                    $(element).parent().parent()
                        .find('.progress.uploading .progress-bar')
                        .addClass('progress-bar-warning');
                    file[i].pause();
                    $(element).hide();
                    $(element).parent().find('button.resume').show();
                }
            };
            progressBar = $('<div></div>').append(
                    $('<div></div>')
                    .addClass('flow-file-progress')
                    .addClass('progress')
                    .addClass('uploading')
                    .append(
                        $('<div></div>')
                        .addClass('progress-bar')
                        .attr('role', 'progressbar')
                        .attr('aria-valuenow', '0')
                        .attr('aria-valuemin', '0')
                        .attr('aria-valuemax', '100')
                        .css('width', '0%')))
                .append(
                    $('<span></span>')
                    .addClass('text-danger')
                    .addClass('error-span'));
            pause = $('<button></button>')
                .addClass('btn')
                .addClass('btn-warning')
                .addClass('fa')
                .addClass('fa-pause')
                .addClass('pause')
                .attr('onclick', "AsalaeUploader['"+file[i].uniqueIdentifier+"'].pause(this)");
            resume = $('<button></button>')
                .addClass('btn')
                .addClass('btn-success')
                .addClass('fa')
                .addClass('fa-play')
                .addClass('resume')
                .hide()
                .attr('onclick', "AsalaeUploader['"+file[i].uniqueIdentifier+"'].resume(this)");
            cancel = $('<button></button>')
                .addClass('btn')
                .addClass('btn-danger')
                .addClass('fa')
                .addClass('fa-ban')
                .addClass('cancel')
                .attr('data-target', file[i].uniqueIdentifier)
                .attr('onclick', "AsalaeUploader['"+file[i].uniqueIdentifier+"'].cancel(this)");

            table.data.unshift({
                id: file[i].uniqueIdentifier,
                prev: null,
                name: file[i].name,
                size: file[i].size,
                transfert: progressBar,
                valid: true,
                actions: {
                    action_0: $('<div></div>')
                        .append(pause)
                        .append(resume)
                        .append(cancel)
                }
            });
            table.generateTbody();
            $('#table-'+table.instanceIndex+'-tr-'+file[i].uniqueIdentifier+' .td-checkbox input').remove();
        }
        this.resume();
    }

    complete() {
        $('#'+this.config.id+'.dropbox').trigger('complete');
    }

    fileSuccess(file, message) {
        message = JSON.parse(message);
        var table = eval(this.config.table),
            data = table.getDataId(file.uniqueIdentifier);

        $('#'+this.config.id+'.dropbox').trigger('fileSuccess', [file, message]);

        if (!message.report.completion || message.report.error) {
            return this.asalae.fileError(file, message);
        }

        $('#table-'+table.instanceIndex+'-tr-'+file.uniqueIdentifier+' .flow-file-progress .progress-bar')
            .addClass('progress-bar-success');

        delete data['actions'];
        data.id = message.report.id;
        data.prev = message.report.prev;
        data.convertable = message.report.convertable;
        data.mime = message.report.mime;
        data.statefr = message.report.statefr;
        data.siegfrieds = [{pronom: message.report.pronom}];
        data.openable = message.report.openable;
        table.replaceDataId(
            file.uniqueIdentifier,
            TableGenerator.appendActions([data], table.actions)[0]
        );
        table.generateTbody();
    }

    fileError(file, message) {
        var table = eval(this.config.table), data;
        if (typeof message === 'string') {
            try {
                message = JSON.parse(message);
            } catch (e) {
                message = {report: {error: "Une erreur inattendue a eu lieu"}};
            }
        }
        $('#'+this.config.id+'.dropbox').trigger('fileError', [file, message]);
        $('#table-'+table.instanceIndex+'-tr-'+file.uniqueIdentifier+' .flow-file-progress .progress-bar')
            .css('width', '100%')
            .attr('aria-valuenow', 100)
            .html('100%')
            .addClass('progress-bar-danger')
            .closest('td').append(message.report.error)
            .closest('tr').find('button.pause').remove();
        data = table.getDataId(file.uniqueIdentifier);
        data.transfert = $('#table-'+table.instanceIndex+'-tr-'+file.uniqueIdentifier+' td.transfert').html();
        table.replaceDataId(file.uniqueIdentifier, data);
    }

    fileProgress(file) {
        var table = eval(this.config.table),
            percent = Math.floor(file.progress()*100);
        $('#'+this.config.id+'.dropbox').trigger('fileProgress', [file]);
        $('#table-'+table.instanceIndex+'-tr-'+file.uniqueIdentifier+' .flow-file-progress .progress-bar')
            .css('width', percent+'%')
            .attr('aria-valuenow', percent)
            .html(percent + '% ');
    }

    uploadStart() {
        $('#'+this.config.id+'.dropbox').trigger('uploadStart');
    }

    catchAll() {

    }
}

/**
 * Fonctionnement générique d'une dropbox d'upload
 */
var genericConfiguration = {
    tableGenerator: undefined,
    urlDelete: undefined,
    dropBox: undefined,
    paginationElement: undefined,
    selectUrl: undefined,
    messages: {
        confirmDelete: "Êtes-vous sûr de vouloir supprimer ce fichier ?",
        confirmDeleteAll: "Êtes-vous sûr de vouloir supprimer les fichiers sélectionnés ?",
        deleteError: "Une erreur a eu lieu lors de la suppression",
        selectionFailed: "Une erreur inattendue a eu lieu"
    }
};
/**
 * Initialise une dropbox avec son tableau d'affichage pour une utilisation générique
 */
class GenericUploader {
    constructor(config) {
        genericConfiguration = $.extend(true, {}, genericConfiguration, config);

        $(config.tableGenerator.table).change(function () {
            if ($(this).find('tbody tr').length === 0) {
                $(this).addClass('hide')
                    .closest('section')
                    .find('button.btn-primary')
                    .parent()
                    .addClass('hide');
            }
        });
        $(config.tableGenerator.table).find('thead input.checkall').change(function () {
            $(this).closest('table')
                .find('tbody .td-checkbox input')
                .prop('checked', $(this).prop('checked'));
        });

        $('#'+config.dropBox.config.id+'.dropbox').off().on('fileAdded', function() {
            $(config.tableGenerator.table).removeClass('hide')
                    .closest('section').find('button.btn-primary').parent().removeClass('hide');
        });
    }

    /**
     * Supprime un fichier uploadé
     *
     * @param {string} id
     * @param {boolean} useConfirm
     */
    static fileDelete(id, useConfirm = true) {
        var trId = '#table-' + genericConfiguration.tableGenerator.instanceIndex + '-tr-' + id;
        if (useConfirm && !confirm(genericConfiguration.messages.confirmDelete)) {
            return;
        }
        $.ajax({
            url: genericConfiguration.urlDelete + '/' + id,
            success: function (message) {
                if (message.report === 'done') {
                    genericConfiguration.tableGenerator.removeDataId(id);
                    $(trId).fadeOut(400, function () {
                        $(this.remove());
                        $(genericConfiguration.tableGenerator.table).trigger('change');
                    });
                } else {
                    this.error();
                }
            },
            error: function () {
                var data;
                $(trId)
                    .addClass('danger')
                    .find('td.transfert span.error-span')
                    .text(genericConfiguration.messages.deleteError);
                data = genericConfiguration.tableGenerator.getDataId(id);
                data.transfert = $(trId+' td.transfert').html();
                data.valid = false;
                genericConfiguration.tableGenerator.replaceDataId(id, data);
            }
        });
    }

    /**
     * Modal d'informations supplémentaire sur le fichier
     *
     * @param {string} id
     */
    static loadDataModal(id) {
        var data = genericConfiguration.tableGenerator.getDataId(id),
            div = $('<div></div>'),
            schema = [
                {separator: 'h3', value: 'Informations générales'},
                {title: 'Nom de fichier', value: data.name},
                {title: 'Taille', value: data.size, callback: TableHelper.readableBytes}
            ],
            value;

        if (typeof data.mediainfo !== 'undefined') {
            schema = schema.concat(
                {separator: 'h3', value: 'Mediainfo'},
                {title: 'Format de fichier', value: data.mediainfo.format},
                {title: 'Durée', value: data.mediainfo.duration},
            );

            for (let i in data.mediainfo.mediainfo_videos) {
                schema = schema.concat(
                    {separator: 'h4', value: 'Video'},
                    {title: 'Format', value: data.mediainfo.mediainfo_videos[i].format},
                    {title: 'Débit', value: data.mediainfo.mediainfo_videos[i].bit_rate},
                    {title: 'Largeur', value: data.mediainfo.mediainfo_videos[i].width},
                    {title: 'Hauteur', value: data.mediainfo.mediainfo_videos[i].height},
                    {title: 'Langue', value: data.mediainfo.mediainfo_videos[i].language}
                );
            }

            for (let i in data.mediainfo.mediainfo_audios) {
                schema = schema.concat(
                    {separator: 'h4', value: 'Audio'},
                    {title: 'Format', value: data.mediainfo.mediainfo_audios[i].format},
                    {title: 'Débit', value: data.mediainfo.mediainfo_audios[i].bit_rate},
                    {title: 'Langue', value: data.mediainfo.mediainfo_audios[i].language}
                );
            }

            for (let i in data.mediainfo.mediainfo_texts) {
                schema = schema.concat(
                    {separator: 'h4', value: 'Sous-titres'},
                    {title: 'Titre', value: data.mediainfo.mediainfo_texts[i].title},
                    {title: 'Langue', value: data.mediainfo.mediainfo_texts[i].language}
                );
            }

            for (let i in data.mediainfo.mediainfo_images) {
                schema = schema.concat(
                    {separator: 'h4', value: 'Image'},
                    {title: 'Format', value: data.mediainfo.mediainfo_images[i].format}
                );
            }
        }

        if (typeof data.siegfrieds !== 'undefined' && data.siegfrieds.length) {
            schema = schema.concat(
                {separator: 'h3', value: 'Siegfried'}
            );

            for (let i in data.siegfrieds) {
                schema = schema.concat(
                    {title: 'Pronom', value: data.siegfrieds[i].pronom},
                    {title: 'Format', value: data.siegfrieds[i].format},
                    {title: 'Mime', value: data.siegfrieds[i].mime}
                );
            }
        }

        for (let i in schema) {
            value = typeof schema[i].callback !== 'undefined'
                ? schema[i].callback(schema[i].value)
                : schema[i].value;

            if (typeof schema[i].separator !== 'undefined') {
                div.append(
                    $('<' + schema[i].separator + '></' + schema[i].separator + '>')
                        .append(schema[i].value));
            } else {
                div.append(
                    $('<span></span>').addClass('field').append(schema[i].title))
                    .append(
                        $('<span></span>').addClass('value').append(value));
            }
        }

        $('#info-file').find('.modal-body').html('').append(div);
    }

    /**
     * Ajoute un fichier selectionné
     *
     * @param {string} id
     */
    static fileSelect(id) {
        var trId = '#table-'+genericConfiguration.tableGenerator.instanceIndex+'-tr-'+id;
        $(trId)
            .addClass('warning')
            .find('td.transfert')
            .append($('<div class="text-center"><i aria-hidden="true" class="fa fa-2x fa-spinner faa-spin animated"></i></div>'));
        $(trId+' td.action a').disable();
        $.ajax({
            url: genericConfiguration.selectUrl + '/' + id,
            success: function (content, textStatus, jqXHR) {
                var data;
                if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
                    genericConfiguration.tableGenerator.removeDataId(id);
                    $(trId).fadeOut(400, function () {
                        AsalaeGlobal.updatePaginationAjax($(genericConfiguration.paginationElement));
                        $(this).remove();
                        $(genericConfiguration.tableGenerator.table).trigger('change');
                    });
                } else {
                    $(trId+' td.action a').enable()
                        .closest('tr').addClass('danger').removeClass('warning')
                        .find('.fa-spinner').parent().remove();
                    if (typeof content === 'object' && typeof content.report === 'string') {
                        $(trId+' td.transfert span.error-span')
                            .text(content.report);
                    } else {
                        $(trId+' td.transfert span.error-span')
                            .text(genericConfiguration.messages.selectionFailed);
                    }
                    data = genericConfiguration.tableGenerator.getDataId(id);
                    data.transfert = $(trId+' td.transfert').html();
                    data.valid = false;
                    genericConfiguration.tableGenerator.replaceDataId(id, data);
                }
            },
            error: function () {
                var data;
                $(trId+' td.action a').enable()
                    .closest('tr').addClass('danger').removeClass('warning')
                    .find('.fa-spinner').parent().remove();
                data = genericConfiguration.tableGenerator.getDataId(id);
                data.valid = false;
                genericConfiguration.tableGenerator.replaceDataId(id, data);
                alert("Une erreur a eu lieu");
            }
        });
    }

    /**
     * Ajoute les fichier selectionnés (table upload)
     */
    static addSelected() {
        $(genericConfiguration.tableGenerator.table).find('tbody > tr > td.td-checkbox input:enabled:checked').each(
            function() {
                $(this).disable();
                GenericUploader.fileSelect($(this).val());
            });
        $(genericConfiguration.tableGenerator.table).find('input:checked').prop('checked', false).enable();
    }
}