<?php
/**
 * AsalaeCore\Error\AppExceptionRenderer
 */

namespace AsalaeCore\Error;

use AsalaeCore\Http\AppServerRequestFactory;
use Cake\Controller\Controller;
use Cake\Core\Configure;
use Cake\Core\Exception\MissingPluginException;
use Cake\Error\ExceptionRenderer;
use Cake\Event\Event;
use Cake\Http\Response;
use Cake\Routing\Router;
use Cake\View\Exception\MissingTemplateException;
use Exception;

/**
 * Ajoute des gestions d'erreur à l'application
 *
 * @category    Error
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */
class AppExceptionRenderer extends ExceptionRenderer
{
    protected function _getController()
    {
        $errorController = Configure::read('Error.errorController');
        if (!$errorController) {
            return parent::_getController();
        }

        // Copié collé (modifié du parent::_getController())
        if (!$request = Router::getRequest(true)) {
            $request = AppServerRequestFactory::fromGlobals(); // modifié
        }
        $response = new Response();
        $controller = null;

        try {
            // supprimé
            /* @var \Cake\Controller\Controller $controller */
            $controller = new $errorController($request, $response); // modifié
            $controller->startupProcess();
            $startup = true;
        } catch (Exception $e) {
            $startup = false;
        }

        // Retry RequestHandler, as another aspect of startupProcess()
        // could have failed. Ignore any exceptions out of startup, as
        // there could be userland input data parsers.
        if ($startup === false && !empty($controller) && isset($controller->RequestHandler)) {
            try {
                $event = new Event('Controller.startup', $controller);
                $controller->RequestHandler->startup($event);
            } catch (Exception $e) {
            }
        }
        if (empty($controller)) {
            $controller = new Controller($request, $response);
        }

        return $controller;
    }

    /**
     * Generate the response using the controller object.
     *
     * @param string $template The template to render.
     * @return \Cake\Http\Response A response object that can be sent.
     */
    protected function _outputMessage($template)
    {
        try {
            $this->controller->render($template);

            return $this->_shutdown();
        } catch (MissingTemplateException $e) {
            $attributes = $e->getAttributes();
            if (isset($attributes['file']) && strpos($attributes['file'], 'error500') !== false) {
                return $this->_outputMessageSafe('error500');
            }

            return $this->_outputMessage('error500');
        } catch (MissingPluginException $e) {
            $attributes = $e->getAttributes();
            if (isset($attributes['plugin']) && $attributes['plugin'] === $this->controller->plugin) {
                $this->controller->plugin = null;
            }

            return $this->_outputMessageSafe('error500');
        } catch (Exception $e) {
            debug($e); // Unique modification : seul moyen de comprendre pourquoi le render à échoué
            return $this->_outputMessageSafe('error500');
        }
    }

    /**
     * A safer way to render error messages, replaces all helpers, with basics
     * and doesn't call component methods.
     *
     * @param string $template The template to render.
     * @return \Cake\Http\Response A response object that can be sent.
     */
    protected function _outputMessageSafe($template)
    {
        $response = parent::_outputMessageSafe($template);
        $this->controller->viewBuilder()->setLayoutPath('Layout')->setLayout('empty');
        return $response;
    }
}
