<?php
/**
 * AsalaeCore\Model\Entity\Webservice
 */

namespace AsalaeCore\Model\Entity;

use Cake\ORM\Entity;
use Cake\Utility\Security;

/**
 * Entité de la table webservices
 *
 * @category Entity
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */
class Webservice extends Entity
{
    /**
     * @var string clef de sécurité
     */
    private static $key;

    /**
     * Permet d'obtenir la clef de sécurité
     * @return string
     */
    private function getSecurityKey(): string
    {
        if (!self::$key) {
            $path = include CONFIG . 'path_to_local.php';
            $confJson = file_get_contents($path);
            $conf = json_decode($confJson, true);
            self::$key = $conf['Security']['salt'];
        }
        return self::$key;
    }
    /**
     * Permet d'obtenir le noeud parent d'un point de vue Permissions
     *
     * @return null
     */
    public function parentNode()
    {
        return null;
    }

    /**
     * Encrypte le mot de passe
     *
     * @param string $password
     * @return string
     */
    protected function _setPassword(string $password): string
    {
        return Security::hash($password, 'sha256', true);
    }

    /**
     * Protège le client_secret
     * @param string $secret
     * @return string
     */
    protected function _setClientSecret(string $secret): string
    {
        return bin2hex(Security::encrypt($secret, $this->getSecurityKey()));
    }

    /**
     * Getter du secret décripté
     * @return string
     */
    protected function _getClientSecretUnsecure(): string
    {
        return $this->_properties['client_secret']
            ? Security::decrypt(hex2bin($this->_properties['client_secret']), $this->getSecurityKey())
            : ''
        ;
    }

    /**
     * Permet de vérifier si un password correspond à celui de l'entité
     * @param string $password
     * @return bool
     */
    public function comparePassword(string $password): bool
    {
        $hashed = $this->_setPassword($password);
        return $this->_properties['password'] === $hashed;
    }

    /**
     * Génère un code via random_bytes, qui est cryptographiquement sûr
     * @param int $size
     * @return string
     */
    public function generateCode(int $size = 10): string
    {
        return bin2hex(random_bytes($size));
    }
}
