<?php
/**
 * AsalaeCore\Model\Table\AccessTokensTable
 */

namespace AsalaeCore\Model\Table;

use AsalaeCore\Model\Entity\AccessToken;
use AsalaeCore\Model\Entity\Webservice;
use Cake\Core\Configure;
use Cake\ORM\Entity;
use Cake\ORM\Table;
use DateTime;
use DateTimeZone;

/**
 * Table webservices
 *
 * @category Table
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */
class AccessTokensTable extends Table
{

    /**
     * Configuration initale de la table
     *
     * @param array $config
     */
    public function initialize(array $config)
    {
        $this->addBehavior('Timestamp');

        $this->belongsTo('Webservices');

        parent::initialize($config);
    }

    /**
     * Créer un accès token de connexion
     * @param Webservice $webservice
     * @param null|Entity $user
     * @return AccessToken
     */
    public function createCodeToken(Webservice $webservice, $user = null): AccessToken
    {
        $interval = Configure::read('Webservices.tokens-duration.code') ?: '1 minute';
        $date = new DateTime('now', new DateTimeZone('Europe/Paris'));
        $date->add(date_interval_create_from_date_string($interval));

        $code = $this->newEntity([
            'webservice_id' => $webservice->get('id'),
            'user_id' => $user ? $user->get('id') : null,
            'type' => 'code',
            'code' => $webservice->generateCode(),
            'expiration_date' => $date
        ]);
        $this->save($code);
        return $code;
    }

    /**
     * Tokens échangés contre un token code
     * @param Webservice $webservice
     * @param null|int $user_id
     * @return array
     */
    public function createAccessTokens(Webservice $webservice, $user_id): array
    {
        $iAccess = Configure::read('Webservices.tokens-duration.access-token') ?: '1 hour';
        $iRefresh = Configure::read('Webservices.tokens-duration.refresh-token') ?: '1 day';
        $timezone = new DateTimeZone('Europe/Paris');
        $now = new DateTime('now', $timezone);
        $dateAccess = clone $now;
        $dateAccess->add(date_interval_create_from_date_string($iAccess));
        $dateRefresh = clone $now;
        $dateRefresh->add(date_interval_create_from_date_string($iRefresh));

        $access = $this->newEntity([
            'webservice_id' => $webservice->get('id'),
            'user_id' => $user_id,
            'type' => 'access_token',
            'code' => $webservice->generateCode(),
            'expiration_date' => $dateAccess,
        ]);
        $this->save($access);

        $refresh = $this->newEntity([
            'webservice_id' => $webservice->get('id'),
            'user_id' => $user_id,
            'type' => 'refresh_token',
            'code' => $webservice->generateCode(),
            'expiration_date' => $dateRefresh
        ]);
        $this->save($refresh);

        return [
            'access_token' => $access->get('code'),
            'refresh_token' => $refresh->get('code'),
            'expiration_date' => $access->get('expiration_date'),
            'expires_in' => $dateAccess->getTimestamp() - $now->getTimestamp(),
        ];
    }
}
