<?php
/**
 * AsalaeCore\Utility\Notify
 */

namespace AsalaeCore\Utility;

use Cake\Core\Configure;
use Cake\Database\Connection;
use Cake\Datasource\ConnectionManager;
use ZMQ;
use ZMQContext;

/**
 * Utilitaire d'envoi des notifications
 *
 * @category    Utility
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */
class Notify
{

    /**
     * Permet d'envoyer des notifications aux utilisateurs
     *
     * @param int $user_id
     * @param string $message text affiché à l'utilisateur (HTML)
     * @param string $class CSS
     * @return array
     * @throws \ZMQSocketException
     */
    public static function send(int $user_id, string $message, string $class = 'alert-info'): array
    {
        $data = [
            'user_id' => $user_id,
            'text' => $message,
            'css_class' => $class,
            'created' => new \DateTime('now')
        ];

        $connexion = ConnectionManager::get('default');
        if ($connexion instanceof Connection) {
            $connexion->insert('notifications', $data, ['created' => 'datetime']);
            $results = $connexion->newQuery()
                ->select('id')
                ->from('notifications')
                ->where(['user_id' => $user_id])
                ->order(['id' => 'DESC'])
                ->limit(1)
                ->execute()
                ->fetch();
            $data['id'] = $results[0];
        }

        static::emit('user_'.$user_id, $data);

        return $data;
    }

    /**
     * Permet d'envoyer un message sur le serveur Ratchet
     *
     * @param string $chanel
     * @param mixed $message
     * @throws \ZMQSocketException
     */
    public static function emit(string $chanel, $message)
    {
        $config = Configure::read('Ratchet.zmq');
        $protocol = $config['protocol'];
        $domain = $config['domain'];
        $port = $config['port'];
        if (($ip = gethostbyname($domain)) !== $domain) {
            $domain = $ip;
        }
        $dsn = "$protocol://$domain:$port";
        $context = new ZMQContext();
        $socket = $context->getSocket(ZMQ::SOCKET_PUSH);
        $socket->setSockOpt(ZMQ::SOCKOPT_LINGER, 30);
        $socket->connect($dsn);
        $emit = [
            'category' => $chanel,
            'message' => $message
        ];
        $socket->send(json_encode($emit), ZMQ::MODE_DONTWAIT);
        $socket->disconnect($dsn);
    }
}
