<?php
/**
 * AsalaeCore\View\Helper\AclHelper
 */

namespace AsalaeCore\View\Helper;

use Acl\Controller\Component\AclComponent;
use Acl\Model\Table\AcosTable;
use AsalaeCore\Utility\Check;
use Cake\Controller\ComponentRegistry;
use Cake\Http\ServerRequest;
use Cake\Log\Log;
use Cake\ORM\TableRegistry;
use Cake\Routing\Router;
use Cake\View\Helper;
use Cake\View\View;

/**
 * Permet de vérifier les droits d'accès à un controller/action d'un utilisateur
 *
 * @category Helper
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 * @property \Cake\View\Helper\HtmlHelper $Html
 */
class AclHelper extends Helper
{
    /**
     * @var AclComponent
     */
    public $Acl;

    /**
     * @var AcosTable
     */
    public $Acos;

    /**
     * @var bool Connecté à la base de données
     */
    private $connected;

    /**
     * @var array Helpers utilisés
     */
    public $helpers = [
        'Html',
    ];

    /**
     * Construct method.
     *
     * @param View $view
     * @param array $config
     */
    public function __construct(View $view, array $config = [])
    {
        parent::__construct($view, $config);
        $collection = new ComponentRegistry();
        $this->Acl = new AclComponent($collection);
        $this->Acos = TableRegistry::getTableLocator()->get('Acl.Acos');
        $this->connected = Check::database();
    }

    /**
     * Permet de vérifier l'url pour l'utilisateur connecté
     *
     * @param mixed $url
     * @return boolean
     * @throws \Aura\Intl\Exception
     */
    public function check($url): bool
    {
        if (!isset($this->request->getSession()->read()['Auth']['User']) || !$this->connected) {
            return false;
        }
        $user = $this->request->getSession()->read()['Auth']['User'];
        $params = Router::parseRequest(new ServerRequest(['url' => Router::url($url)]));

        $aco = $this->Acos->find()
            ->where(['alias' => $params['action'], 'model' => $params['controller']])
            ->first();
        if (!$aco) {
            Log::error(__("La méthode {0}::{1} n'est pas définie dans les acos", $params['controller'], $params['action']));
        }
        return $this->Acl->check(['Users' => $user], $aco->id);
    }

    /**
     * Génère un lien (ou un lien avec son container) si les permissions sur
     * url le permettent
     * @param string $msg
     * @param $url
     * @param array $params
     * @param string $container si besoin, génère un container (ex: 'li', 'div')
     * @param array $containerParams
     * @return string
     * @throws \Aura\Intl\Exception
     */
    public function link(string $msg, $url, array $params = [], $container = '', array $containerParams = []): string
    {
        if (!$this->check($url)) {
            return '';
        }
        $output = $this->Html->link($msg, $url, $params);
        if (!empty($container)) {
            $output = $this->Html->tag($container, $output, $containerParams);
        }
        return $output;
    }
}
