<?php
/**
 * AsalaeCore\View\Helper\DateHelper
 */

namespace AsalaeCore\View\Helper;

use Cake\Core\Configure;
use Cake\I18n\Date;
use Cake\View\Helper;
use Cake\View\View;

/**
 * Utile pour la gestion des dates localisés
 *
 * @category Helper
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2018, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */
class DateHelper extends Helper
{
    /**
     * @var array Helpers utilisés
     */
    public $helpers = [
        'AsalaeCore.Fa',
        'Html',
    ];

    /**
     * @var array Locales disponnibles pour le datepicker jquery
     */
    const DATE_PICKER_AVALAIBLE_LOCALES = [
        'en', 'en-US', 'fr'
    ];

    /**
     * @var string options locales pour le datepicker
     */
    public $datePickerOptions;

    /**
     * Default Constructor
     *
     * @param \Cake\View\View $View The View this helper is being attached to.
     * @param array $config Configuration settings for the helper.
     */
    public function __construct(View $View, array $config = [])
    {
        parent::__construct($View, $config);
        $local = Configure::read('App.defaultLocale');
        list($prefix) = explode('_', $local, 2);
        switch ($local) {
            case 'fr':
            case ($prefix === 'fr'):
                $this->datePickerOptions = "$.datepicker.regional['fr']";
                break;
            case 'en_US':
                $this->datePickerOptions = "$.datepicker.regional['en-US']";
                break;
            case 'en':
            default:
                $this->datePickerOptions = "$.datepicker.regional['en']";
        }
    }

    /**
     * A utiliser dans le paramètre 'append' d'un champ de formulaire
     *
     * exemple:
     *      $this->Form->control('date', ['append' => $this->Date->picker('date')])
     *
     * @param string $selector
     * @return string
     */
    public function picker(string $selector): string
    {
        return $this->Fa->i('fa-calendar')
            . $this->Html->tag('script', '$("'.$selector.'").last().datepicker('.$this->datePickerOptions.');')
        ;
    }

    /**
     * Affiche la date du jour au format court localisé
     * ex 16/10/2018
     * @return string
     */
    public function today(): string
    {
        $today = new Date;
        return $today->format($today);
    }
}
