<?php
/**
 * AsalaeCore\View\Helper\FaHelper
 */

namespace AsalaeCore\View\Helper;

use Cake\View\Helper;

/**
 * Permet de générer des icônes, intègre la charte graphique Libriciel
 *
 * @category Helper
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 * @property \Cake\View\Helper\HtmlHelper $Html
 */
class FaHelper extends Helper
{
    /**
     * @var array Helpers utilisés
     */
    public $helpers = [
        'Html',
    ];

    /**
     * Liste les règles de la charte
     * @link https://gitlab.libriciel.fr/outils/charte-graphique
     */
    const CHARTE = [
        // Actions
        'Activer' => ['fa-toggle-off'],
        'Actualiser' => ['fa-refresh'],
        'Ajouter' => ['fa-plus-circle'],
        'Annuler' => ['fa-times-circle', 'fa-times-circle-o'],
        'Archiver' => ['fa-archive'],
        'Cacheter' => ['icon-cachet', 'icon-cachet-o'],
        'Clore' => ['fa-lock'],
        'Créer' => ['fa-plus'],
        'Commenter' => ['fa-commenting', 'fa-commenting-o'],
        'Annoter' => ['fa-commenting', 'fa-commenting-o'],
        'Désactiver ' => ['fa-toggle-off'],
        'Déverrouiller ' => ['fa-unlock-alt'],
        'Dupliquer' => ['fa-files-o'],
        'Enregistrer' => ['fa-floppy-o'],
        'Envoyer' => ['fa-paper-plane', 'fa-paper-plane-o'],
        'Exécuter' => ['fa-cog', 'fa-cogs'],
        'Filtrer' => ['fa-filter'],
        'Fusionner' => ['icon-fusion'],
        'Générer' => ['fa-cog', 'fa-cogs'],
        'Importer' => ['fa-upload'],
        'Parcourir' => ['fa-folder-open', 'fa-folder-open-o'],
        'Imprimer' => ['fa-print'],
        'Modifier' => ['fa-pencil'],
        'Rechercher' => ['fa-search'],
        'Refuser' => ['fa-times'],
        'Réinitialiser' => ['fa-undo'],
        'Supprimer' => ['fa-trash', 'fa-trash-o'],
        'Signer' => ['icon-signature', 'icon-signature-o'],
        'Se connecter' => ['fa-sign-in'],
        'Se déconnecter' => ['fa-sign-out'],
        'Télécharger' => ['fa-download'],
        'Valider' => ['fa-check'],
        'Verrouiller' => ['fa-lock'],
        'Visualiser' => ['fa-eye'],
        'Voter' => ['fa-gavel'],

        // Navigation
        'Accueil' => ['fa-home'],
        'Aide' => ['fa-question'],
        'Certificat' => ['fa-certificate'],
        'Circuit' => ['fa-road'],
        'Collectivité' => ['fa-building', 'fa-building-o'],
        'Connecteurs' => ['fa-plug'],
        'Dossier' => ['fa-folder', 'fa-folder-o'],
        'Fermer' => ['fa-times'],
        'Fichier' => ['fa-file-text', 'fa-file-text-o'],
        'Fichier PDF' => ['fa-file-pdf'],
        'Historique' => ['fa-history'],
        'Informations' => ['fa-info'],
        'Journal des évènements' => ['fa-list-alt'],
        'Liste' => ['fa-list-ul'],
        'Menu' => ['fa-bars'],
        'Mot de passe' => ['fa-lock'],
        'Notification' => ['fa-bell', 'fa-bell-o'],
        'Paramètres' => ['fa-sliders'],
        'Préférences' => ['fa-cogs'],
        'Retour' => ['fa-arrow-left'],
        'Services' => ['fa-sitemap'],
        'Statistiques' => ['fa-chart'],
        'Tableau de bord' => ['fa-tachometer'],
        'Utilisateur' => ['fa-user', 'fa-user-o'],
        'Utilisateurs' => ['fa-user', 'fa-user-o'],
    ];

    /**
     * Permet de générer une icône très facilement :
     *
     * exemple:
     * $this->Fa->i('fa-times') === '<i class="fa fa-times" aria-hidden="true"></i>'
     *
     * @param string $class
     * @param array $options
     * @return string
     * @throws \Aura\Intl\Exception
     */
    public function i(string $class, array $options = []): string
    {
        if (isset($options['class'])) {
            trigger_error(__("\$options['class'] ne doit pas être défini."));
            unset($options['class']);
        }
        $options += [
            'class' => 'fa '.$class,
            'aria-hidden' => 'true'
        ];
        return $this->Html->tag('i', '', $options);
    }

    /**
     * Renvoi l'icône définie par la charte graphique Libriciel
     * @param string $keyword
     * @param int|array|string $index si integer, $options si array ou string
     * @param array|string $options , si string, equivalent à ['class' => $options]
     * @return string
     * @throws \Aura\Intl\Exception
     */
    public function charte(string $keyword, $index = 0, $options = []): string
    {
        if (!isset(self::CHARTE[$keyword])) {
            trigger_error(__("{0} ne fait pas partie des valeurs autorisées", $keyword));
            debug(array_keys(self::CHARTE));
        }
        if (!is_numeric($index)) {
            $options = $index;
            $index = 0;
        }
        if (is_string($options)) {
            $options = ['class' => $options];
        }
        if (isset($options['icon'])) {
            $options['class'] = trim(($options['class'] ?? '').' '.$options['icon']);
            unset($options['icon']);
        }
        $class = trim(self::CHARTE[$keyword][$index].' '.($options['class'] ?? ''));
        unset($options['class']);
        return $this->i($class, $options);
    }

    /**
     * Permet d'obtenir un bouton
     * @param string $class classes css de l'icône font awesome
     * @param string $title texte descriptif de l'icône en title et sr-only
     * @param array $options à passer au bouton (ex: id, onclick etc...)
     * @return string
     * @throws \Aura\Intl\Exception
     */
    public function button(string $class, string $title = '', array $options = []): string
    {
        $options += [
            'type' => 'button',
            'class' => 'btn-link',
            'title' => $title,
            'escape' => false
        ];
        $text = '';
        $srOnly = $this->Html->tag('span', $title, ['class' => 'sr-only']);
        if (isset($options['srOnly'])) {
            $srOnly = $options['srOnly'];
            unset($options['srOnly']);
        }
        if (isset($options['text'])) {
            $text = $options['text'];
            unset($options['text']);
        }
        return $this->Html->tag('button', $this->i($class) . $srOnly . $text, $options);
    }

    /**
     * Donne un bouton avec icône de charte
     * @param string $keyword
     * @param string $title
     * @param int $index
     * @param array $options
     * @return string
     * @throws \Aura\Intl\Exception
     */
    public function charteBtn(string $keyword, string $title = '', $index = 0, $options = []): string
    {
        if (!isset(self::CHARTE[$keyword])) {
            trigger_error(__("{0} ne fait pas partie des valeurs autorisées", $keyword));
            debug(array_keys(self::CHARTE));
        }
        if (!is_numeric($index)) {
            $options = $index;
            $index = 0;
        }
        if (is_string($options)) {
            $options = ['icon' => $options];
        }
        $class = trim(self::CHARTE[$keyword][$index].' '.($options['icon'] ?? ''));
        unset($options['icon']);

        $options += [
            'type' => 'button',
            'class' => 'btn-link',
            'title' => $title,
            'escape' => false
        ];
        $text = '';
        $srOnly = $this->Html->tag('span', $title, ['class' => 'sr-only']);
        if (isset($options['srOnly'])) {
            $srOnly = $options['srOnly'];
            unset($options['srOnly']);
        }
        if (isset($options['text'])) {
            $text = $options['text'];
            unset($options['text']);
        }
        return $this->Html->tag('button', $this->i($class) . $srOnly . $text, $options);
    }
}
