<?php
/**
 * AsalaeCore\View\Helper\FilterHelper
 */

namespace AsalaeCore\View\Helper;

use AsalaeCore\View\Helper\Object\Filter;
use AsalaeCore\View\Helper\Object\ObjectInterface;
use Cake\View\Helper;

/**
 * Permet de générer des filtres de recherche
 *
 * @category Helper
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 * @property \AsalaeCore\View\Helper\FormHelper $Form
 * @property \AsalaeCore\View\Helper\DateHelper $Date
 * @property \Cake\View\Helper\HtmlHelper $Html
 * @property \Bootstrap\View\Helper\ModalHelper $Modal
 * @property \Cake\View\Helper\UrlHelper $Url
 */
class FilterHelper extends Helper implements ObjectHelperInterface
{
    /**
     * @var array Helpers utilisés
     */
    public $helpers = [
        'AsalaeCore.Form',
        'Html',
        'Bootstrap.Modal',
        'Url',
        'AsalaeCore.Date',
    ];

    /**
     * Permet de créer et de récupérer l'objet
     * @param string $id
     * @param array $params
     * @return ObjectInterface|Filter
     */
    public function create(string $id, array $params = []): ObjectInterface
    {
        return new Filter($this, $id);
    }

    /**
     * Fenetre affiché au survol pour selection des filtres
     * @param array $filters
     * @return string
     */
    public function selectSaved(array $filters): string
    {
        $selected = (int)$this->request->getParam('?.SaveFilterSelect');
        $lis = [];
        foreach ((array)$filters as $entity) {
            $id = $entity->get('id');
            $name = $entity->get('name');
            $filters = array_map(function ($v) {
                /** @var \AsalaeCore\Model\Entity\Filter $v */
                return $v->get('key') . '=' . urlencode($v->get('value'));
            }, $entity->get('filters'));
            $filters[] = "SaveFilterSelect=$id";
            $url = $this->Url->build(
                '/'.$entity->get('controller').'/'.$entity->get('action')
                ).'?'.implode('&', $filters)
            ;
            $params = $id === $selected ? ['class' => 'active'] : [];
            $params['class'] = empty($params['class'])
                ? 'savedFilter'
                : $params['class'] . ' savedFilter';
            $lis[] = $this->Html->tag(
                'li',
                $this->Html->tag(
                    'a',
                    $this->Html->tag('i', '', ['class' => 'fa fa-filter'])
                    . ' ' . $name,
                    [
                        'class' => 'navbar-link',
                        'href' => str_replace('[', '%5B', str_replace(']', '%5D', $url))
                    ]
                ),
                $params + ['data-id' => $id]
            );
        }
        return implode(PHP_EOL, $lis).PHP_EOL;
    }
}
