<?php
/**
 * AsalaeCore\View\Helper\FormHelper
 */

namespace AsalaeCore\View\Helper;

use Bootstrap\View\Helper\FormHelper as BootstrapFormHelper;
use Cake\View\View;

/**
 * Surcharge du Bootstrap\View\Helper\FormHelper
 *
 * @category Helper
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 * @property \Cake\View\Helper\UrlHelper $Url
 * @property \Cake\View\Helper\HtmlHelper $Html
 */
class FormHelper extends BootstrapFormHelper
{
    /**
     * {@inheritDoc}
     * @param View $View
     * @param array $config
     */
    public function __construct(View $View, array $config = [])
    {
        $this->_defaultConfig['templates']['checkboxContainer']
            = '<div class="form-group checkbox {{required}}">{{content}}</div>';
        parent::__construct($View, $config);
    }

    /**
     * Returns an HTML form element.
     *
     * ### Options
     *
     * - `context` Additional options for the context class. For example the
     * EntityContext accepts a 'table' option that allows you to set the specific Table
     * class the form should be based on.
     * - `encoding` Set the accept-charset encoding for the form. Defaults to
     * `Configure::read('App.encoding')`.
     * - `enctype` Set the form encoding explicitly. By default `type => file` will set
     * `enctype` to `multipart/form-data`.
     * - `horizontal` Boolean specifying if the form should be horizontal.
     * - `idPrefix` Prefix for generated ID attributes.
     * - `inline` Boolean specifying if the form should be inlined.
     * - `method` Set the form's method attribute explicitly.
     * - `templates` The templates you want to use for this form. Any templates will be
     * merged on top of the already loaded templates. This option can either be a filename
     * in /config that contains the templates you want to load, or an array of templates
     * to use.
     * - `templateVars` Provide template variables for the formStart template.
     * - `type` Form method defaults to autodetecting based on the form context. If
     *   the form context's isCreate() method returns false, a PUT request will be done.
     * - `url` The URL the form submits to. Can be a string or a URL array. If you use 'url'
     *    you should leave 'action' undefined.
     *
     * @override Bootstrap::create() - retrait du 'role' => 'form' forcé incompatible W3C
     *
     * @param mixed $model The context for which the form is being defined. Can
     *   be an ORM entity, ORM resultset, or an array of meta data. You can use false or null
     *   to make a model-less form.
     * @param array $options An array of html attributes and options.
     *
     * @return string An formatted opening FORM tag.
     */
    public function create($model = null, array $options = array()): string
    {
        $options += [
            'horizontal' => false,
            'inline' => false
        ];
        $this->horizontal = $options['horizontal'];
        $this->inline = $options['inline'];
        unset($options['horizontal'], $options['inline']);
        if ($this->horizontal) {
            $options = $this->addClass($options, 'form-horizontal');
        } elseif ($this->inline) {
            $options = $this->addClass($options, 'form-inline');
        }
        return \Cake\View\Helper\FormHelper::create($model, $options);
    }

    /**
     * Generates a form input element complete with label and wrapper div.
     *
     * ### Options
     *
     * See each field type method for more information. Any options that are part of
     * `$attributes` or `$options` for the different **type** methods can be included
     * in `$options` for input().
     * Additionally, any unknown keys that are not in the list below, or part of the
     * selected type's options will be treated as a regular HTML attribute for the
     * generated input.
     *
     * - `append` Content to append to the input, may be a string or an array of buttons.
     * - `empty` String or boolean to enable empty select box options.
     * - `error` Control the error message that is produced. Set to `false` to disable
     * any kind of error reporting (field error and error messages).
     * - `help` Help message to add below the input.
     * - `label` Either a string label, or an array of options for the label.
     * See FormHelper::label().
     * - `labelOptions` - Either `false` to disable label around nestedWidgets e.g. radio, multicheckbox or an array
     *   of attributes for the label tag. `selected` will be added to any classes e.g. `class => 'myclass'` where
     *   widget is checked.
     * - `nestedInput` Used with checkbox and radio inputs. Set to false to render
     * inputs outside of label elements. Can be set to true on any input to force the
     * input inside the label. If you enable this option for radio buttons you will also
     * need to modify the default `radioWrapper` template.
     * - `inline` Only used with radio inputs, set to `true` to have inlined radio buttons.
     * - `options` For widgets that take options e.g. radio, select.
     * - `templates` The templates you want to use for this input. Any templates will be
     * merged on top of the already loaded templates. This option can either be a filename
     * in /config that contains the templates you want to load, or an array of templates
     * to use.
     * - `prepend` Content to prepend to the input, may be a string or an array of buttons.
     * - `templateVars` Array of template variables.
     * - `type` Force the type of widget you want. e.g. `type => 'select'`
     *
     * @overide évite une erreur W3C en cas de name à vide
     *
     * @param string $fieldName This should be "modelname.fieldname"
     * @param array $options Each type of input takes different options.
     *
     * @return string Completed form widget.
     */
    public function control($fieldName, array $options = array())
    {
        $input = parent::control($fieldName, $options);
        if (empty($name)) {
            return str_replace(' name=""', '', $input);
        }
        return $input;
    }
}
