<?php
/**
 * AsalaeCore\View\Helper\ModalViewHelper
 */

namespace AsalaeCore\View\Helper;

use AsalaeCore\View\Helper\Object\ModalView;
use AsalaeCore\View\Helper\Object\ObjectInterface;
use Cake\View\Helper;

/**
 * Initialise une modal et crée le bouton/lien pour l'afficher
 *
 * @category Helper
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017-2018, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 * @property \AsalaeCore\View\Helper\FormHelper $Form
 * @property \Cake\View\Helper\HtmlHelper $Html
 * @property \Bootstrap\View\Helper\ModalHelper $Modal
 * @property \Cake\View\Helper\UrlHelper $Url
 */
class ModalViewHelper extends Helper implements ObjectHelperInterface
{
    /**
     * Helpers utilisés
     *
     * @var array
     */
    public $helpers = [
        'Form',
        'Html',
        'Modal' => ['className' => 'Bootstrap.Modal'],
        'Url'
    ];

    /**
     * Initialise une nouvelle modale
     * @param string $id
     * @param array $params
     * @return ObjectInterface|ModalView
     */
    public function create(string $id, array $params = []): ObjectInterface
    {
        return new ModalView($this, $id, $params);
    }

    /**
     * Permet d'obtenir le javascript additionnel qui va avec la modale
     * @param string $id
     * @return string
     */
    public function modalScript(string $id): string
    {
        return "<script> AsalaeModal.append($('#$id')); </script>";
    }
}
