<?php
/**
 * AsalaeCore\View\Helper\UploadHelper
 */

namespace AsalaeCore\View\Helper;

use Cake\I18n\I18n;
use Cake\View\Helper;

/**
 * Permet de générer les outils pour effectuer l'upload
 *
 * @category Helper
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 * @property \Cake\View\Helper\HtmlHelper $Html
 */
class TranslateHelper extends Helper
{
    /**
     * Helpers utilisés
     *
     * @var array
     */
    public $helpers = ['Html'];

    /**
     * Permet d'obtenir la traduction pour un controller / action dans un
     * contexte de droits
     *
     * @param string $controller
     * @param string $action
     * @return string
     * @throws \Aura\Intl\Exception
     */
    public function permission(string $controller, string $action): string
    {
        switch ($action) {
            case 'api.create':
                return __d('droit', 'Création via Web service');
            case 'api.read':
                return __d('droit', 'Lecture via Web service');
            case 'api.update':
                return __d('droit', 'Mise à jour via Web service');
            case 'api.delete':
                return __d('droit', 'Suppression via Web service');
        }
        if ($controller === 'Download') {
            switch ($action) {
                case 'file':
                    return __dx('droit', 'Download', 'file');
                case 'open':
                    return __dx('droit', 'Download', 'open');
                case 'thumbnailImage':
                    return __dx('droit', 'Download', 'thumbnailImage');
                case 'zip':
                    return __dx('droit', 'Download', 'zip');
            }
        }
        // TODO continuer avec les autres controllers

        return I18n::getTranslator('droit')->translate($action, ['_context' => $controller]);
    }

    /**
     * Traduction des options de permission
     * @param string $name
     * @return string
     * @throws \Aura\Intl\Exception
     */
    public function optionsPerms(string $name): string
    {
        switch ($name) {
            case 'versant':
                return __("Versant");
            case 'producteur':
                return __("Producteur");
            case 'admin':
                return __("Administrateur");
        }
        return I18n::getTranslator()->translate($name);
    }
}
