<?php
/**
 * AsalaeCore\View\Helper\UploadHelper
 */

namespace AsalaeCore\View\Helper;

use Cake\Core\Configure;
use Cake\View\Helper;
use Cake\Utility\Inflector;

/**
 * Permet de générer les outils pour effectuer l'upload
 *
 * @category Helper
 *
 * @author    Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright (c) 2017, Libriciel
 * @license   http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 * @property \Cake\View\Helper\HtmlHelper $Html
 */
class UploadHelper extends Helper
{
    /**
     * Helpers utilisés
     *
     * @var array
     */
    public $helpers = ['Html', 'AsalaeCore.Fa'];

    /**
     * @var array Configuration du helper
     */
    public $config;

    /**
     * @var string div de la dropbox
     */
    public $dropbox;

    /**
     * @var string id de la dropbox
     */
    public $dropboxId;

    /**
     * @var string nom de la variable du TableGenerator
     */
    public $table;

    /**
     * @var string nom de la variable AsalaeUploader
     */
    public $jsObject;

    /**
     * Initialisation du helper
     *
     * @param array $config
     */
    public function initialize(array $config)
    {
        parent::initialize($config);
        $this->config = (array)Configure::read('Flow') + [
            'chunkSize' => 10 * 1024 * 1024,
            'target' => '/upload'
        ];
    }

    /**
     * Créer une dropbox
     *
     * @param  string $domId
     * @param  string $table nom de la variable du TableGenerator (javascript)
     * @return \self
     * @throws \Aura\Intl\Exception
     */
    public function create(string $domId, string $table): self
    {
        $i = $this->Html->tag(
            'i',
            '',
            ['class' => 'fa fa-cloud-upload fa-4x text-primary']
        );
        $p = $this->Html->tag(
            'p',
            __('Glissez-déposez vos fichiers ici')
        );
        $button = $this->Html->tag(
            'div',
            $this->Fa->i('fa-folder-open').' '.__("Parcourir..."),
            ['class' => 'btn btn-success', 'tabindex' => 0]
        );

        $this->dropbox = $this->Html->tag(
            'div',
            $i.$p.$button,
            ['class' => 'dropbox', 'id' => $domId]
        );

        $this->dropboxId = $domId;
        $this->table = $table;
        $this->jsObject = '';

        return $this;
    }

    /**
     * Permet d'obtenir une dropbox et son javascript
     *
     * @param array $config Configuration supplémentaire à envoyer à AsalaeUploader
     * @return string
     */
    public function generate(array $config = []): string
    {
        $config += [
            'id' => $this->dropboxId,
            'table' => $this->table,
            'target' => $this->config['target'],
            'chunkSize' => $this->config['chunkSize'],
            'thumbnail_path' => '/download/thumbnail-image?path=',
            'download_path' => '/download/file?path=',
        ];

        $this->jsObject = Inflector::camelize(
            'uploader_'.preg_replace('/[^\w]/', '_', $this->dropboxId)
        );

        $script = $this->Html->tag(
            'script',
            $this->jsObject . ' = new AsalaeUploader('.json_encode($config).')'
        );

        return $this->dropbox . $script;
    }
}
