# Cake PHP 3 - Beanstalkd manager

## Prérequis

Un serveur Beanstalkd est indispensable.
```bash
sudo apt-get install beanstalkd
```

Si vous voulez conserver les jobs après le redémarrage du service, il vous faut
modifier la conf.
```bash
sudo nano /etc/default/beanstalkd
```

Décommettez la dernière ligne :
`BEANSTALKD_EXTRA="-b /var/lib/beanstalkd"`

Autre méthode : lancez __beanstalkd__ avec l'option __-d__

##### Utile mais pas indispensable

```bash
wget https://github.com/schickling/beanstalkd-cli/releases/download/0.3.0/beanstalkd-cli-linux.tar.gz
sudo tar -zxvf beanstalkd-cli-linux.tar.gz -C /bin && rm beanstalkd-cli-linux.tar.gz
beanstalkd-cli monitor
```

## Installation

```bash
composer config repositories.libriciel/cakephp-beanstalk git https://gitlab.libriciel.fr/lganee/cakephp-datacompressor.git
composer require libriciel/cakephp-beanstalk ~1.0
```

Ajoutez le plugin dans votre bootstrap.php

```php
Plugin::load('Beanstalk', ['bootstrap' => true]);
```

### Base de données

Vous pouvez au choix, utiliser le SQL (postgres) fourni dans **config/schema/beanstalk.sql**, ou bien récupérer le fichier de migration dans **config/Migrations/20170418074926_initial.php** pour utiliser le plugin de migration :
```bash
bin/cake migrations migrate
```

## Utilisation

Maitenant que vous disposez de votre serveur Beanstalkd, il va vous falloir
un ou plusieurs workers selon le besoin.

Exemple sur plugins/Beanstalk/exemple/DefaultWorker.php
Copiez ce fichier vers src/Shell/Worker
Ajoutez à votre configuration :
```php
Configure::write([
    'Beanstalk' => [
        /**
         * Connection au serveur Beanstalkd
         */
        'host' => '127.0.0.1',
        'port' => 11300,
        'timeout' => null,
        'persistant' => false,
        
        /**
         * Fréquence de vérification du worker (0: jamais, 100: toujours)
         */
        'worker_check_ratio' => $debugMode ? 100 : 10,
        
        /**
         * Liste des workers exécutables par l'application
         */
        'workers' => [
            'default_worker' => [
                'path' => ROOT.DS.'plugins'.DS.'Beanstalk'
                    .DS.'exemples'.DS.'DefaultWorker.php',
                'namespace' => '\App\Shell\Worker',
                'auto_launch' => false,
                'keep_alive' => false,
                'tube' => 'default',
                'params' => ['user_id' => 1]
            ],
        ]
    ]
]);
```

Lancez votre shell :
```bash
bin/cake Beanstalk.Worker default_worker
```

Lancez un job
```bash
beanstalkd-cli put 1000
beanstalkd-cli monitor
```

On peut lancer plusieurs workers en même temps. Attention car un worker peut
occuper un thread à 100% jusqu’à la fin de la file de _job_. Si le nombre de
workers dépasse celui du nombre de cœur la charge sera équitablement répartie.
Si les workers sont sur le même serveur que la page web, l’expérience
utilisateur peut s'en retrouver affectée.

## La classe Worker

Elle n'a pas vocation à être un client __beanstalkd__, mais plutôt à faire une
surcouche sur __Pheanstalk__ (client php pour _beanstalkr).

### La methode __\_\_construct()__

Prends en paramètre le tube sur lequel va travailler le worker. Il est bon
d'avoir un tube par worker.
Les autres paramètres sont pour la connexion au serveur __beanstalkd__.

### getNext()

Utile pour boucler dessus, il se charge de réserver le prochain job et l'affecter
à `Worker->job`
Si aucun job n'est à faire, il attend tranquillement qu'un nouveau job arrive
sans prendre de la charge serveur.

### getData()

Attention, peut renvoyer faux si la signature des données n'est pas validée.
Cette particularité a pour but d'éviter les conflits si une autre application
utilise le même tube.

Sinon, la donnée renvoyée peut être de tous types grâce à une conversion json.

### done()

Supprime le job car celui-ci est terminé

### Attribut __Pheanstalk__

Permets de faire appel à des méthodes plus poussées directement sur le client.

exemple :
```php
$Worker->Pheanstalk->bury($Worker->job);
$Worker->getNext();
```

## Faire un worker

Créez une classe _<Votre_worker>_Worker qui étend __AbstractWorker__ dans
src/Shell/Worker (de préférence).

Ajouter ce worker à votre configuration, exemple :
```php
Configure::write(
    'Beanstalk.workers._Votre_worker_, [
        'path' => '/path/to/_Votre_worker_Worker.php',
        'namespace' => '\App\Shell\Worker', // Doit être identique à votre classe
        'auto_launch' => true, // Se lancera tout seul lors d'une visite sur votre page
        'keep_alive' => true, // Persistant
        'tube' => 'default', // Tube qu'utilisera votre worker
        'params' => null // Si vous voulez envoyer des paramètres supplémentaire
]);
```
