<?php
/**
 * Bootstrap du plugin
 *
 * Inclut l'autoload du vendor, la configuration par défaut ainsi que le
 * lancement automatique des workers
 *
 * @category    Beanstalk
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */

use Cake\Cache\Cache;
use Cake\Core\Configure;
use Beanstalk\Utility\WorkerManager;
use Cake\Datasource\ConnectionManager;
use Cake\Log\Log;
use Cake\Network\Email\Email;
use Cake\Utility\Security;

$default = [
    'Beanstalk' => [
        'host' => '127.0.0.1',
        'port' => 11300,
        'timeout' => null,
        'persistant' => false,
        'autoLaunch' => true,
        'worker_check_ratio' => 10,
        'workers' => []
    ]
];

$conf = Configure::read('Beanstalk');

if (!$conf) {
    Configure::write($default);
}

try {
    if (php_sapi_name() !== "cli"
        && mt_rand(0, 99) < Configure::read('Beanstalk.worker_check_ratio')
        && WorkerManager::connected()
    ) {
        WorkerManager::initialize();
        WorkerManager::autoLaunch();
    }
} catch (Exception $e) {
}

/**
 * Permet le lancement des tests en standalone
 */
if (!Configure::read('App')
    && !defined('APP')
    && is_file(dirname(__DIR__).'/vendor/cakephp/cakephp/config/bootstrap.php')
) {
    define('ROOT', dirname(__DIR__));
    define('APP_DIR', 'src');
    define('APP', ROOT . DS . APP_DIR . DS);
    define('CONFIG', ROOT . DS . 'config' . DS);
    define('WWW_ROOT', ROOT . DS . 'webroot' . DS);
    define('TESTS', ROOT . DS . 'tests' . DS);
    define('TMP', ROOT . DS . 'tmp' . DS);
    define('LOGS', ROOT . DS . 'logs' . DS);
    define('CACHE', TMP . 'cache' . DS);
    define('CAKE_CORE_INCLUDE_PATH', ROOT . DS . 'vendor' . DS . 'cakephp' . DS . 'cakephp');
    define('CORE_PATH', CAKE_CORE_INCLUDE_PATH . DS);
    define('CAKE', CORE_PATH . 'src' . DS);
    require CORE_PATH . 'config' . DS . 'bootstrap.php';
    Configure::write(require 'app.php');
    Cache::setConfig(Configure::consume('Cache'));
    ConnectionManager::setConfig(Configure::consume('Datasources'));
    Email::setConfigTransport(Configure::consume('EmailTransport'));
    Email::setConfig(Configure::consume('Email'));
    Log::setConfig(Configure::consume('Log'));
    Security::setSalt((string)Configure::consume('Security.salt'));
}