<?php
/**
 * Entité de la table beanstalk_workers
 *
 * Défini des champs virtuels
 *
 * @category    Beanstalk
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */

namespace Beanstalk\Model\Entity;

use Cake\ORM\Entity;

/**
 * Classe de l'entité
 *
 * Permet notamment de savoir sous unix si le worker de l'entité est en cours de
 * fonctionnement.
 */
class BeanstalkWorker extends Entity
{
    /**
     * Champs virtuels
     *
     * @var array
     */
    protected $_virtual = ['running'];

    /**
     * Champ virtuel
     *
     * @return bool
     */
    protected function _getRunning()
    {
        // Cas docker, on part du principe que si on obtient une ip, c'est que le container tourne
        if ($this->_properties['hostname'] && $this->_properties['hostname'] !== gethostname()) {
            $ip = gethostbyname($this->_properties['hostname']);
            return $ip !== $this->_properties['hostname'];
        }
        return $this->_properties['pid']
            ? file_exists(sprintf('/proc/%d', $this->_properties['pid']))
            : false;
    }
}
