<?php
/**
 * Table beanstalk_workers
 *
 * Défini la validation
 *
 * @category    Beanstalk
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */

namespace Beanstalk\Model\Table;

use Beanstalk\Model\Entity\BeanstalkWorker;
use Cake\Event\Event;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * Classe de la table
 */
class BeanstalkWorkersTable extends Table
{
    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator)
    {
        $validator
            ->integer('id')
            ->allowEmpty('id', 'create');

        return $validator;
    }

    /**
     * La table est t-elle synchronisée ?
     *
     * @var boolean
     */
    public $sync = false;

    /**
     * A lancer avant un find
     *
     * @param Event $Event
     */
    public function beforeFind(/** @noinspection PhpUnusedParameterInspection */Event $Event)
    {
        if (!$this->sync) {
            $this->sync = true;
            $this->sync();
        }
    }

    /**
     * Synchronise la base de donnée avec Beanstalk
     *
     * @return boolean
     */
    public function sync()
    {
        $hostname = gethostname();
        $query = $this->find()->select(['id', 'hostname', 'pid']);
        /** @var BeanstalkWorker $entity */
        foreach ($query as $entity) {
            if ($entity->get('hostname') !== $hostname && !$entity->get('running')) {
                $this->delete($entity);
            }
        }

        return true;
    }
}
