<?php
/**
 * Worker client cake
 *
 * Permet de récupérer les message du serveur Beanstalk.
 *
 * @category    Beanstalk
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */

namespace Beanstalk\Utility;

use Cake\Core\Configure;
use Beanstalk\lib\Worker;
use Exception;
use Pheanstalk\Connection;

/**
 * Classe du client
 * @property int $pid
 * @property string $name
 */
class CakeWorker extends CakeClient
{
    /**
     * Instance Worker
     *
     * @var Worker
     */
    protected $worker = null;

    /**
     * Constructeur multi-ton
     *
     * @param string $tube
     * @throws Exception
     */
    protected function __construct($tube = 'default')
    {
        $conf = Configure::read('Beanstalk');
        $this->worker = new Worker(
            $tube,
            $conf['host'],
            $conf['port'],
            $conf['timeout'],
            $conf['persistant']
        );
        $conn = $this->worker->Pheanstalk->getConnection();
        if ($conn instanceof Connection && !$conn->isServiceListening()) {
            throw new Exception("Pheanstalk not listening Beanstalkd");
        }

        parent::__construct($tube);
    }

    /**
     * Permet de récupérer l'instance du worker
     *
     * @param string $tube
     * @return Worker
     * @throws Exception
     */
    public static function getWorker($tube = 'default')
    {
        $instance = static::getInstance($tube);
        if (!$instance instanceof CakeWorker) {
            throw new Exception("Instance != CakeEmitter");
        }

        return $instance->worker;
    }

    /**
     * Méthode générique pour obtenir le client
     *
     * @param string $tube
     * @return mixed
     * @throws Exception
     */
    public static function getClient($tube = 'default')
    {
        return static::getWorker($tube);
    }

    /**
     * Appel magique au worker
     * (nécéssite comme premier paramètre, le nom du worker)
     *
     * @param string $name
     * @param array $arguments
     * @return mixed
     * @throws Exception
     */
    public static function __callStatic($name, array $arguments = [])
    {
        $arguments = empty($arguments[0]) ? 'default' : $arguments;
        $worker = $arguments[0];
        unset($arguments[0]);

        return call_user_func_array(
            [static::getWorker($worker), $name],
            $arguments);
    }
}
