<?php
/**
 * Interface du client CakePhp3
 *
 * Assure les méthodes communes d'un client Beanstalk pour CakePhp3
 *
 * @category    Beanstalk
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */

namespace Beanstalk\Utility;

/**
 * Interface du client
 *
 * Assure l'existence des getters pour du multi-ton et la présence des méthodes
 * <b>getCount()</b> et <b>connected()</b>
 */
interface ClientInterface
{
    /**
     * Permet d'obtenir l'instance selon le tube
     *
     * @param string $tube
     * @return CakeClient
     */
    public static function getInstance($tube = 'default');

    /**
     * Méthode générique pour obtenir le client
     *
     * @param string $tube
     * @return CakeClient
     */
    public static function getClient($tube = 'default');

    /**
     * Vérifie la connexion au serveur sans provoquer d'exceptions
     *
     * @return boolean
     */
    public static function connected();

    /**
     * Permet d'obtenir l'instance de Pheanstalk pour un tube donné
     *
     * @param string $tube
     * @return \Pheanstalk\Pheanstalk
     */
    public static function getPheanstalk($tube = 'default');

    /**
     * Donne le nombre de jobs ready d'une tube
     *
     * @param string $tube
     * @return integer
     */
    public static function getCount($tube = 'default');
}
