<?php
/**
 * Beanstalk\lib\Client
 */

namespace Beanstalk\lib;

use Pheanstalk\Pheanstalk;

/**
 * Squelette d'un client Beanstalk
 *
 * Permet la connexion entre le serveur Beanstalk et l'application.
 * Utilise pour cela, la bibliothèque Pheanstalk.
 *
 * @category    Beanstalk
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */
abstract class Client
{
    /**
     * Instance de la library Pheanstalk (permet de communiquer avec Beanstalkd)
     *
     * @var Pheanstalk
     */
    public $Pheanstalk;

    /**
     * Tube actuellement utilisé par Pheanstalk
     * @var string
     */
    public $tube;

    /**
     * Constructeur du Worker
     * Effectue la connexion au serveur
     *
     * @param string $tube
     * @param string $host
     * @param int $port
     * @param null|int $connectTimeout
     * @param bool $connectPersistent
     */
    public function __construct(
        $tube = 'default',
        $host = '127.0.0.1',
        $port = 11300,
        $connectTimeout = null,
        $connectPersistent = false
    ) {
        $this->tube = $tube;

        $this->Pheanstalk = new Pheanstalk(
            $host, $port, $connectTimeout, $connectPersistent);

        $this->Pheanstalk->useTube($tube)->watchOnly($tube);
    }

    /**
     * Détruit proprement Pheanstalk
     */
    public function __destruct()
    {
        $this->Pheanstalk->getConnection()->disconnect();
    }
}
