<?php
/**
 * Émetteur Beanstalk
 *
 * Permet d'envoyer des messages au serveur Beanstalk.
 *
 * @category    Beanstalk
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */

namespace Beanstalk\lib;

/**
 * Classe de l'émetteur
 *
 * <b>emit()</b> permet d'envoyer un message sous n'importe quelle forme au
 * server Beanstalk. Un signature est générée afin d'éviter les conflits avec
 * d'autres outils.
 */
class Emitter extends Client
{
    /**
     * Emet un message structuré afin d'en vérifier la provenance
     *
     * @param string|array $data
     * @return string response id
     */
    public function emit($data)
    {
        $args = func_get_args();
        $json = json_encode($data);

        $formatedData = [
            'data' => $data,
            'key' => sha1($json.strlen($json).'salted')
        ];

        $args[0] = json_encode($formatedData);

        return call_user_func_array(
            array($this->Pheanstalk->useTube($this->tube), 'put'),
            $args);
    }
}
