<?php

namespace Beanstalk\Test\TestCase\Model;

use Beanstalk\Model\Entity\BeanstalkJob;
use Beanstalk\Model\Table\BeanstalkJobsTable;
use Beanstalk\Utility\CakeWorker;
use Cake\Cache\Cache;
use Cake\ORM\TableRegistry;
use Cake\TestSuite\Fixture\FixtureManager;
use Cake\TestSuite\TestCase;

/**
 * Class BeanstalkJobTest
 * @package Beanstalk\Test\Model
 * @property BeanstalkJob $Entity
 */
class BeanstalkJobTest extends TestCase
{
    private $CakeWorker;
    private $Worker;

    public $fixtures = array(
//        'plugin.beanstalk.beanstalk_jobs'
        'app.beanstalk_jobs'
    );

    const TUBE = 'testunit-beanstalkjobs';

    /**
     * @var BeanstalkJobsTable
     */
    private $BeanstalkJobs;

    public function __construct($name = null, array $data = [], $dataName = '')
    {
        parent::__construct($name, $data, $dataName);
        if (empty($this->fixtureManager)) {
            $this->fixtureManager = new FixtureManager;
            $this->fixtureManager->fixturize($this);
            $this->loadFixtures();
        }
        Cache::disable();
    }

    public function setUp()
    {
        parent::setUp();
        $this->BeanstalkJobs = TableRegistry::getTableLocator()->get('Beanstalk.BeanstalkJobs');
        $this->BeanstalkJobs->sync = true; // Evite la tentative de syncronisation de la table avec Beanstalkd
        $this->CakeWorker = CakeWorker::getInstance(self::TUBE);
        $this->Worker = CakeWorker::getWorker(self::TUBE);
        while ($this->CakeWorker->getCount(self::TUBE) > 0) {
            $this->Worker->getNext();
            $this->Worker->done();
        }
    }

    public function tearDown()
    {
        parent::tearDown();
        $this->BeanstalkJobs->deleteAll(['tube' => self::TUBE]);
        TableRegistry::getTableLocator()->clear();
    }

    public function testTable()
    {
        /** @var BeanstalkJob $Entity */
        $Entity = $this->BeanstalkJobs->newEntity();
        $data = [
            'jobid' => 123,
            'tube' => 'testunit-validtubename',
            'data' => 'message',
            'priority' => 1024,
            'delay' => 0,
            'ttr' => 60
        ];
        $this->BeanstalkJobs->patchEntity($Entity, $data);
        $this->assertEquals([], $Entity->getErrors());

        $Entity = $this->BeanstalkJobs->newEntity();
        $data = [
            'jobid' => 123,
            'tube' => 'testunit_invalidtubename',
            'data' => '',
            'priority' => 4294967295 + 10,
            'delay' => -10,
            'ttr' => -10
        ];
        $this->BeanstalkJobs->patchEntity($Entity, $data);

        $expected = [
            'tube' => ['validFormat'],
            'data' => ['_empty'],
            'priority' => ['range'],
            'delay' => ['nonNegativeInteger'],
            'ttr' => ['nonNegativeInteger'],
        ];
        $actual = [];
        foreach ($Entity->getErrors() as $field => $errors) {
            $actual[$field] = array_keys($errors);
        }
        $this->assertEquals($expected, $actual);

        $success = $this->BeanstalkJobs->saveAndEmit([
            'tube' => self::TUBE,
            'data' => [
                'message' => 'Hello world',
                'user_id' => 123
            ]
        ]);
        $this->assertTrue((bool)$success);
        $CakeWorker = CakeWorker::getInstance(self::TUBE);
        $Worker = CakeWorker::getWorker(self::TUBE);
        $actual = $CakeWorker->getCount(self::TUBE);
        $expected = 1;
        $this->assertEquals($expected, $actual);

        $result = $this->BeanstalkJobs->find()->where(['tube' => self::TUBE])->first()->toArray();
        $actual = $result['data'];
        $expected = '{"data":{"message":"Hello world","user_id":123},"key":"3705698b9a02177412219783dc233302f064bd48"}';
        $this->assertEquals($expected, $actual);

        $Worker->getNext();
        $actual = $Worker->getData();
        $Worker->done(); // Le job est supprimé sur Beanstalkd
        $expected = [
            'message' => 'Hello world',
            'user_id' => 123
        ];
        $this->assertEquals($expected, $actual);

        $this->BeanstalkJobs->sync(); // Le job supprimé sur Beanstalkd l'est aussi en base
        $result = $this->BeanstalkJobs->find()->where(['tube' => self::TUBE])->first();
        $this->assertEmpty($result);
    }

    public function testEntity()
    {
        /** @var BeanstalkJob $Entity */
        $Entity = $this->BeanstalkJobs->saveAndEmit([
            'tube' => self::TUBE,
            'data' => [
                'id' => 987,
                'message' => 'Hello world',
                'user_id' => 123
            ]
        ]);
        $data = $Entity->toArray();
        $expected = [
            'data' => [
                'id' => 987,
                'message' => 'Hello world',
                'user_id' => 123,
            ],
            'key' => 'c50bfa22948b30fda897e698aa0676b94dfab9e4'
        ];
        $this->assertEquals($expected, json_decode($data['data'], true));
        $this->assertEquals('ready', $data['state']);
        $this->assertEquals(987, $data['related_id']);
    }
}
