<?php

namespace FileValidator\Test;

use FileValidator\Utility\FileValidator;
use PHPUnit\Framework\TestCase;
use RecursiveDirectoryIterator;
use RecursiveIteratorIterator;

class FileValidatorTest extends TestCase
{
    public function testValidation()
    {
        $path = TESTS.DS.'samples';

        $iterator = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path));
        $files = [];
        foreach ($iterator as $file) {
            if ($file->isDir()){
                continue;
            }
            $filename = $file->getPathname();
            $files[basename($filename)] = FileValidator::check($filename);
        }
        // Note : les fichiers "bad" sont légèrement altérés mais peuvent fonctionner
        //        les fichiers "broken" sont complètement corrompu
        $expected = [
            'Sample-SQL-File-100-Rows.sql' => true,
            'SampleAudio_0.5mb.mp3' => true,
            'SampleCSVFile_2kb.csv' => true,
            'SampleDOCFile_100kb.doc' => true,
            'SamplePPTFile_500kb.ppt' => true,
            'SampleTextFile_10kb.txt' => true,
            'SampleVideo_1280x720_1mb.flv' => true,
            'SampleVideo_1280x720_1mb.mkv' => true,
            'SampleVideo_1280x720_1mb.mp4' => true,
            'SampleVideo_176x144_1mb.3gp' => true,
            'SampleXLSFile_19kb.xls' => true,
            'alert.js' => true,
            'animated.gif' => true,
            'apple-touch-icon-114-precomposed.png' => true,
            'apple-touch-icon-57-precomposed.png' => true,
            'apple-touch-icon-72-precomposed.png' => true,
            'bad_Sample-SQL-File-100-Rows.sql' => true,
            'bad_SampleAudio_0.5mb.mp3' => true,
            'bad_SampleCSVFile_2kb.csv' => true,
            'bad_SampleDOCFile_100kb.doc' => false,
            'bad_SamplePPTFile_500kb.ppt' => false,
            'bad_SampleTextFile_10kb.txt' => true,
            'bad_SampleVideo_1280x720_1mb.flv' => false,
            'bad_SampleVideo_1280x720_1mb.mkv' => false,
            'bad_SampleVideo_1280x720_1mb.mp4' => false,
            'bad_SampleVideo_176x144_1mb.3gp' => false,
            'bad_SampleXLSFile_19kb.xls' => false,
            'bad_alert.js' => true,
            'bad_animated.gif' => false,
            'bad_apple-touch-icon-114-precomposed.png' => false,
            'bad_apple-touch-icon-57-precomposed.png' => false,
            'bad_apple-touch-icon-72-precomposed.png' => false,
            'bad_badheader2_fichier Excel 97-2003.xls' => false,
            'bad_badheader_fichier Excel 97-2003.xls' => false,
            'bad_badheader_sample.ods' => false,
            'bad_badmime_fichier Excel 97-2003.xls' => false,
            'bad_false.jpg' => false,
            'bad_false2.jpg' => false,
            'bad_favicon.ico' => false,
            'bad_fichier Excel 97-2003.xls' => false,
            'bad_fichier PPT.ppt' => false,
            'bad_fichier Word 97-2003.doc' => false,
            'bad_geojson.json' => false,
            'bad_har.json' => false,
            'bad_htaccess.txt' => false,
            'bad_humans.txt' => false,
            'bad_image.svg' => false,
            'bad_images.png' => false,
            'bad_kosovo.svg' => false,
            'bad_placemark.gfs' => false,
            'bad_placemark.gml' => false,
            'bad_placemark.kml' => false,
            'bad_progressive.jpg' => false,
            'bad_reset.css' => true,
            'bad_robots.txt' => false,
            'bad_rss.xml' => false,
            'bad_sample.aac' => true,
            'bad_sample.ac3' => true,
            'bad_sample.ai' => true,
            'bad_sample.aiff' => true,
            'bad_sample.amr' => false,
            'bad_sample.au' => false,
            'bad_sample.avi' => false,
            'bad_sample.bz2' => false,
            'bad_sample.dat' => true,
            'bad_sample.doc' => false,
            'bad_sample.docx' => false,
            'bad_sample.epub' => false,
            'bad_sample.flac' => true,
            'bad_sample.flv' => false,
            'bad_sample.gif' => false,
            'bad_sample.html' => true,
            'bad_sample.jpg' => false,
            'bad_sample.json' => false,
            'bad_sample.m4a' => false,
            'bad_sample.mid' => false,
            'bad_sample.mka' => false,
            'bad_sample.mkv' => false,
            'bad_sample.mov' => false,
            'bad_sample.mp3' => true,
            'bad_sample.mp4' => false,
            'bad_sample.mpg' => false,
            'bad_sample.ods' => false,
            'bad_sample.odt' => false,
            'bad_sample.ogg' => false,
            'bad_sample.otf' => false,
            'bad_sample.pdf' => true,
            'bad_sample.phps' => false,
            'bad_sample.png' => false,
            'bad_sample.ppt' => false,
            'bad_sample.pptx' => false,
            'bad_sample.ra' => false,
            'bad_sample.rar' => false,
            'bad_sample.rtf' => true,
            'bad_sample.swf' => true,
            'bad_sample.tar' => false,
            'bad_sample.tar.gz' => false,
            'bad_sample.tiff' => false,
            'bad_sample.ttf' => true,
            'bad_sample.voc' => false,
            'bad_sample.wav' => false,
            'bad_sample.webm' => false,
            'bad_sample.webp' => false,
            'bad_sample.wma' => false,
            'bad_sample.wmv' => false,
            'bad_sample.woff' => true,
            'bad_sample.xml' => false,
            'bad_sample.xsd' => true,
            'bad_sample.zip' => false,
            'bad_sample2.xml' => false,
            'bad_test.jpg' => false,
            'bad_us_ski_areas_.dbf' => false,
            'bad_us_ski_areas_.shp' => false,
            'bad_us_ski_areas_.shx' => false,
            'badheader2_fichier Excel 97-2003.xls' => false,
            'badheader_fichier Excel 97-2003.xls' => false,
            'badheader_sample.ods' => false,
            'badmime_fichier Excel 97-2003.xls' => false,
            'broken_Sample-SQL-File-100-Rows.sql' => true,              // mauvais
            'broken_SampleAudio_0.5mb.mp3' => false,
            'broken_SampleCSVFile_2kb.csv' => false,
            'broken_SampleDOCFile_100kb.doc' => false,
            'broken_SamplePPTFile_500kb.ppt' => false,
            'broken_SampleTextFile_10kb.txt' => false,
            'broken_SampleVideo_1280x720_1mb.flv' => false,
            'broken_SampleVideo_1280x720_1mb.mkv' => false,
            'broken_SampleVideo_1280x720_1mb.mp4' => false,
            'broken_SampleVideo_176x144_1mb.3gp' => false,
            'broken_SampleXLSFile_19kb.xls' => false,
            'broken_alert.js' => true,                                  // mauvais
            'broken_animated.gif' => false,
            'broken_apple-touch-icon-114-precomposed.png' => false,
            'broken_apple-touch-icon-57-precomposed.png' => false,
            'broken_apple-touch-icon-72-precomposed.png' => false,
            'broken_badheader2_fichier Excel 97-2003.xls' => false,
            'broken_badheader_fichier Excel 97-2003.xls' => false,
            'broken_badheader_sample.ods' => false,
            'broken_badmime_fichier Excel 97-2003.xls' => false,
            'broken_false.jpg' => false,
            'broken_false2.jpg' => false,
            'broken_favicon.ico' => false,
            'broken_fichier Excel 97-2003.xls' => false,
            'broken_fichier PPT.ppt' => false,
            'broken_fichier Word 97-2003.doc' => false,
            'broken_geojson.json' => false,
            'broken_har.json' => false,
            'broken_htaccess.txt' => false,
            'broken_humans.txt' => false,
            'broken_image.svg' => false,
            'broken_images.png' => false,
            'broken_kosovo.svg' => false,
            'broken_placemark.gfs' => false,
            'broken_placemark.gml' => false,
            'broken_placemark.kml' => false,
            'broken_progressive.jpg' => false,
            'broken_reset.css' => false,
            'broken_robots.txt' => false,
            'broken_rss.xml' => false,
            'broken_sample.aac' => false,
            'broken_sample.ac3' => false,
            'broken_sample.ai' => false,
            'broken_sample.aiff' => true,                               // mauvais
            'broken_sample.amr' => false,
            'broken_sample.au' => false,
            'broken_sample.avi' => false,
            'broken_sample.bz2' => false,
            'broken_sample.dat' => true,                                // mauvais
            'broken_sample.doc' => false,
            'broken_sample.docx' => false,
            'broken_sample.epub' => false,
            'broken_sample.flac' => true,                               // mauvais
            'broken_sample.flv' => false,
            'broken_sample.gif' => false,
            'broken_sample.html' => false,
            'broken_sample.jpg' => false,
            'broken_sample.json' => false,
            'broken_sample.m4a' => false,
            'broken_sample.mid' => false,
            'broken_sample.mka' => false,
            'broken_sample.mkv' => false,
            'broken_sample.mov' => false,
            'broken_sample.mp3' => false,
            'broken_sample.mp4' => false,
            'broken_sample.mpg' => false,
            'broken_sample.ods' => false,
            'broken_sample.odt' => false,
            'broken_sample.ogg' => false,
            'broken_sample.otf' => false,
            'broken_sample.pdf' => false,
            'broken_sample.phps' => true,                               // mauvais
            'broken_sample.png' => false,
            'broken_sample.ppt' => false,
            'broken_sample.pptx' => false,
            'broken_sample.ra' => false,
            'broken_sample.rar' => false,
            'broken_sample.rtf' => false,
            'broken_sample.swf' => false,
            'broken_sample.tar' => false,
            'broken_sample.tar.gz' => false,
            'broken_sample.tiff' => false,
            'broken_sample.ttf' => false,
            'broken_sample.voc' => false,
            'broken_sample.wav' => false,
            'broken_sample.webm' => false,
            'broken_sample.webp' => false,
            'broken_sample.wma' => false,
            'broken_sample.wmv' => false,
            'broken_sample.woff' => true,                               // mauvais
            'broken_sample.xml' => false,
            'broken_sample.xsd' => true,                                // mauvais
            'broken_sample.zip' => false,
            'broken_test.jpg' => false,
            'broken_us_ski_areas_.dbf' => true,                         // mauvais
            'broken_us_ski_areas_.shp' => false,
            'broken_us_ski_areas_.shx' => false,
            'favicon.ico' => true,
            'fichier Excel 97-2003.xls' => true,
            'fichier PPT.ppt' => true,
            'fichier Word 97-2003.doc' => true,
            'geojson.json' => true,
            'har.json' => true,
            'htaccess.txt' => true,
            'humans.txt' => true,
            'image.svg' => true,
            'images.png' => true,
            'kosovo.svg' => true,
            'placemark.gfs' => true,
            'placemark.gml' => true,
            'placemark.kml' => true,
            'progressive.jpg' => true,
            'reset.css' => true,
            'robots.txt' => true,
            'rss.xml' => true,
            'sample.aac' => true,
            'sample.ac3' => true,
            'sample.ai' => true,
            'sample.aiff' => true,
            'sample.amr' => true,
            'sample.au' => true,
            'sample.avi' => true,
            'sample.bz2' => true,
            'sample.dat' => true,
            'sample.doc' => true,
            'sample.docx' => true,
            'sample.epub' => true,
            'sample.flac' => true,
            'sample.flv' => true,
            'sample.gif' => true,
            'sample.html' => true,
            'sample.jpg' => true,
            'sample.json' => true,
            'sample.m4a' => true,
            'sample.mid' => true,
            'sample.mka' => true,
            'sample.mkv' => true,
            'sample.mov' => true,
            'sample.mp3' => true,
            'sample.mp4' => true,
            'sample.mpg' => true,
            'sample.ods' => true,
            'sample.odt' => true,
            'sample.ogg' => true,
            'sample.otf' => true,
            'sample.pdf' => true,
            'sample.phps' => true,
            'sample.png' => true,
            'sample.ppt' => true,
            'sample.pptx' => true,
            'sample.ra' => true,
            'sample.rar' => true,
            'sample.rtf' => true,
            'sample.swf' => true,
            'sample.tar' => true,
            'sample.tar.gz' => true,
            'sample.tiff' => true,
            'sample.ttf' => true,
            'sample.voc' => true,
            'sample.wav' => true,
            'sample.webm' => true,
            'sample.webp' => true,
            'sample.wma' => true,
            'sample.wmv' => true,
            'sample.woff' => true,
            'sample.xml' => true,
            'sample.xsd' => true,
            'sample.zip' => true,
            'test.jpg' => true,
            'us_ski_areas_.dbf' => true,
            'us_ski_areas_.shp' => true,
            'us_ski_areas_.shx' => true,
        ];
        $this->assertEquals($expected, $files);
    }
}