<?php
/**
 * Ce fichier permet de savoir si l'utilisateur est connecté et si sa session
 * n'a pas expiré, sans la prolonger
 */

require dirname(__DIR__) . '/vendor/autoload.php';
require realpath(dirname(__DIR__).'/config/bootstrap.php');

use Cake\Http\Session;
use Cake\Core\Configure;

/**
 * Surcharge de la classe de Session
 */
class ConnSession extends Session
{
    /**
     * @override Retire le Session::write('Config.time')
     *
     * Returns true if the session is no longer valid because the last time it was
     * accessed was after the configured timeout.
     *
     * @return bool
     */
    protected function _timedOut()
    {
        $time = $this->read('Config.time');
        $result = false;

        $checkTime = $time !== null && $this->_lifetime > 0;
        if ($checkTime && (time() - $time > $this->_lifetime)) {
            $result = true;
        }

        return $result;
    }

    /**
     * Rend public _timedOut
     *
     * @return bool
     */
    public function timedOut()
    {
        return $this->_timedOut();
    }

    /**
     * Temps restant avant timeout
     *
     * @return int
     */
    public function timeLeft(): int
    {
        return (int)$this->_lifetime - (time() - $this->read('Config.time'));
    }
}

$config = Configure::read('Session') + [
    'cookie' => 'CAKEPHP',
	'ini' => [
		'session.use_trans_sid' => (int) 0,
		'session.cookie_secure' => (int) 1,
		'session.name' => 'CAKEPHP',
		'session.cookie_httponly' => (int) 1
	],
	'defaults' => 'php',
	'cookiePath' => '/'
];

$session = new ConnSession($config);

$isAdmin = !empty($_GET['admin']);

if ($session->timedOut()) {
    $session->destroy();
}

header('Content-Type: application/json');
echo json_encode([
    'connected' => !$session->timedOut() && ($isAdmin ? $session->read('Admin.tech') : $session->read('Auth')),
    'remaining' => $session->timeLeft()
]);