/* global jQuery, $ */

/**
 * Notifications
 */
class AsalaeNotifications
{
    constructor(config) {
        this.session = null;
        this.data = null;
        this.count = 0;
        this.element = config.element;
        this.elementUl = config.elementUl;
        this.favicon = new Favico({
            type : 'rectangle',
            animation:'none',
            bgColor : '#f05922',
            fontFamily : 'Arial'
        });
        this.ajax = {
            deleteNotification: config.ajax.deleteNotification
        };
        // Arrete l'animation de la cloche
        $('#container-number-notification').hover(function() {
            $(this).find('i.fa-bell').removeClass('faa-shake');
        }).click(function(event) {
            $(config.element).children().each(function() {
                $(this).slideUp(400, function() {$(this).remove();});
            })
        });
    }

    connect(user_id, url) {
        var that = this;
        ab.connect(
            url,
            function(session) {
                that.session = session;
                that.user_id = user_id;
                session.subscribe('user_'+user_id, function(topic, data) {
                    that.updateCount(1);
                    if (that.count <= 20) {
                        that.append(data.message);
                    }
                    if (that.count === 20) {
                        alert("L'affichage des notifications a été désactivé afin de préserver les performances de votre navigateur. Afin de voir les notifications suivantes, il vous faudra lire les notifications actuelles et recharger la page.")
                    }
                });
                session.subscribe('close_user_'+user_id, function(topic, data) {
                    $(data.target).slideToggle(400, function() {$(data.target).remove();});
                    $(data.targetLi).slideToggle(400, function() {$(data.targetLi).remove();});
                    that.updateCount(-1);
                });
                session.subscribe('ping', function(topic, data) {
                    session.publish('pong', user_id);
                });
                $(that.element).trigger('connected.autobahn');
            },
            function() {
            },
            {'skipSubprotocolCheck': true}
        );

        return this;
    }

    updateCount(amount) {
        var containerNumberNotification = $('#container-number-notification'),
            notificationBell = containerNumberNotification.find('i.fa-bell'), 
            numberNotification = $('#number-notification'); 
        if (amount === 0) {
            notificationBell.removeClass("faa-shake");
            return;
        } else if (amount > 0 && !containerNumberNotification.hasClass('active')) {
            notificationBell.addClass("faa-shake");
        }
        this.count += amount;
        this.favicon.badge(this.count);
        numberNotification.text(this.count).addClass("faa-horizontal");
        setTimeout(function() {
            numberNotification.removeClass('faa-horizontal');
        }, 1000);

        return this;
    }

    insert(data, deleteByAjax = true) {
        $(this.element).empty();

        this.updateCount(data.length);

        for (var key in data) {
            this.append(data[key], deleteByAjax);
        }

        return this;
    }

    append(data, deleteByAjax = true) {
        var div, button, that = this, action, li;

        div = $('<div></div>')
            .addClass('alert')
            .addClass('alert-dismissible')
            .addClass(data.css_class)
            .attr('id', 'notification-'+data.id)
            .attr('role', 'alert');

        button = $('<button></button>')
            .addClass('close')
            .attr('type', 'button')
            .attr('aria-label', 'Close')
            .append($('<span></span>').append('&times;'));

        div.append(button)
            .append(data.text);

        li = $('<li></li>')
            .addClass('navbar-link')
            .addClass('alert')
            .addClass(data.css_class)
            .attr('id', 'notification-li-'+data.id)
            .append($('<a></a>')
                .attr('href', 'javascript:')
                .attr('role', 'button')
                .append(data.text));

        action = function(event) {
            event.stopPropagation();
            // Evite les clics succéssifs
            div.unbind('click');
            li.unbind('click');
            // On envoi l'evenement de cloture aux autres fenètres
            if (that.session !== null) {
                that.session.publish('close_user_'+that.user_id, {
                    target: '#notification-'+data.id,
                    targetLi: '#notification-li-'+data.id
                });
            } else {
                $(div).slideToggle(400, function() {$(div).remove();});
                $(li).slideToggle(400, function() {$(li).remove();});
                that.updateCount(-1);
            }
            // Ajax pour suppression en base
            if (deleteByAjax) {
                $.ajax({
                    method: 'POST',
                    url: that.ajax.deleteNotification,
                    data: data,
                    error: function() {
                        console.error("Notification delete failed");
                    }
                });
            }
        };
        div.click(action);
        li.click(action);

        setTimeout(function() {
            div.slideUp(400, function() {
                $(this).remove();
            });
        }, 5000);

        $(this.element).append(div);
        $(this.elementUl).append(li);

        return this;
    }
}

class AsalaeFilter
{
    /**
     * Ajoute un element de filtre
     *
     * @param {string|jQuery} element
     * @param {string} template
     */
    static append(element, template) {
        var filter = $(template),
            container = $('<div>')
                .append(
                    $('<button></button>')
                        .addClass('close')
                        .attr('type', 'button')
                        .attr('aria-label', __('Retirer le filtre de recherche'))
                        .append('<span>×</span>')
                        .click(function() {
                            $(this).parent().slideUp(400, function() {$(this).remove();});
                        })
                )
        ;

        // Remplace l'id et le name dans template
        filter.find('select, input, textarea').each(function() {
            var base_id = $(this).attr('id'),
                id = base_id + '_0',
                base_name = $(this).attr('name'),
                label = filter.find('label[for="'+base_id+'"]'),
                name,
                i = 0
            ;

            while (id && $('#'+id).length) {
                i++;
                id = base_id + '_' + i;
            }

            if (label) {
                label.attr('for', id);
            }
            $(this).attr('id', id);

            if (!base_name) {
                return;
            }

            i = 0;
            name = AsalaeFilter._newName(base_name, i);
            while (name && $('[name="'+name+'"]').length) {
                i++;
                name = AsalaeFilter._newName(base_name, i);
            }
            $(this).attr('name', name);
        });

        container.append(filter);
        $(element).append(container);

        // Bouton Rechercher
        if (!$(element).parent().find('button[type="submit"]').length) {
            let btn = $('<button></button>')
                .attr('type', 'button')
                .addClass('btn')
                .addClass('btn-default')
                .html('<i class="fa fa-undo" aria-hidden="true"></i> '+__('Réinitialiser les filtres'))
                .click(AsalaeGlobal.confirmRemoveFilters);
            let saveBtn = $('<button></button>')
                    .attr('type', 'button')
                    .addClass('btn')
                    .addClass('btn-success')
                    .addClass('newSave')
                    .addClass('save')
                    .html('<i class="fa fa-plus-circle" aria-hidden="true"></i> '+__("Nouvelle sauvegarde"))
                    .click(AsalaeGlobal.saveFilters);
            $(element).parent().append(
                $('<div></div>')
                    .addClass('submit-btn-filter-container')
                    .addClass('modal-footer')
                    .append(btn)
                    .append(saveBtn)
                    .append(
                        $('<button></button>')
                            .attr('type', 'submit')
                            .addClass('btn')
                            .addClass('btn-primary')
                            .html('<i class="fa fa-filter" aria-hidden="true"></i> '+__('Filtrer'))
                    )
            );
        }
    }

    /**
     * Transforme un nom pour y prefixer une valeur
     *
     * ex: myname => myname[0]
     * ex: another_name[256][] => another_name[0][256][]
     *
     * @param {string} name
     * @param {number} index
     * @returns {string}
     */
    static _newName(name, index) {
        var first_brace = name.indexOf('['),
            base_name = first_brace ? name.substr(0, first_brace) : null,
            suffix = base_name ? name.substr(first_brace, name.length) : '';
        return (base_name ? base_name : name) + '[' + index + ']' + suffix;
    }

    /**
     * Evenements liés aux bouttons de la modale "Liste des sauvegardes de recherche"
     */
    static modalListButtons(urls, controller, action) {
        var div = $('.modal-list-buttons');
        div.find('button.delete').off('click.filter').on('click.filter', function() {
            var modal = $(this).closest('.modal'),
                select = modal.find('select.filterSelect'),
                value = select.val(),
                form = $('form.ModalFilters')
            ;
            if (value && !isNaN(value) && value >= 0
                && typeof urls.delete !== 'undefined'
                && confirm(__("Cette action supprimera la sauvegarde de façon définitive. Voulez-vous continuer ?"))
            ) {
                $('html').addClass('ajax-loading');
                $.ajax({
                    url: urls.delete + '/' + value,
                    success: function(content, textStatus, jqXHR) {
                        if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
                            select.find('option[value="'+value+'"]').remove();
                            select.val('-1');
                            setTimeout(function() {
                                $('select.filterSelect').shake(
                                    {duration: 1000, distance: 2}
                                ).parent()
                                    .css('background-color', '#5cb85c')
                                    .animate({'background-color': 'transparent'})
                                ;
                            }, 400);
                        } else {
                            this.error();
                        }
                    },
                    error: function() {
                        alert(PHP.messages.genericError);
                    },
                    complete: function() {
                        $('html').removeClass('ajax-loading');
                    }
                });
            }
        });
        div.find('button.overwrite').off('click.filter').on('click.filter', function() {
            var modal = $(this).closest('.modal'),
                value = modal.find('select.filterSelect').val(),
                form = $('form.ModalFilters')
            ;
            if (value && !isNaN(value) && value >= 0
                && typeof urls.overwrite !== 'undefined'
                && confirm(__("Cette action va modifier la sauvegarde de façon définitive. Voulez-vous continuer ?"))
            ) {
                let modal = $(this).closest('.modal'),
                    data = AsalaeFilter.getFiltersData(form)
                ;
                $('html').addClass('ajax-loading');
                $.ajax({
                    url: urls.overwrite + '/' + value,
                    headers: {Accept: 'application/json'},
                    method: 'POST',
                    data: data,
                    success: function(content, textStatus, jqXHR) {
                        if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
                            let newUrl = urls.overwrite.replace(/\/[\w\-]+\/[\w\-]+(\?.*)?$/, '/'+controller+'/'+action+'?'+data);
                            $('ul.filtersSelectWindow li.savedFilter[data-id="'+content.id+'"] a')
                                .attr('href', newUrl)
                            ;
                            setTimeout(function() {
                                form.find('button.overwrite').shake({duration: 1000, distance: 2}).disable();
                                form.find('.filter-container > div')
                                    .css('background-color', '#5cb85c')
                                    .animate({'background-color': 'transparent'})
                                    .find('input')
                                    .css('background-color', 'transparent')
                                ;
                            }, 400);
                            setTimeout(function() {
                                $('select.filterSelect').shake(
                                    {duration: 1000, distance: 2}
                                ).parent()
                                    .css('background-color', '#5cb85c')
                                    .animate({'background-color': 'transparent'})
                                ;
                            }, 400);
                        } else {
                            this.error();
                        }
                    },
                    error: function() {
                        alert(PHP.messages.genericError);
                    },
                    complete: function() {
                        $('html').removeClass('ajax-loading');
                    }
                });
            }
        });
        div.find('button.load').off('click.filter').on('click.filter', function() {
            var modal = $(this).closest('.modal'),
                value = modal.find('select.filterSelect').val(),
                form = $('form.ModalFilters')
            ;
            if (value !== '-1') {
                $('html').addClass('ajax-loading');
                $.ajax({
                    url: urls.load + '/' + value,
                    type: 'POST',
                    data: {
                        helper: form.find('input[name="ConfigFilter"]').val()
                    },
                    success: function(content) {
                        form.html(content);
                        form.find('button.overwrite').disable();
                        form.find('button.load').disable();
                        setTimeout(function() {
                            form.find('button.load').shake({duration: 1000, distance: 2});
                            form.find('button.delete').enable();
                            form.find('.filter-container > div')
                                .css('background-color', '#5cb85c')
                                .animate({'background-color': 'transparent'})
                                .find('input')
                                .css('background-color', 'transparent')
                            ;
                        }, 400);
                    },
                    error: function() {
                        alert("Une erreur a eu lieu");
                    },
                    complete: function() {
                        $('html').removeClass('ajax-loading');
                    }
                });
            }
        });
    }

    /**
     * Boutons de la modale d'ajout d'une sauvegarde
     * @param {string} url
     * @param {string} controller
     * @param {string} action
     */
    static modalAddSaveButtons(url, controller, action) {
        var div = $('.modal-save-buttons'),
            input = div.closest('.modal-content').find('input.filterNewSave')
        ;
        input.off('keypress.filter').on('keypress.filter', function(event) {
            if (event.originalEvent.keyCode === 13) { // Enter
                $(this).closest('.modal-content').find('button.btn-primary').click();
            }
        });
        div.find('button.btn-primary').off('click.filter').on('click.filter', function() {
            var value = input.val(),
                data = AsalaeFilter.getFiltersData($('form.ModalFilters')),
                parentDiv = input.parent()
            ;
            if (!data) {
                alert(__("Impossible de sauvegarder en cas de formulaire vide ou en erreur!"));
            } else if (!value) {
                let nf = $('<form>');
                parentDiv.append(nf.append(input));
                input.prop('required', true);
                nf.get(0).reportValidity();
                input.prop('required', false);
                nf.parent().append(input);
                nf.remove();
                input.focus();
            } else if (url) {
                let modal = $(this).closest('.modal');
                $('html').addClass('ajax-loading');
                $.ajax({
                    url: url,
                    headers: {Accept: 'application/json'},
                    method: 'POST',
                    data: data+'&savename='+value+'&controller='+controller+'&action='+action,
                    success: function(content, textStatus, jqXHR) {
                        parentDiv.removeClass('has-error')
                            .find('.error-message').remove()
                        ;
                        if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
                            input.val('');
                            modal.modal("hide");
                            $('.modal-list-buttons').closest('.modal-body')
                                .find('select.filterSelect')
                                .append('<option value="'+content.id+'">'+content.name+'</option>')
                                .val(content.id)
                            ;
                            $('[name=SaveFilterSelect]').val(content.id);
                            let newUrl = url.replace(/\/[\w\-]+\/[\w\-]+(\?.*)?$/, '/'+controller+'/'+action+'?'+data);
                            if (/SaveFilterSelect=($|&|-1)/.test(newUrl)) {
                                newUrl = newUrl.replace("SaveFilterSelect=-1", "SaveFilterSelect=");
                                newUrl = newUrl.replace("SaveFilterSelect=", "SaveFilterSelect="+content.id);
                            }
                            $('ul.filtersSelectWindow').append(
                                $('<li class="savedFilter" data-id="'+content.id+'"></li>').append(
                                    $('<a class="navbar-link"><i class="fa fa-filter" aria-hidden="true"></i> </a>')
                                        .append(content.name)
                                        .attr('href', newUrl)
                                        .click(AsalaeGlobal.interceptedLinkhandler)
                                )
                            );
                            setTimeout(function() {
                                $('.modal-content button.listSaves').shake(
                                    {duration: 1000, distance: 2}
                                ).parent()
                                    .css('background-color', '#5cb85c')
                                    .animate({'background-color': 'transparent'})
                                ;
                            }, 400);
                        } else if (jqXHR.getResponseHeader('X-Asalae-Success') === 'false') {
                            let error = $('<span class="help-block error-message"></span>');
                            for (let key in content.name) {
                                error.append(content.name[key]);
                            }
                            parentDiv.addClass('has-error').append(error);
                        } else {
                            this.error();
                        }
                    },
                    error: function() {
                        alert(PHP.messages.genericError);
                    },
                    complete: function() {
                        $('html').removeClass('ajax-loading');
                    }
                });
            }
        });
    }

    /**
     * Permet de vérifier le formulaire et d'obtenir ses filtres
     * @param {Element|jQuery} form
     * @returns {string|bool}
     */
    static getFiltersData(form) {
        if ($(form).get(0).checkValidity()) {
            let clone = $(form).clone();
            clone.find('input.configuration-filter').remove();
            clone.find('select.filterSelect').remove();
            clone.find('[name=SaveFilterSelect]').remove();
            clone.find('select[name="filter"]').remove();
            return clone.serialize();
        } else {
            $(form).get(0).reportValidity();
            return false;
        }
    }

    /**
     * Ajoute selon la valeur de select dans le container, le filtre voulu à
     * l'aider du templates[value]
     * @param {string|jQuery} select
     * @param {string|jQuery} container
     * @param {Array} templates
     */
    static selectHandler(select, container, templates) {
        var form = $(select).closest('form');
        $(select).off('change.filter').on('change.filter', function() {
            var val = $(this).val();
            if (!val) {
                return;
            }
            $(this).val('');
            AsalaeFilter.append(container, templates[val]);
        });
        if (form.find('select.filterSelect').val() > 0) {
            form.find('button.delete').enable();
        }
    }

    /**
     * onchange du formulaire pour "griser" les boutons au besoin
     */
    static formChangedHandler() {
        var selectedSave = $('select.filterSelect').val(),
            clone = $(this).clone(),
            listButtons = $(this).find('.modal-list-buttons'),
            filterData
        ;
        clone.find('input.configuration-filter').remove();
        clone.find('select.filterSelect').remove();
        clone.find('[name=SaveFilterSelect]').remove();
        clone.find('select[name="filter"]').remove();
        filterData = clone.serialize();

        if (selectedSave && selectedSave > 0) {
            listButtons.find('button.delete').enable();
            listButtons.find('button.load').enable();
            if (filterData && this.checkValidity()) {
                listButtons.find('button.overwrite').enable();
            } else {
                listButtons.find('button.overwrite').disable();
            }
        } else {
            listButtons.find('button.delete').disable();
            listButtons.find('button.load').disable();
            listButtons.find('button.overwrite').disable();
        }
    }
}

/**
 * Namespace foure-tout pour les fonctions globales
 */
var AsalaeGlobal = {};

/**
 * Complexité du mot de passe
 *
 * @param {string} password
 * @returns {number} score de complexitée
 */
AsalaeGlobal.passwordScore = function(password) {
    var mult, chars = 0;
    if (/[a-z]/.test(password)) {
        chars += 26;
    }
    if (/[A-Z]/.test(password)) {
        chars += 26;
    }
    if (/[0-9]/.test(password)) {
        chars += 10;
    }
    if (/[ ! #$*%]/.test(password)) {
        chars += 8;
    }
    if (/[^a-zA-Z0-9 ! #$*%]/.test(password)) {
        chars += 20;
    }

    return password.length * Math.log2(chars);
}

/**
 * Initialise la pagination javascript
 *
 * @param {string|jQuery} section
 */
AsalaeGlobal.paginationAjax = function(section) {
   $(section).find('ul.pagination a, table > thead > tr > th a').click(function(event) {
        event.preventDefault();
        if ($(this).parent().hasClass('disabled')
            || $(this).parent().hasClass('active')
            || $(this).attr('href').indexOf('javascript') === 0
        ) {
            return;
        }
        $(this).disable().closest('.pagination').find('a').disable().parent().addClass('disabled');
       AsalaeGlobal.updatePaginationAjax(section, $(this).attr('href'));
   });
}

var updatingPaginationAjax = 0,
    queuedPaginationAjax = 0;
/**
 * Met à jour la section liée à la pagination
 *
 * @param {string|jQuery} section
 * @param {string} url
 */
AsalaeGlobal.updatePaginationAjax = function(section, url = window.location.href) {
    var initialHeight = $(section).height(),
        block = $('<div></div>').addClass('transition-container')
            .css('min-height', initialHeight),
        loading = $('<div class="text-center loading-div"><i aria-hidden="true" class="fa fa-4x fa-spinner faa-spin animated"></i></div>'),
        tables = $(section).find('table').map(function() { return $(this).attr('data-table-uid') }).toArray(),
        tablesObj = tables.length ? tables.map((table) => TableGenerator.instance[table]) : null
    ;

    // Empeche le déclanchement trop rapide des mises à jour (max 1 mise à jour par seconde)
    // Si une mise à jour est commandé, les précédentes sont annulés au profit de la nouvelle
    if (updatingPaginationAjax) {
        clearTimeout(queuedPaginationAjax);
        queuedPaginationAjax = setTimeout(function() {
            AsalaeGlobal.updatePaginationAjax(section, url);
        }, 1000);
        return;
    }
    updatingPaginationAjax = true;

    $(section).after(block).css('min-height', initialHeight);
    block.append($(section));
    $(section).find('[id]').each(function() {
        $(this).attr('id', $(this).attr('id')+'-transition');
    }).find('table').after(loading);
    $('html').addClass('ajax-loading');
    AsalaeLoading.start();
    $.ajax({
        url: url,
        success: function(content) {
            var id = $(section).attr('id'),
                html = $(content).hide();

            $(section).attr('id', id+'-transition').after(html).fadeOut(400, function() {
                $(this).remove();
            });
            html.fadeIn(400, function() {
                $(window).trigger('resize');
            });
            AsalaeGlobal.paginationAjax('#' + id);
            AsalaeGlobal.initContentScope();
            $(window).trigger('resize');
            window.history.pushState({}, '', url);
            $(window).trigger('history.change');
            let newTables = html.find('table[data-table-uid]').map(function() { return $(this).attr('data-table-uid'); }).toArray(),
                newTablesObj = newTables.length ? newTables.map((table) => TableGenerator.instance[table]) : null
            ;
            for (let i in tablesObj) {
                let newObj = newTablesObj.shift();
                eval($(newObj.table).attr('data-for')+' = tablesObj[i]');
                tablesObj[i].table = $(newObj.table).attr('data-table-uid', tablesObj[i].instanceIndex);
                tablesObj[i].data = newObj.data;
                tablesObj[i]._params = newObj._params;
                TableGenerator.instance.splice(newObj.instanceIndex, 1);
                TableGenerator.lastInstance = tablesObj[i];
                tablesObj[i].applyCookies();
                tablesObj[i].generateAll();
            }
        },
        error: function(jqXHR, textStatus, errorThrown) {
            if (jqXHR.status === 404 && url.indexOf('page=')) {
                AsalaeGlobal.updatePaginationAjax(
                    section,
                    url.replace(/page=[0-9]+&?/, '').replace(/[?&]+$/, ''));
            } else {
                alert('Une erreur a eu lieu');
            }
        },
        complete: function() {
            loading.remove();
            AsalaeLoading.stop();
            setTimeout(function() {
                block.after(block.children()).remove();
                updatingPaginationAjax = false;
            }, 400);
        }
    });
}

/**
 * Permet d'obtenir la valeur d'une variable passée en GET
 *
 * @param {string} name
 * @param {string} url
 * @returns {string} valeur du param
 */
AsalaeGlobal.getParameterByName = function(name, url = window.location.href) {
    var regex = new RegExp("[?&]" + name.replace(/[\[\]]/g, "\\$&") + "(=([^&#]*)|&|#|$)"),
        results = regex.exec(url);
    if (!results) {
        return null;
    }
    if (!results[2]) {
        return '';
    }
    return decodeURIComponent(results[2].replace(/\+/g, " "));
}

/**
 * Permet de décoder une chaine encodé par la fonction PHP urlencode()
 *
 * @param {string} str url à décoder
 * @returns {string} url décodé
 */
AsalaeGlobal.urldecode = function (str) {
    return decodeURIComponent((str + '')
        .replace(/%(?![\da-f]{2})/gi, '%25')
        .replace(/\+/g, '%20'))
}

/**
 * Supprime les filtres actifs (boutton Supprimer les filtres)
 */
AsalaeGlobal.confirmRemoveFilters = function() {
    $('form.ModalFilters').empty().submit();
}

/**
 * Sauvegarde les filtres actifs
 */
AsalaeGlobal.saveFilters = function() {
    var modal = $(this).closest('.modal'),
        form = modal.find('form'),
        formData = AsalaeFilter.getFiltersData(form)
    ;
    if (formData === "") {
        let select = modal.find('select[name=filter]');
        select.prop('required', true);
        form.get(0).reportValidity();
        select.prop('required', false);
    } else if (formData) {
        let uid = modal.attr('data-uid'),
            saveModal = $('.filter-save-modal[data-uid='+uid+']')
        ;
        saveModal.modal('show');
        setTimeout(function() { saveModal.find('input').first().focus() }, 500);
    }
}

/**
 * Lance la suppression de toutes les notifications
 */
AsalaeGlobal.deleteAllNotifications = function(event) {
    event.stopPropagation();
    $('ul#notifications-ul-container li.alert a').click();
}

/**
 * Redirige vers la page indiqué dans l'input crée lors du clic sur les "..."
 */
AsalaeGlobal.paginationSelectInput = function() {
    $('.pagination li.ellipsis a[href="javascript:"]').off('click').click(function() {
        var max = parseInt($(this).closest('.pagination').find('li:not(.next) a').last().text().replace(/\s/, ''), 10),
            url = $($(this).closest('.pagination').find('li:not(.active):not(.ellipsis) a')[2]);
        $(this).css({position: 'relative'}).append(
            $('<input/>')
                .css({position: 'absolute', top: '-30px', left: '0', width: '3em'})
                .attr('min', 1)
                .attr('max', max)
                .attr('step', 1)
                .attr('type', 'text')
                .attr('id', 'pagination-select-input')
                .bind('blur keydown', function(event) {
                    var pageNum = url.text().trim(),
                        value = parseInt($(this).val(), 10);

                    if (event.type === 'blur'
                        || event.keyCode === 13
                        || event.keyCode === 10
                    ) {
                        if (!AsalaeGlobal.is_numeric($(this).val()) === false
                            && value > 1
                            && value < max
                        ) {
                            url.attr('href', url.attr('href').replace('page='+pageNum, 'page='+$(this).val())).click();
                        }
                        $('input#pagination-select-input').remove();
                    }
                }));
        $('#pagination-select-input').focus();
    });
}

/**
 * Colore en rouge les onglets qui comportent une erreur
 * et selectionne le premier onglet en erreur
 */
AsalaeGlobal.colorTabsInError = function() {
    var firstErrors = [];
    $('div.ui-tabs-panel:has(.has-error)').each(function() {
        var targetId = $(this).attr('id'),
            tab = $('a.ui-tabs-anchor[href="#'+targetId+'"]').first(),
            firstErrorId;

        tab.parent().addClass('bg-danger');
        firstErrorId = $(tab).closest('ul')
            .find('> li.bg-danger > a.ui-tabs-anchor')
            .first()
            .attr('id');

        if (firstErrors.indexOf(firstErrorId) === -1) {
            firstErrors.push(firstErrorId);
        }
    });
    for (let i=0; i<firstErrors.length; i++) {
        $('#'+firstErrors[i]).click();
    }
}

/**
 * Attribue des événements lors d'un appel ajax
 * @type window.XMLHttpRequest|Window.XMLHttpRequest
 */
var OldXHR = window.XMLHttpRequest;
AsalaeGlobal.NewXHR = function() {
    var RealXHR = new OldXHR();
    RealXHR.addEventListener("readystatechange", function() {
        var eventName;
        switch (RealXHR.readyState) {
            case 1:
                eventName = 'ajax_connection';
                break;
            case 2:
                eventName = 'ajax_send';
                break;
            case 3:
                eventName = 'ajax_processing';
                break;
            case 4:
                eventName = 'ajax_complete';
                break;
        }
        $('html').trigger(eventName);
    }, false);
    return RealXHR;
}
window.XMLHttpRequest = AsalaeGlobal.NewXHR;

/**
 * Affiche une animation lors de l'attente d'une réponse ajax
 * @param {jQuery} modal
 */
AsalaeGlobal.addWaitingAjaxResponse = function(modal) {
    $(modal).fadeTo(400, 0.8).css('cursor', 'wait', 'important')
        .find('button').css('cursor', 'wait', 'important').disable();
}

/**
 * Retire l'animation d'attente de réponse ajax
 * @param {jQuery} modal
 */
AsalaeGlobal.removeWaitingAjaxResponse = function(modal) {
    $(modal).fadeTo(400, 1).css('cursor', '')
        .find('button').css('cursor', '').enable();
}

/**
 * Vérifi qu'une chaine ne contiens que des chiffres
 * @param {string|number} str
 * @returns {boolean}
 */
AsalaeGlobal.is_numeric = function(str){
    return /^\d+$/.test(''+str);
}

var miniWindowCount = 0;
/**
 * Permet la réduction des modals
 */
AsalaeGlobal.minifiableModal = function() {
    var minifierIcon = $('<i aria-hidden="true" class="fa fa-window-minimize float-right soften"></i>'),
        maximizerIcon = $('<i aria-hidden="true" class="fa fa-window-maximize float-right soften"></i>'),
        h4 = $(this).find('div.modal-header > h4'),
        title = h4.text(),
        modal = $(this),
        num,
        _minifyModal,
        windowLabel = modal.attr('data-window-label'),
        _maximizeModal,
        _removeMaximized;
    modal.attr("data-backdrop", "static");

    _removeMaximized = function() {
        modal.find('.maximized-modal.modal-dialog').removeClass('maximized-modal')
            .find('>.modal-content').css('height', '')
            .closest('.modal').css('overflow', 'hidden')
            .find('iframe').css('height', '');
    };
    _maximizeModal = function() {
        var dialog = $(this).closest('.modal-dialog'),
            content = dialog.find('>.modal-content'),
            iframe = content.find('iframe');
        dialog.toggleClass('maximized-modal');
        if (dialog.hasClass('maximized-modal')) {
            content.attr('data-width', content.css('width'))
                .attr('data-height', content.css('height'))
                .css('width', '')
                .css('height', '')
                .height($(window).height() -50)
                .closest('.modal').css('overflow', 'hidden');
            iframe.css('height', $(window).height() -50);
            modal.find('button.close').off('click', _removeMaximized).click(_removeMaximized);
        } else {
            content.css('width', content.attr('data-width'))
                .css('height', content.attr('data-height'));
            _removeMaximized();
        }
    };
    _minifyModal = function() {
        var modalClone = modal.clone(),
            restore = $('<i aria-hidden="true" class="fa fa-window-restore float-right slide-icon"></i>'),
            destroy = $('<i aria-hidden="true" class="fa fa-times float-right slide-icon"></i>'),
            label = title.length > 18 ? title.substr(0, 15)+'...' : title,
            h4 = $('<h4></h4>').append(label).append(destroy).append(restore).attr('title', title),
            miniWindow = $('<div class="mini-window"></div>').append(h4),
            top = 110,
            tops = [],
            num;
        if (!(num = minifierIcon.attr('data-num'))) {
            miniWindowCount++;
            num = miniWindowCount;
        }
        restore.append(num);

        modal.css('overflow', 'hidden');// Evite l'affichage du scroll durant l'animation

        destroy.click(function() {
            miniWindow.toggle('puff', function () {
                $(this).remove();
            })
        })

        // On calcul la position vertical de la vignette par rapport aux autres
        $('.mini-window').each(function() {
            tops.push(parseInt($(this).css('top'), 10));
        });
        tops.sort();
        for (let i = 0; i < tops.length; i++) {
            if ((top >= tops[i] && top < tops[i] + 40) || (top < tops[i] && top + 40 > tops[i])) {
                top = tops[i] + 40;
            }
        }

        miniWindow.css({
            position: 'fixed', // Important pour "Chrome"
            left: 0,
            top: top+'px'
        }).draggableCustom({handle: 'h4, .bottom-handle'});

        restore.click(function() {
            miniWindow.toggle('puff');
            if (modal.hasClass('in')) {
                _minifyModal();
                modal.fadeOut(200, function() { $(this).fadeIn(200); });
            }

            modal.find('.modal-body').replaceWith(modalClone.find('.modal-body'));
            modal.modal('show');
            minifierIcon.attr('data-num', num);
            _removeMaximized();
            setTimeout(function() { miniWindow.remove(); }, 500);
        });

        _removeMaximized();
        $('html > body').append(miniWindow);
    };

    minifierIcon.click(function() {
        if (windowLabel) {
            (function() {
                var label = eval(windowLabel);
                if (label) {
                    title = label;
                }
            }).call(modal);
        }
        _minifyModal();
        modal.toggle('drop');
        modal.modal('hide');
        $(this).removeAttr('data-num');
    });


    maximizerIcon.click(_maximizeModal);
    h4.dblclick(_maximizeModal);

    h4.find('i.fa-window-minimize, i.fa-window-maximize').remove();
    h4.append(maximizerIcon).append(minifierIcon);
}


/**
 * Permet de charger les filtres de recherche par ajax
 * @param event
 */
AsalaeGlobal.modalFiltersFormHandler = function(event) {
    var form = $(this).get(0);
    event.preventDefault();
    if (form.checkValidity()) {
        let clone = $(this).clone();
        clone.find('input.configuration-filter').remove();
        let url = $(this).attr('action') + '?' + clone.serialize();
        $(this).closest('.modal').modal('hide');
        $('html').addClass('ajax-loading');
        setTimeout(function() {
            AsalaeGlobal.interceptedLinkToAjax(url);
        }, 500);
    } else {
        form.reportValidity();
    }

}

/**
 * Assignation dynamique de la taille des table td
 */
AsalaeGlobal.smartTdSize = function() {
    $('.smart-td-size').each(function() {
        var columnsCount = 0,
            maxWidth = $(this).parent().width(),
            widths = [],
            max,
            trs = $(this).find('> thead > tr, > tbody > tr, > tr');

        if (trs.length === 0) {
            return;
        }

        // On compte le nombre max de colonnes
        trs.each(function() {
            var tdCount = 0;
            $(this).find('> td:visible, > th:visible').each(function() {
                var colspan = $(this).attr('colspan');
                if (!colspan) {
                    colspan = 1;
                }
                tdCount += parseInt(colspan, 10);
            });
            columnsCount = columnsCount < tdCount ? tdCount : columnsCount;
        });

        // On quantifi la taille maximum de chaques colonnes
        trs.each(function() {
            var i = 0;
            $(this).find('> td:visible, > th:visible').each(function() {
                var colspan = $(this).attr('colspan'),
                    width = $(this).outerWidth();
                if (!colspan) {
                    colspan = 1;
                }
                $(this).css('max-width', '');
                width = width / colspan;
                for (let col = 0; col < colspan; ++col) {
                    if (widths[i + col]) {
                        widths[i + col] = Math.max(widths[i + col], width);
                    } else {
                        widths[i + col] = width;
                    }
                }
                i += colspan;
            });
        });
        if (widths.length === 0) {
            return;
        }

        // On cherche le max-width idéal pour chaques colonnes
        max = Math.max.apply(null, widths);
        while (widths.reduce(function(a, b) { return a + b; }, 0) > maxWidth && isFinite(max)) {
            max--;
            for (let i = 0; i < widths[i]; i++) {
                if (widths[i] > max) {
                    widths[i] = max;
                }
            }
        }

        // On destribu la taille maximum de chaques colonnes
        trs.each(function() {
            var i = 0;
            $(this).find('> td:visible, > th:visible').each(function() {
                var colspan = $(this).attr('colspan'),
                    width = 0,
                    divScrollable = $('<div></div>');
                if (!colspan) {
                    colspan = 1;
                }
                for (let col = 0; col < colspan; ++col) {
                    width += widths[i + col];
                }
                $(this).css('max-width', width);
                i += colspan;
                if (!$(this).find('> div').length) {
                    $(this).contents().appendTo(divScrollable);
                    $(this).append(divScrollable);
                }
            });
        });
    });
}


AsalaeGlobal.replacePageContent = function(content, push = '') {
    var pageEmpty = true,
        mainContent = $('.main-content');
    mainContent.find('*').off();
    mainContent.empty();
    $(content).each(function() {
        var id = $(this).attr('id');
        if (id === 'layout-content') {
            pageEmpty = false;
            $(this).find('*[id]').each(function() {
                var id = $(this).attr('id'),
                    target = $('#'+id);
                if (Object.keys(target).length > 0) {
                    target.replaceWith($(this));
                } else {
                    mainContent.append($(this));
                }
            });
        } else if (id === 'template-content') {
            pageEmpty = false;
            $('.main-content').append($(this).html());
        } else if (id === 'head-content') {
            $('html > head > title').text($(this).find('title').html());
        }
    });
    if (pageEmpty) {
        mainContent.append(content);
    }
    setTimeout(AsalaeGlobal.initContentScope, 0);
    if (push) {
        window.history.pushState({}, '', push);
        $(window).trigger('history.change');
    }
}

/**
 * Empeche la page de se recharger completement lors d'un clic sur <a>
 * @param {string} href
 * @param {boolean} push effectu un window.history.pushState
 */
AsalaeGlobal.interceptedLinkToAjax = function(href, push = true) {
    $('.modal').modal('hide');
    setTimeout(function() {
        $('.modal-backdrop').remove();
    }, 400);
    $('.main-content').fadeOut();
    AsalaeLoading.start();
    $.ajax({
        url: href,
        type: 'GET',
        headers: {"No-Layout": true},
        success: function(content, textStatus, jqXHR) {
            AsalaeGlobal.replacePageContent(content, push ? href : '');
        },
        error: function(jqXHR, textStatus, errorThrown) {
            if (jqXHR.status === 403
                && typeof AsalaeSessionTimeout !== 'undefined'
                && typeof AsalaeSessionTimeout.session_timeout !== 'undefined'
                && jqXHR.getResponseHeader('X-Connected') === 'false'
            ) {
                clearTimeout(AsalaeSessionTimeout.session_timeout);
                AsalaeSessionTimeout.checkSessionTimeout(0);
                return;
            }
            var body = $(jqXHR.responseText).find('body');
            if (body.length < 1) {
                let bodyPos = jqXHR.responseText.indexOf('<body'),
                    bodyClose = jqXHR.responseText.indexOf('>', bodyPos+1),
                    bodyEnd = jqXHR.responseText.indexOf('</body')
                ;
                body = $(jqXHR.responseText.substring(bodyClose+1, bodyEnd));
            }
            $('body').html(body);
        },
        complete: function() {
            $('.main-content').fadeIn();
            AsalaeLoading.stop();
        }
    });
}
AsalaeGlobal.interceptedLinkhandler = function(event) {
    var href = $(this).attr('href');
    if (!href || $(this).getEvents().click.length > 1) {
        return;
    }
    if (href[0] !== '#'
        && href.indexOf('javascript:') !== 0
        && ['no-ajax', 'section-ajax'].indexOf($(this).attr('data-mode')) === -1
        && !$(this).attr('target')
    ) {
        event.preventDefault();
        AsalaeGlobal.interceptedLinkToAjax(href);
    }
};

AsalaeGlobal.grabHandlers = {
    mousedown: function(event) {
        $(this).addClass('grabbing')
            .data('clientX', event.clientX)
            .data('clientY', event.clientY)
            .data('scrollX', $(this).parent().scrollLeft())
            .data('scrollY', $(this).parent().scrollTop())
        ;
    },
    mouseup: function() {
        $('.grabbing').removeClass('grabbing');
    },
    mousemove: function(event) {
        if ($(this).hasClass('grabbing')) {
            var diffX = event.clientX - $(this).data('clientX'),
                diffY = event.clientY - $(this).data('clientY')
            ;
            $(this).parent()
                .scrollLeft($(this).data('scrollX') - diffX)
                .scrollTop($(this).data('scrollY') - diffY)
            ;
        }
    }
};

/**
 * Permet un affichage en mode "fixed" et respectant les règles d'accéssibilité
 * @param {jQuery} element
 * @param {string} label aria-label
 * @return {jQuery} element
 */
AsalaeGlobal.chosen = function(element, label) {
    element.chosen({width: ''})
        .on('chosen:showing_dropdown', function() {
            var container = $(this).parent().find('.chosen-container'),
                modal = container.closest('.modal-dialog'),
                drop = container.find('.chosen-drop'),
                offset = container.offset(),
                modalOffset = modal.offset(),
                width = container.parent().width()
            ;
            if (modal) {
                drop.css({
                    position: 'fixed',
                    top: offset.top + container.height() - modalOffset.top,
                    left: offset.left - modalOffset.left,
                    width: width
                });
            } else {
                drop.css({
                    position: 'fixed',
                    top: offset.top + container.height(),
                    left: offset.left,
                    width: container.width()
                });
            }
            // Corrige le décalage de 10px entre le 1er appel et le 2e
            if (container.parent().width() > width) {
                drop.css('width', container.parent().width());
            }
        })
        .parent()
        .find('select').show().css({
            opacity: 0,
            position: 'absolute'
        })
        .addClass('chosenified')
        .on('invalid', function(event) {
            $(this).parent().find('.chosen-container').addClass('invalid').one('click', function() {
                $(this).removeClass('invalid');
            });
        })
        .parent()
        .find('input.chosen-search-input')
        .attr('aria-label', label)
    ;
    return element;
};

/**
 * Déplacement des liens dans le menu utilisateur
 */
class CustomMenu {
    static dropdownToggleOnDragoverHandler(event) {
        var dropdown = $(this).find('.dropdown-toggle'),
            target = dropdown.attr('data-target');
        if (target) {
            $(target).removeClass('open');
        } else {
            $(this).removeClass('open');
        }
        $(this).addClass('dragover');
        dropdown.dropdown('toggle');
    }

    static dragleaveHandler() {
        $(this).removeClass('dragover');
    }

    static dragoverHandler(event) {
        var offset = $(this).offset(),
            height = $(this).height();
        if (!CustomMenu.navbarDragging) {
            return;
        }

        if (event.pageY < offset.top + (height / 2)) {
            CustomMenu.navbarDragging.insertBefore($(this).parent());
        } else {
            CustomMenu.navbarDragging.insertAfter($(this).parent());
        }
    }

    static dragstartHandler() {
        var target = $(this).parent(),
            clone = target.clone();
        if ($('#user-menu').find(this).length) {
            CustomMenu.navbarDraggingBackup = target.hide();
        }
        CustomMenu.navbarDragging = clone.addClass('active');
    }

    static dragleaverHandler(event) {
        $(document).one('mousemove', function(event) {
            var userMenu = $('#user-menu'),
                menu = userMenu.find('.dropdown-menu'),
                offset = menu.offset(),
                width = menu.width(),
                height = menu.height();
            if (!CustomMenu.navbarDragging) {
                return;
            }
            // Si la souris est à l'interieur du menu lors de l'evenement...
            if (offset
                && event.pageX > offset.left
                && event.pageX < offset.left + width
                && event.pageY > offset.top
                && event.pageY < offset.top + height
            ) {
                CustomMenu.navbarDragging.removeClass('active');
                if (CustomMenu.navbarDragging.find('.fa-times').length === 0) {
                    CustomMenu.navbarDragging.append(CustomMenu.getCloseBtn());
                }
                setTimeout(function() {
                    userMenu.trigger('user-menu.changed');
                }, 1);
                if (CustomMenu.navbarDraggingBackup) {
                    CustomMenu.navbarDragging.replaceWith(CustomMenu.navbarDraggingBackup.show());
                }
            } else {
                CustomMenu.navbarDragging.remove();
                if (CustomMenu.navbarDraggingBackup) {
                    CustomMenu.navbarDraggingBackup.show();
                }
            }
            CustomMenu.navbarDragging = undefined;
            CustomMenu.navbarDraggingBackup = undefined;
        });
    }

    static getCloseBtn() {
        var btn = $('<i aria-hidden="true" class="fa fa-times close"></i>');
        return btn.click(function(event) {
            event.stopPropagation();
            $(this).closest('li').remove();
            $('#user-menu').trigger('user-menu.changed');
        })
    }
}
CustomMenu.navbarDragging = undefined;
CustomMenu.navbarDraggingBackup = undefined;

var AsalaeLoading = {};

/**
 * Démarre l'animation de chargement
 */
AsalaeLoading.start = function() {
    $('html').addClass('ajax-loading');
    $('#loading-screen').fadeIn(100);
    // Résout un bug sur ie
    setTimeout(function() {
        $('#loading-screen').removeClass('paused');
    }, 10);
};

/**
 * Termine l'animation de chargement
 */
AsalaeLoading.stop = function() {
    $('html').removeClass('ajax-loading');
    $('#loading-screen').fadeOut(100, function() {
        $(this).addClass('paused');
    });
};