'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* global moment, jQuery */

/**
 * Ensemble de classes javascript pour manipuler/générer des tableaux de 
 * résultats.
 */

/**
 * Permet de manipuler un tableau à partir d'un export de donnée CakePHP
 * transformé en json et de quelques paramètres
 * (nom des colonnes, les liens, les titres...)
 *
 * @param {Element|jQuery} table
 * @returns {TableGenerator}
 */
var TableGenerator = function () {
    function TableGenerator(table) {
        _classCallCheck(this, TableGenerator);

        this.table = table;
        this.data = {};
        this._params = null;
        this._prevParams = null;
        this._rawParams = null;
        this._configId = null;
        this._configChanged = false;
        this.view = 'default';
        this.cookies = {
            base: 'table-generator-config-table-' + $(table).attr('id')
        };
        this.cookies.active = this.cookies.base + '-tr-active';
        this.cookies.accepted = {
            default: this.cookies.base + '-tr-accepted-default',
            div: this.cookies.base + '-tr-accepted-div',
            column: this.cookies.base + '-tr-accepted-column'
        };
        this.cookies.view = this.cookies.base + '-view';
        this.cookies.maxPerPage = this.cookies.base + '-max-per-page';
        this.elements = {
            draggable: { id: 'draggable-' + $(this.table).attr('id') }
        };
        this.messages = {
            favorite: "Add/remove from favorites",
            confirmSwitchView: "Do you want to save your change before switch the view?",
            confirmChangeMaxPerPage: "We need to refresh the page to apply your changes. Do you want to refresh it now ?",
            selectCheckbox: "Select",
            selectAllCheckboxes: "Select all"
        };
        this.max_per_page = 0;
        if (typeof TableGenerator.instance === 'undefined') {
            TableGenerator.instance = [];
        }
        var i = 0;
        do {
            this.instanceIndex = i;
            i++;
        } while (typeof TableGenerator.instance[this.instanceIndex] !== 'undefined');
        TableGenerator.instance[this.instanceIndex] = this;
        TableGenerator.lastInstance = this;
        $(table).attr('data-table-uid', this.instanceIndex);
    }

    /**
     * Génère le contenu d'un tableau (efface le precédent)
     *
     * @param {object} data
     * @param {object} params
     * @returns {TableGenerator}
     */


    _createClass(TableGenerator, [{
        key: 'generateAll',
        value: function generateAll() {
            var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.data;
            var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this._viewParams();

            data = typeof params.tbody.order !== 'undefined' ? TableGenerator.evalSortArray(data, params.tbody.order) : data;

            $(this.table).find('thead').remove();
            $(this.table).prepend(this._generateThead(params));

            $(this.table).find('tbody').remove();
            if (typeof data !== 'undefined') {
                $(this.table).append(this._generateTbody(data, params));
            }

            this._toggleClassActiveCookie();
            $(window).trigger('resize');
            $(this.table).trigger('change');

            return this;
        }
    }, {
        key: 'generateTbody',


        /**
         * Reconstruit le tbody du tableau
         *
         * @param {object} data
         * @param {object} params
         * @returns {TableGenerator}
         */
        value: function generateTbody() {
            var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.data;
            var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this._viewParams();

            data = typeof params.tbody.order !== 'undefined' ? TableGenerator.evalSortArray(data, params.tbody.order) : data;

            $(this.table).find('tbody').remove();
            if (typeof data !== 'undefined') {
                $(this.table).append(this._generateTbody(data, params));
            }

            this._toggleClassActiveCookie();
            $(window).trigger('resize');
            $(this.table).trigger('change');

            return this;
        }

        /**
         * Permet de retirer une ligne de data par identifiant
         * Utile lors d'une suppression
         *
         * @param {string} id
         * @param {string} path
         * @returns {TableGenerator}
         */

    }, {
        key: 'removeDataId',
        value: function removeDataId(id) {
            var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

            var data = this.data;

            if (path === null) {
                path = this._params.tbody.identifier;
            }

            for (var key in data) {
                try {
                    if (TableGenerator._evalData(path, data[key]) + '' === id + '') {
                        this.data.splice(key, 1);
                        break;
                    }
                } catch (e) {}
            }

            return this;
        }

        /**
         * Permet d'obtenir les données d'une ligne à partir d'un identifiant
         *
         * @param {string} id
         * @param {string} path
         * @returns {object}
         */

    }, {
        key: 'getDataId',
        value: function getDataId(id) {
            var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

            var data = this.data;

            if (path === null) {
                path = this._params.tbody.identifier;
            }

            for (var key in data) {
                try {
                    if (TableGenerator._evalData(path, data[key]) + '' === id + '') {
                        return data[key];
                    }
                } catch (e) {}
            }
        }

        /**
         * Permet de remplacer les données d'une ligne à partir d'un identifiant
         *
         * @param {string} id
         * @param {object} newData
         * @param {string} path
         * @returns {TableGenerator}
         */

    }, {
        key: 'replaceDataId',
        value: function replaceDataId(id, newData) {
            var path = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

            var data = this.data;

            if (path === null) {
                path = this._params.tbody.identifier;
            }

            for (var key in data) {
                try {
                    if (TableGenerator._evalData(path, data[key]) === id) {
                        data[key] = newData;
                    }
                } catch (e) {}
            }

            return this;
        }

        /**
         * Permet d'obtenir la liste des cookies et leurs valeurs
         *
         * @returns {object}
         */

    }, {
        key: 'exportCookies',
        value: function exportCookies() {
            var toString = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

            var results = this._recursiveExportCookies(this.cookies);
            return toString ? JSON.stringify(results) : results;
        }

        /**
         * Fait la lecture de la liste de cookies de façon recursive
         *
         * @param {object} cookies
         * @returns {object} cookies
         */

    }, {
        key: '_recursiveExportCookies',
        value: function _recursiveExportCookies(cookies) {
            var results = {},
                value;
            for (var key in cookies) {
                if (typeof cookies[key] === 'string') {
                    value = $.cookie(cookies[key], undefined, { path: '/' });
                    if (typeof value !== 'undefined') {
                        results[cookies[key]] = value;
                    }
                } else {
                    Object.assign(results, this._recursiveExportCookies(cookies[key]));
                }
            }
            return results;
        }

        /**
         * Importe les cookies
         *
         * @param {object|string} cookies
         * @returns {TableGenerator}
         */

    }, {
        key: 'importCookies',
        value: function importCookies(cookies) {
            if (typeof cookies === 'string') {
                cookies = JSON.parse(cookies);
            }
            if ((typeof cookies === 'undefined' ? 'undefined' : _typeof(cookies)) === 'object' && Object.keys(cookies).length) {
                for (var key in cookies) {
                    $.cookie(key, cookies[key], { path: '/', expires: 10 * 365 /* jours */ });
                }
            }
            return this;
        }

        /**
         * Setter de params
         *
         * @param {object} params
         * @returns {TableGenerator}
         */

    }, {
        key: 'params',
        value: function params(_params) {
            this._rawParams = _params;
            this._params = {
                tbody: {},
                thead: {
                    default: [],
                    column: [],
                    div: []
                } };
            this._params.tbody = _params.tbody;

            for (var tr in _params.thead) {
                this._params.thead.default[tr] = [];
                this._params.thead.column[tr] = [];
                this._params.thead.div[tr] = [new TableHead('view-div', {
                    target: null,
                    'data-view': 'div',
                    label: this._params.tbody['view-div-label'],
                    thead: []
                })];

                for (var th in _params.thead[tr]) {
                    switch (_params.thead[tr][th]['data-view']) {
                        case 'default':
                            this._params.thead.default[tr].push(_params.thead[tr][th]);
                        // no-break FALLTHROUGH
                        case 'div':
                            this._params.thead.div[0][0].thead.push(_params.thead[tr][th]);
                            break;
                        case 'column':
                            this._params.thead.default[tr].push(_params.thead[tr][th]);
                            this._params.thead.column[tr].push(_params.thead[tr][th]);
                            this._params.thead.div[tr].push(_params.thead[tr][th]);
                    }
                }
            }

            return this;
        }

        /**
         * Permet d'obtenir les paramètres actif de la vue
         *
         * @returns {object}
         */

    }, {
        key: '_viewParams',
        value: function _viewParams() {
            return {
                tbody: this._params.tbody,
                thead: this._params.thead[this.view]
            };
        }

        /**
         * Applique la configuration en cookie
         *
         * @returns {TableGenerator}
         */

    }, {
        key: 'applyCookies',
        value: function applyCookies() {
            var acceptedCookie = $.cookie(this.cookies.accepted[this.view], undefined, { path: '/' }),
                accepted = acceptedCookie ? JSON.parse(acceptedCookie) : null,
                view = $.cookie(this.cookies.view, undefined, { path: '/' }),
                maxPerPage = $.cookie(this.cookies.maxPerPage, undefined, { path: '/' }),
                thead,
                visible,
                id;

            if (view) {
                this.view = view;
            }

            if (maxPerPage) {
                this.max_per_page = maxPerPage;
            }

            if (accepted && accepted.length) {
                for (var tr in this._viewParams().thead) {
                    var activeThead = this.view !== 'div' ? this._viewParams().thead[tr] : this._viewParams().thead[tr][0].thead;

                    thead = [];
                    visible = false;

                    // On ajoute en premier dans l'ordre mémorisé les colonnes choisies
                    for (var key in accepted) {
                        for (var th in activeThead) {
                            id = this.elements.draggable.id + '-' + activeThead[th]['_id'];
                            if (id === accepted[key]) {
                                activeThead[th].display = true;
                                thead.push(activeThead[th]);
                                break;
                            }
                        }
                    }

                    // On ajoute ensuite en champs cachés les elements restant
                    for (var _th in activeThead) {
                        id = this.elements.draggable.id + '-' + activeThead[_th]['_id'];
                        visible = false;
                        for (var _key in accepted) {
                            if (id === accepted[_key]) {
                                visible = true;
                                break;
                            }
                        }
                        if (!visible) {
                            activeThead[_th].display = false;
                            thead.push(activeThead[_th]);
                        }
                    }

                    if (this.view !== 'div') {
                        this._params.thead[this.view][tr] = thead;
                    } else {
                        this._params.thead.div[tr][0].thead = thead;
                    }
                }
            }

            return this;
        }

        /**
         * Supprime tous les cookies liés à la table courante
         *
         * @param {object} cookies - default this.cookies
         * @returns {TableGenerator}
         */

    }, {
        key: 'deleteCookies',
        value: function deleteCookies() {
            var cookies = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.cookies;

            for (var key in cookies) {
                if (typeof cookies[key] === 'string') {
                    $.removeCookie(cookies[key], { path: '/' });
                } else {
                    this.deleteCookies(cookies[key]);
                }
            }

            return this;
        }

        /**
         * Rempli la zone de configuration du tableau de résultats
         *
         * @param {string} id
         * @returns {TableGenerator}
         */

    }, {
        key: 'insertConfiguration',
        value: function insertConfiguration(id) {
            this._configId = id;
            this._dropZone($(id + ' div.showed div.drop-zone'));
            this._dropZone($(id + ' div.hidded div.drop-zone'));
            this._btnSwitchViews();
            this._btnAcceptChange();
            this._btnCancelChange();
            this._btnApplyChange();
            this._prevParams = this._viewParams();
            this.fillDropZone();
            this._selectMaxResultsPerPage();

            return this;
        }

        /**
         * Select du nombre de résultats par pages
         *
         * @returns {TableGenerator}
         */

    }, {
        key: '_selectMaxResultsPerPage',
        value: function _selectMaxResultsPerPage() {
            var elements = $(this._configId + ' .nb-max-results'),
                that = this;

            this.initial_max_per_page = $(elements).val();
            if (this.max_per_page) {
                $(elements).val(this.max_per_page);
            } else {
                this.max_per_page = this.initial_max_per_page;
            }

            $(elements).change(function () {
                that.max_per_page = $(this).val();
            });

            return this;
        }

        /**
         * Boutons permettant de changer de "rendu" du tableau
         *
         * @returns {TableGenerator}
         */

    }, {
        key: '_btnSwitchViews',
        value: function _btnSwitchViews() {
            var elements = $(this._configId + ' .switch-table-view'),
                that = this;

            $(elements).off('.clickEnter').clickEnter(function (event) {
                if ($(this).hasClass('active')) {
                    return;
                }
                $(elements).removeClass('active');
                $(this).addClass('active');

                if (that._configChanged && confirm(that.messages.confirmSwitchView)) {
                    that._applyChange();
                }
                that._configChanged = false;

                if (!that._prevView) {
                    that._prevView = that.view;
                }
                that.view = $(this).attr('data-view');

                $(that._configId + ' div.drop-zone').parent().css('min-height', function () {
                    return $(this).height() + 'px';
                });

                var animationSpeed = 200;
                $(that._configId + ' div.drop-zone').slideUp(animationSpeed, 'swing', function () {
                    that.fillDropZone();
                    setTimeout(function () {
                        $(that._configId + ' div.drop-zone').slideDown(animationSpeed);
                    }, 10);
                });
            });

            return this;
        }

        /**
         * Rempli les drop zones avec le paramétrage actuel
         *
         * @param {object} params
         * @returns {TableGenerator}
         */

    }, {
        key: 'fillDropZone',
        value: function fillDropZone() {
            var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this._viewParams();

            $(this._configId + ' div.drop-zone div.draggable-element').remove();

            for (var tr in params.thead) {
                if (this.view === 'div') {
                    for (var th in params.thead[tr][0].thead) {
                        var dropZone = params.thead[tr][0].thead[th].display ? 'div.showed' : 'div.hidded';

                        $(this._configId + ' ' + dropZone + ' div.drop-zone').append(this._getDraggableElement(params.thead[tr][0].thead[th]));
                    }
                    continue;
                }

                for (var th in params.thead[tr]) {
                    var dropZone = params.thead[tr][th].display ? 'div.showed' : 'div.hidded';

                    $(this._configId + ' ' + dropZone + ' div.drop-zone').append(this._getDraggableElement(params.thead[tr][th]));
                }
            }

            return this;
        }

        /**
         * Bouton Valider
         *
         * @returns {TableGenerator}
         */

    }, {
        key: '_btnAcceptChange',
        value: function _btnAcceptChange() {
            var that = this;
            $(this._configId + ' button.accept').off('.clickEnter').clickEnter(function () {
                that._applyChange();
                if (that.max_per_page !== that.initial_max_per_page && confirm(that.messages.confirmChangeMaxPerPage)) {
                    location.reload();
                }
            });
            return this;
        }

        /**
         * Bouton Annuler
         *
         * @returns {TableGenerator}
         */

    }, {
        key: '_btnCancelChange',
        value: function _btnCancelChange() {
            var that = this,
                accepted = [];

            $(this._configId + ' button.cancel').off('.clickEnter').clickEnter(function () {
                var maxPerPage = $.cookie(that.cookies.maxPerPage, undefined, { path: '/' });
                that._prevParams = that._params;
                if (that._prevView) {
                    that.view = that._prevView;
                }

                $(that._configId + ' div.showed div.drop-zone div.draggable-element').remove();
                $(that._configId + ' div.hidded div.drop-zone div.draggable-element').remove();
                that.insertConfiguration(that._configId);
                that._configChanged = false;
                if (maxPerPage) {
                    $(that._configId + ' .nb-max-results').val(maxPerPage);
                } else {
                    var v = $(that._configId + ' .nb-max-results option').first();
                    $(that._configId + ' .nb-max-results').val(typeof v.val() !== 'undefined' ? v.val() : v.text());
                }
            });

            return this;
        }

        /**
         * Bouton Appliquer
         *
         * @returns {TableGenerator}
         */

    }, {
        key: '_btnApplyChange',
        value: function _btnApplyChange() {
            var that = this;
            $(this._configId + ' button.apply').off('.clickEnter').clickEnter(function () {
                that._applyChange();
            });
            return this;
        }

        /**
         * Applique les modifications de la configuration du tableau
         *
         * @returns {TableGenerator}
         */

    }, {
        key: '_applyChange',
        value: function _applyChange() {
            var accepted = [],
                opt = { path: '/', expires: 10 * 365 /* jours */ };

            $(this._configId + ' div.showed div.drop-zone div.draggable-element').each(function () {
                accepted.push($(this).attr('id'));
            });

            $.cookie(this.cookies.accepted[this.view], JSON.stringify(accepted), opt);
            $.cookie(this.cookies.view, this.view, opt);
            $.cookie(this.cookies.maxPerPage, this.max_per_page, opt);

            this.applyCookies();
            this.generateAll();
            this._configChanged = false;
            this._prevParams = this._params;
            this._prevView = null;

            return this;
        }

        /**
         * Applique les evenements pour transformer un element en drop zone
         *
         * @param {jQuery|Element} element
         * @returns {TableGenerator}
         */

    }, {
        key: '_dropZone',
        value: function _dropZone(element) {
            var that = this;

            $(element).sortable();
            $(element).droppable({
                drop: function drop(event, ui) {
                    that._configChanged = true;
                    if (event.target !== $(event.originalEvent.target).parent().get(0)) {
                        $(event.target).append($(event.originalEvent.target).removeAttr('style').clone());
                        $(event.originalEvent.target).remove();
                    }
                }
            });
            $(element).disableSelection();

            return this;
        }

        /**
         * Retourne un element draggable
         *
         * @param {string} info
         * @returns {jQuery}
         */

    }, {
        key: '_getDraggableElement',
        value: function _getDraggableElement(info) {
            var element = $('<div></div>'),
                label = info.label;
            try {
                if ($(label).length >= 1) {
                    label = $(label).children().remove().end().text();
                }
            } catch (e) {}

            element.addClass('draggable-element').addClass('btn').addClass('btn-info').addClass(info._id).attr('id', this.elements.draggable.id + '-' + info._id).attr('role', 'button').attr('tabindex', 0).append(label).dblclick(function () {
                if ($(this).parent().parent().hasClass('showed')) {
                    $(this).parent().parent().parent().find('.hidded .drop-zone').append(this);
                } else {
                    $(this).parent().parent().parent().find('.showed .drop-zone').append(this);
                }
            }).on('keyup', function (event) {
                switch (event.originalEvent.keyCode) {
                    case 13:
                        // Enter
                        $(this).trigger('dblclick');
                        break;
                    case 37:
                        // gauche
                        TableGenerator._moveDraggableElement($(this), 'left');
                        break;
                    case 38:
                        // haut
                        break;
                    case 39:
                        // droite
                        TableGenerator._moveDraggableElement($(this), 'right');
                        break;
                    case 40:
                        // bas
                        break;
                }
            });

            if (typeof info.class !== 'undefined' && info.class !== null) {
                element.addClass(info.class);
            }

            return element;
        }

        /**
         * Modifi l'ordre d'affichage d'un element dans son parent
         * @param element
         * @param direction
         */

    }, {
        key: '_generateThead',


        /**
         * Génère l'entête du tableau
         *
         * @param {object} params
         * @returns {jQuery} Element thead
         */
        value: function _generateThead() {
            var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this._viewParams();

            var thead = $('<thead></thead>'),
                tr,
                th,
                label;

            // Boucle sur les tr du thead
            for (var _i in params.thead) {
                tr = $('<tr></tr>');

                if (typeof params.tbody.checkbox !== 'undefined' && params.tbody.checkbox) {
                    tr.append($('<th></th>').append($('<input>').attr('id', 'table-' + this.instanceIndex + '-checkall').addClass('checkall').attr('data-table-uid', this.instanceIndex).attr('onclick', 'TableHelper.checkall(this)').attr('type', 'checkbox')).append('<label class="sr-only" for="table-' + this.instanceIndex + '-checkall">' + this.messages.selectAllCheckboxes + '</label>'));
                }
                if (typeof params.tbody.favorites !== 'undefined' && params.tbody.favorites) {
                    var fav = params.tbody.favorites;
                    tr.append($('<th></th>').append(fav === true ? '' : fav));
                }

                // Boucle sur les th du head
                for (var j in params.thead[_i]) {
                    label = params.thead[_i][j].link ? $('<a></a>').attr('href', params.thead[_i][j].link).text(params.thead[_i][j].label) : params.thead[_i][j].label;

                    th = $('<' + params.thead[_i][j].type + '></' + params.thead[_i][j].type + '>').toggle(params.thead[_i][j].display).append(label);

                    if (params.thead[_i][j].colspan > 1) {
                        th.attr('colspan', params.thead[_i][j].colspan);
                    }

                    TableGenerator._addTdOptions(th, params.thead[_i][j]);

                    tr.append(th);
                }

                thead.append(tr);
            }

            return thead;
        }
    }, {
        key: '_generateTbody',


        /**
         * Génère le corp du tableau
         *
         * @param {object} data
         * @param {object} params
         * @returns {jQuery} Element tbody
         */
        value: function _generateTbody() {
            var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.data;
            var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this._viewParams();

            var tr,
                tbody = $('<tbody></tbody>'),
                theadInfo = this._theadInfo(params);

            // On boucle sur chaque résultat (tr)
            for (var _i2 = 0; _i2 < data.length; _i2++) {
                tr = this._generateTr(_i2, theadInfo, data, params);
                tbody.append(tr);
            }

            return tbody;
        }
    }, {
        key: '_generateTr',


        /**
         * Génère une ligne du tbody
         *
         * @param {number} index pour récupérer les données dans data
         * @param {object} theadInfo
         * @param {object} data
         * @param {object} params
         * @returns {jQuery} Element tr
         */
        value: function _generateTr() {
            var index = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
            var theadInfo = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this._theadInfo(params);
            var data = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.data;
            var params = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this._viewParams();

            var cssClass,
                tr = $('<tr></tr>'),
                content,
                td,
                id = 'eval-failed-' + this.instanceIndex,
                favorite;

            try {
                cssClass = eval(params.tbody.classEval.replace(/{index}/g, index));
                tr.addClass(cssClass);
            } catch (e) {}
            try {
                id = TableGenerator._evalData(params.tbody.identifier, data[index]);
            } catch (e) {}

            // On boucle sur chaque colonne (td)
            for (var j = 0; j < theadInfo.length; j++) {
                try {
                    content = TableGenerator._evalData(theadInfo[j].target, data[index]);
                } catch (e) {
                    content = null;
                }

                if ((typeof content === 'undefined' ? 'undefined' : _typeof(content)) === 'object' && !content instanceof jQuery || theadInfo[j].thead !== null) {
                    content = this._parseContent(data[index], theadInfo[j]);
                }

                if (typeof theadInfo[j].callback !== 'undefined' && theadInfo[j].callback !== null) {
                    if (typeof theadInfo[j].callback === 'string') {
                        theadInfo[j].callback = eval(theadInfo[j].callback);
                    }
                    if (typeof theadInfo[j].callback === 'function') {
                        content = theadInfo[j].callback(content, data[index]);
                    }
                }

                td = $('<td></td>').addClass(TableGenerator._targetToClass(theadInfo[j].target)).toggle(theadInfo[j].display).append(content);

                if (theadInfo[j].colspan > 1) {
                    td.attr('colspan', theadInfo[j].colspan);
                }

                if (typeof theadInfo[j].class !== 'undefined') {
                    td.addClass(theadInfo[j].class);
                }

                td = TableGenerator._addTdOptions(td, theadInfo[j]);

                tr.append(td);
            }
            tr.attr('id', 'table-' + this.instanceIndex + '-tr-' + id).attr('data-id', id);

            if (typeof params.tbody.favorites !== 'undefined' && params.tbody.favorites) {
                favorite = $('<button title="' + this.messages.favorite + '" aria-label="' + this.messages.favorite + '" type="button" class="btn-link"></button>').append('<i aria-hidden="true" class="fa fa-star-o favorites-star"></i>');
                favorite.click(TableGenerator._favoriteHandler);
                TableGenerator._initFavorite(favorite, tr, this.table);
                tr.prepend($('<td></td>').addClass('td-favorite').append(favorite));
            }
            if (typeof params.tbody.checkbox !== 'undefined' && params.tbody.checkbox) {
                tr.prepend($('<td></td>').addClass('td-checkbox').append($('<input>').attr('id', 'table-' + this.instanceIndex + '-checkbox' + id).attr('name', 'data[table][' + this.instanceIndex + '][]').attr('type', 'checkbox').addClass('table-' + this.instanceIndex + '-checkbox').val(id)).append('<label class="sr-only" for="table-' + this.instanceIndex + '-checkbox' + id + '">' + this.messages.selectCheckbox + ' ' + id + '</label>'));
            }

            this._trActiveCookie(tr);

            return tr;
        }
    }, {
        key: '_subElements',


        /**
         * Ajoute les sous elements de façon récursive
         *
         * @param {jQuery|Element} subElement
         * @param {object} info
         * @param {Array|object} data
         * @returns {jQuery|Element}
         */
        value: function _subElements(subElement, info, data) {
            var subSubElement, field;

            if (Array.isArray(data)) {
                for (var _i3 in data) {
                    subSubElement = $('<div></div>').addClass('td-group-sub');
                    subSubElement = this._subElements(subSubElement, info, data[_i3]);
                    subElement.append(subSubElement);
                }
                return subElement;
            }

            for (var _i4 in info.thead) {
                subSubElement = $('<div></div>').addClass('td-group-sub-sub');

                try {
                    field = TableGenerator._evalData(info.thead[_i4].target, data);
                } catch (e) {
                    field = undefined;
                }

                if (typeof field === 'undefined' || field === null) {
                    continue;
                }

                if ((typeof field === 'undefined' ? 'undefined' : _typeof(field)) === 'object' && !(field instanceof jQuery)) {
                    field = this._parseContent(data, info.thead[_i4]);
                }

                if (!field) {
                    continue;
                }

                if (typeof info.thead[_i4].callback !== 'undefined' && info.thead[_i4].callback !== null) {
                    if (typeof info.thead[_i4].callback === 'string') {
                        info.thead[_i4].callback = eval(info.thead[_i4].callback);
                    }
                    if (typeof info.thead[_i4].callback === 'function') {
                        field = info.thead[_i4].callback(field);
                    }
                }

                if (info.thead[_i4].display === false || info.thead[_i4].display === 'none') {
                    subSubElement.hide();
                }

                if (info.thead[_i4].label) {
                    subSubElement.append($('<span></span>').addClass('td-group-fieldname').append(info.thead[_i4].label));
                }

                subSubElement.append($('<div></div>').addClass('td-group-fieldvalue').append(field));

                subElement.append(subSubElement);
            }

            return subElement;
        }

        /**
         * Permet de créer le contenu d'une cellule de façon récursive
         *
         * @param {object} data
         * @param {object} info theadInfo
         * @returns {jQuery} Element (div du td)
         */

    }, {
        key: '_parseContent',
        value: function _parseContent(data, info) {
            var element = $('<div></div>').addClass('td-group').attr('tabindex', -1);

            try {
                element = this._subElements(element, info, !info.target ? data : TableGenerator._evalData(info.target, data));
            } catch (e) {}

            return element;
        }

        /**
         * Fait une evaluation compatible avec la syntaxe suivante :
         * data['ma-var'] comme pour data.maVar
         *
         * @param {string} str
         * @param {object} data
         * @returns {string|number|Array|object|null|undefined|bool} résultat de l'eval
         */

    }, {
        key: '_theadInfo',


        /**
         * Simplifie le thead pour construire le tbody
         *
         * @param {object} params
         * @returns {array}
         */
        value: function _theadInfo() {
            var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this._viewParams();

            var thead = [];

            // Boucle sur les tr du thead
            for (var _i5 in params.thead) {
                // Boucle sur les th du head
                for (var j in params.thead[_i5]) {
                    thead.push(params.thead[_i5][j]);
                }
            }

            return thead;
        }
    }, {
        key: '_trActiveCookie',


        /**
         * Lors du clic sur une ligne, elle gagne/perd la classe active.
         * Cette information est conservé en cookie
         *
         * @param {jQuery|Element} tr
         * @returns {TableGenerator}
         */
        value: function _trActiveCookie(tr) {
            var that = this;
            $(tr).click(function () {
                if ($(this).hasClass('do-not-change')) {
                    $(this).removeClass('do-not-change');
                    return;
                }
                if ($(this).hasClass('active')) {
                    $.removeCookie(that.cookies.active);
                    $(this).parent().find('tr').removeClass('active');
                } else {
                    $(this).parent().find('tr').removeClass('active');
                    $(this).addClass('active');
                    $.cookie(that.cookies.active, 'table#' + $(tr).parent().parent().attr('id') + ' tr#' + $(tr).attr('id'), 10 * 365 /* jours */);
                }
            });
            $(tr).find('a, button.btn-link').click(function (event) {
                $(tr).closest('table').find('tbody tr').removeClass('active');
                $(tr).addClass('active').addClass('do-not-change');
                setTimeout(function () {
                    $(tr).removeClass('do-not-change');
                }, 100);
                $.cookie(that.cookies.active, 'table#' + $(tr).parent().parent().attr('id') + ' tr#' + $(tr).attr('id'), 10 * 365 /* jours */);
            });
            return this;
        }

        /**
         * Ajoute la classe active sur une ligne si elle est désignée active en cookie
         * @returns {TableGenerator}
         */

    }, {
        key: '_toggleClassActiveCookie',
        value: function _toggleClassActiveCookie() {
            var active = $.cookie(this.cookies.active, undefined, { path: '/' });
            $(active).addClass('active');
            return this;
        }
    }], [{
        key: '_moveDraggableElement',
        value: function _moveDraggableElement(element, direction) {
            var index = element.index(),
                parent = element.parent(),
                children = parent.children(),
                count = children.length;
            if (index === 0 && direction === 'left' || index === count - 1 && direction === 'right') {
                return;
            }
            if (direction === 'left') {
                element.insertBefore(children.eq(index - 1));
            } else {
                element.insertAfter(children.eq(index + 1));
            }
            element.focus();
        }

        /**
         * Permet d'ordonner un tableau (data) de façon dynamique selon str
         *
         * @param {object} data
         * @param {string} str Chemin sur lequel ranger le tableau ex: Model.name
         * @returns {Array} data ordonné
         */

    }, {
        key: 'evalSortArray',
        value: function evalSortArray(data, str) {
            data.sort(function (a, b) {
                var a2, b2;
                try {
                    a2 = TableGenerator._evalData(str, a);
                } catch (e) {
                    a2 = -Infinity();
                }
                try {
                    b2 = TableGenerator._evalData(str, b);
                } catch (e) {
                    b2 = -Infinity();
                }

                return a2 > b2;
            });

            return data;
        }

        /**
         * Ajoute à un json de data, les actions selon params
         *
         * @param {object} data
         * @param {object} params
         * @returns {object} data complété
         */

    }, {
        key: 'appendActions',
        value: function appendActions(data, params) {
            var _loop = function _loop(_i6) {
                data[_i6].actions = {};

                // Pour chaque actions

                var _loop2 = function _loop2(key) {
                    data[_i6].actions[key] = typeof params[key].href === 'string' ? $('<a></a>') : $('<button></button>');

                    // Pour chaque paramètre d'action

                    var _loop3 = function _loop3(k) {
                        switch (k) {
                            case 'label':
                                data[_i6].actions[key].append(TableGenerator._sprintf(params[key][k], params[key]['params'], data[_i6]));
                                break;
                            case 'params':
                                break;
                            case 'confirm':
                                data[_i6].actions[key].click(function (event) {
                                    if (!confirm(TableGenerator._sprintf(params[key][k], params[key]['params'], data[_i6]))) {
                                        return false;
                                    }
                                    if (params[key]['data-callback']) {
                                        eval(TableGenerator._sprintf(params[key]['data-callback'], params[key]['params'], data[_i6]));
                                    }
                                });
                                break;
                            case 'displayEval':
                                try {
                                    data[_i6].actions[key].toggle(eval(params[key][k].replace(/{index}/g, _i6)));
                                } catch (e) {}
                                break;
                            default:
                                data[_i6].actions[key].attr(k, TableGenerator._sprintf(params[key][k], params[key]['params'], data[_i6]));
                        }
                    };

                    for (var k in params[key]) {
                        _loop3(k);
                    }
                };

                for (var key in params) {
                    _loop2(key);
                }
            };

            // Pour chaque lignes dans data (tr)
            for (var _i6 = 0; _i6 < data.length; _i6++) {
                _loop(_i6);
            }

            return data;
        }

        /**
         * Modifi une chaine de caractère en lui remplacant les {0}, {1} ...
         * par ce qui est défini dans params
         *
         * @param {string} str chaine à modifier
         * @param {Array} params array de chemin vers la donnée (dans data)
         * @param {object} data données brut
         * @returns {string}
         */

    }, {
        key: '_sprintf',
        value: function _sprintf(str, params, data) {
            for (var _i7 = 0; _i7 < params.length; _i7++) {
                try {
                    str = str.replace('{' + _i7 + '}', TableGenerator._evalData(params[_i7], data));
                } catch (e) {}
            }
            return str;
        }

        /**
         * Echape une chaine tel addslashes de php
         *
         * @param {string} str
         * @returns {String}
         */

    }, {
        key: 'addslashesJson',
        value: function addslashesJson(str) {
            var a = {};
            a[str] = 1;
            return JSON.stringify(a).slice(2, -4);
        }
    }, {
        key: '_getFavoriteCookieName',


        /**
         * Permet d'obtenir le nom du cookie pour le favori d'une ligne d'un tableau
         * @param {jQuery} element
         * @param {jQuery} tr
         * @param {jQuery} table
         * @returns {string}
         */
        value: function _getFavoriteCookieName(element, tr, table) {
            return 'table-favorite-' + table.attr('id') + '-' + tr.attr('data-id');
        }

        /**
         * Action du clic sur l'etoile des favoris
         */

    }, {
        key: '_favoriteHandler',
        value: function _favoriteHandler() {
            var fa = $(this).find('i.fa'),
                cookieName = TableGenerator._getFavoriteCookieName(fa, fa.closest('tr'), fa.closest('table'));
            fa.toggleClass('fa-star').toggleClass('fa-star-o');
            if (fa.hasClass('fa-star')) {
                $.cookie(cookieName, '1', 10 * 365 /* jours */);
            } else {
                $.removeCookie(cookieName);
            }
        }

        /**
         * Affiche en favori si dans les cookies
         * @param {jQuery} element
         * @param {jQuery} tr
         * @param {jQuery} table
         */

    }, {
        key: '_initFavorite',
        value: function _initFavorite(element, tr, table) {
            var cookieName = TableGenerator._getFavoriteCookieName(element, tr, table);
            if ($.cookie(cookieName, undefined, { path: '/' })) {
                element.find('i.fa').addClass('fa-star').removeClass('fa-star-o');
            }
        }

        /**
         * Ajoute à un element jQuery les attributs données par options
         *
         * @param {jQuery} element
         * @param {object} options
         * @returns {jQuery}
         */

    }, {
        key: '_addTdOptions',
        value: function _addTdOptions(element, options) {
            var attrs = ['_id', 'label', 'link', 'type', 'display', 'target', 'thead', 'callback', 'colspan', 'style'];
            for (var key in options) {
                if (attrs.indexOf(key) < 0) {
                    element.attr(key, options[key]);
                }
            }

            // Merge les styles
            if (typeof options.style !== 'undefined' && options.style) {
                var style = element.attr('style') ? element.attr('style') : '';
                element.attr('style', style + options.style);
            }

            return element;
        }
    }, {
        key: '_evalData',
        value: function _evalData(str, data) {
            return eval(str.indexOf('[') === 0 ? 'data' + str : 'data.' + str);
        }
    }, {
        key: '_targetToClass',


        /**
         * Transforme une cible en nom de classe css
         * ex: Monmodel[0].field -> Monmodel-0-field
         *
         * @param {string} target
         * @returns {string}
         */
        value: function _targetToClass(target) {
            if (typeof target === 'undefined' || !target) {
                return null;
            }
            return target.replace('.', '-').replace('[', '-').replace(']', '');
        }
    }]);

    return TableGenerator;
}();

TableGenerator.instance = [];

/**
 * Aide à la construction d'un object adapté pour l'entête de TableGenerator
 *
 * @param {string} label
 * @param {object} params
 * @returns {TableHead}
 */

var TableHead = function TableHead(label, params) {
    _classCallCheck(this, TableHead);

    this._id = label.replace(/[\W]/g, '_');
    this.label = label;

    for (var key in params) {
        this[key] = params[key];
    }

    this.title = typeof this.title === 'undefined' ? this.label : this.title;
    this.link = typeof this.link === 'undefined' ? false : this.link;
    this.type = typeof this.type === 'undefined' ? 'th' : this.type;
    this.display = typeof this.display === 'undefined' ? true : this.display;
    this.colspan = typeof this.colspan === 'undefined' ? 1 : this.colspan;
    this.target = typeof this.target === 'undefined' ? label : this.target;
    this.thead = typeof this.thead === 'undefined' ? null : this.thead;
    this['data-view'] = typeof this['data-view'] === 'undefined' ? 'default' : this['data-view'];
};

/**
 * Ensemble de callbacks pour formater les champs pour le TableGenerator
 */


var TableHelper = function () {
    function TableHelper() {
        _classCallCheck(this, TableHelper);
    }

    _createClass(TableHelper, null, [{
        key: 'locale',

        /**
         * Permet de modifier la locale de la lib momentjs
         *
         * @param {string} str
         */
        value: function locale(str) {
            moment.locale(str);
        }

        /**
         * Permet de formater une date selon "format"
         *
         * @param {string} format ex: 'Y-M-D h:m:s'
         * @returns {Function}
         */

    }, {
        key: 'date',
        value: function date() {
            var format = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'Y-M-D';

            return function (value) {
                if (typeof value === 'undefined' || !value) {
                    return null;
                }

                return moment(new Date(value)).format(format);
            };
        }

        /**
         * Transforme une valeur boolean en mot
         * @param {string} yes
         * @param {string} no
         * @returns {Function}
         */

    }, {
        key: 'boolean',
        value: function boolean() {
            var yes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'yes';
            var no = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'no';

            return function (value) {
                if (typeof value === 'undefined') {
                    return null;
                }

                return value ? yes : no;
            };
        }

        /**
         * Conversion de bytes
         *
         * @param {string} bytes
         * @returns {String}
         */

    }, {
        key: 'readableBytes',
        value: function readableBytes(bytes) {
            if (!AsalaeGlobal.is_numeric(bytes)) {
                return bytes;
            }
            var s = ['bytes', 'kB', 'MB', 'GB', 'TB', 'PB'],
                e = Math.floor(Math.log(parseFloat(bytes)) / Math.log(1024));
            return Number.isFinite(e) ? (bytes / Math.pow(1024, e)).toFixed(2) + " " + s[e] : bytes;
        }

        /**
         * Permet de cocher/décocher toutes les cases d'un tableau
         * @param {Element} element
         */

    }, {
        key: 'checkall',
        value: function checkall(element) {
            $('table[data-table-uid="' + $(element).attr('data-table-uid') + '"]').find('tbody .td-checkbox input').prop('checked', $(element).prop('checked'));
        }
    }]);

    return TableHelper;
}();

//# sourceMappingURL=asalae.table-generator-compiled.js.map