'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*global Flow, TableHelper, TableGenerator*/

/**
 * Permet de faire le pont entre asalae2 et Flow.js
 * Est utilisé par \App\View\Helper\UploadHelper
 *
 * @category    Helper
 *
 * @author      Ludovic Ganée <ludovic.ganee@libriciel.coop>
 * @copyright   (c) 2017, Libriciel
 * @license     http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
 */

/**
 * Une nouvelle instance de cette classe permet une nouvelle instance de Flow
 * Selon "config", l'objet sera couplé avec une "dropbox" et remplira une table.
 */
var AsalaeUploader = function () {
    function AsalaeUploader(config) {
        _classCallCheck(this, AsalaeUploader);

        var dropbox = $('#' + config.id + '.dropbox');
        this.config = config;
        this.uploader = new Flow({
            target: config.target,
            chunkSize: config.chunkSize,
            testChunks: false
        });
        this.uploader.config = config;
        this.uploader.asalae = this;

        if (!this.uploader.support) {
            dropbox.css('background-color', '#f2dede').find('*').attr('disabled', 'disabled');
            dropbox.find('p').text('Votre navigateur ne supporte pas la fragmentation de fichiers, merci de le mettre à jour.');
            dropbox.find('label').text('Ou contactez un administrateur.');
            return;
        }

        this.uploader.assignDrop(dropbox.get(0));

        var isDirectory = typeof config.isDirectory === 'undefined' ? false : config.isDirectory;
        var singleFile = typeof config.singleFile === 'undefined' ? false : config.singleFile;
        var attributes = typeof config.attributes === 'undefined' ? {} : config.attributes;
        var button = dropbox.find('div.btn').off('.upload').on('keydown', function (event) {
            if (event.originalEvent.keyCode === 13) {
                $(this).click();
            }
        });
        this.uploader.assignBrowse(button.get(0), isDirectory, singleFile, attributes);

        var events = ['fileAdded', 'filesSubmitted', 'complete', 'fileSuccess', 'fileError', 'fileProgress', 'uploadStart', 'catchAll'];

        for (var key in events) {
            // noinspection JSCheckFunctionSignatures
            this.uploader.on(events[key], eval('this.' + events[key]));
        }

        var input = dropbox.find('input[type="file"]'),
            label = $('<label class="sr-only"></label>').text('File') // TODO Traduction (le fichier entier)
        ;
        label.insertBefore(input);
        label.append(input);
    }

    _createClass(AsalaeUploader, [{
        key: 'fileAdded',
        value: function fileAdded(file) {
            $('#' + this.config.id + '.dropbox').trigger('fileAdded', [file]);
            $('#' + this.config.id + '.dropbox.dragover').removeClass('dragover');
        }
    }, {
        key: 'filesSubmitted',
        value: function filesSubmitted(file) {
            var progressBar,
                pause,
                resume,
                cancel,
                prev,
                table = eval(this.config.table);

            $('#' + this.config.id + '.dropbox').trigger('filesSubmitted', [file]);

            var _loop = function _loop(i) {
                AsalaeUploader[file[i].uniqueIdentifier] = {
                    resume: function resume(element) {
                        $(element).parent().parent().find('.progress.uploading .progress-bar').removeClass('progress-bar-warning');
                        file[i].resume();
                        $(element).hide();
                        $(element).parent().find('button.pause').show();
                    },
                    cancel: function cancel(element) {
                        file[i].cancel();
                        table.removeDataId($(element).attr('data-target'));
                        $(element).closest('tr').fadeOut(200, function () {
                            $(this).remove();
                            $(table.table).trigger('change');
                        });
                    },
                    pause: function pause(element) {
                        $(element).parent().parent().find('.progress.uploading .progress-bar').addClass('progress-bar-warning');
                        file[i].pause();
                        $(element).hide();
                        $(element).parent().find('button.resume').show();
                    }
                };
                progressBar = $('<div></div>').append($('<div></div>').addClass('flow-file-progress').addClass('progress').addClass('uploading').append($('<div></div>').addClass('progress-bar').attr('role', 'progressbar').attr('aria-valuenow', '0').attr('aria-valuemin', '0').attr('aria-valuemax', '100').css('width', '0%'))).append($('<span></span>').addClass('text-danger').addClass('error-span'));
                pause = $('<button></button>').addClass('btn').addClass('btn-warning').addClass('fa').addClass('fa-pause').addClass('pause').attr('onclick', "AsalaeUploader['" + file[i].uniqueIdentifier + "'].pause(this)");
                resume = $('<button></button>').addClass('btn').addClass('btn-success').addClass('fa').addClass('fa-play').addClass('resume').hide().attr('onclick', "AsalaeUploader['" + file[i].uniqueIdentifier + "'].resume(this)");
                cancel = $('<button></button>').addClass('btn').addClass('btn-danger').addClass('fa').addClass('fa-ban').addClass('cancel').attr('data-target', file[i].uniqueIdentifier).attr('onclick', "AsalaeUploader['" + file[i].uniqueIdentifier + "'].cancel(this)");

                table.data.unshift({
                    id: file[i].uniqueIdentifier,
                    prev: null,
                    name: file[i].name,
                    size: file[i].size,
                    transfert: progressBar,
                    valid: true,
                    actions: {
                        action_0: $('<div></div>').append(pause).append(resume).append(cancel)
                    }
                });
                table.generateTbody();
                $('#table-' + table.instanceIndex + '-tr-' + file[i].uniqueIdentifier + ' .td-checkbox input').remove();
            };

            for (var i = 0; i < file.length; i++) {
                _loop(i);
            }
            this.resume();
        }
    }, {
        key: 'complete',
        value: function complete() {
            $('#' + this.config.id + '.dropbox').trigger('complete');
        }
    }, {
        key: 'fileSuccess',
        value: function fileSuccess(file, message) {
            message = JSON.parse(message);
            var table = eval(this.config.table),
                data = table.getDataId(file.uniqueIdentifier);

            $('#' + this.config.id + '.dropbox').trigger('fileSuccess', [file, message]);

            if (!message.report.completion || message.report.error) {
                return this.asalae.fileError(file, message);
            }

            $('#table-' + table.instanceIndex + '-tr-' + file.uniqueIdentifier + ' .flow-file-progress .progress-bar').addClass('progress-bar-success');

            delete data['actions'];
            data.id = message.report.id;
            data.prev = message.report.prev;
            data.convertable = message.report.convertable;
            data.mime = message.report.mime;
            data.statefr = message.report.statefr;
            data.siegfrieds = [{ pronom: message.report.pronom }];
            data.openable = message.report.openable;
            table.replaceDataId(file.uniqueIdentifier, TableGenerator.appendActions([data], table.actions)[0]);
            table.generateTbody();
        }
    }, {
        key: 'fileError',
        value: function fileError(file, message) {
            var table = eval(this.config.table),
                data;
            if (typeof message === 'string') {
                try {
                    message = JSON.parse(message);
                } catch (e) {
                    message = { report: { error: "Une erreur inattendue a eu lieu" } };
                }
            }
            $('#' + this.config.id + '.dropbox').trigger('fileError', [file, message]);
            $('#table-' + table.instanceIndex + '-tr-' + file.uniqueIdentifier + ' .flow-file-progress .progress-bar').css('width', '100%').attr('aria-valuenow', 100).html('100%').addClass('progress-bar-danger').closest('td').append(message.report.error).closest('tr').find('button.pause').remove();
            data = table.getDataId(file.uniqueIdentifier);
            data.transfert = $('#table-' + table.instanceIndex + '-tr-' + file.uniqueIdentifier + ' td.transfert').html();
            table.replaceDataId(file.uniqueIdentifier, data);
        }
    }, {
        key: 'fileProgress',
        value: function fileProgress(file) {
            var table = eval(this.config.table),
                percent = Math.floor(file.progress() * 100);
            $('#' + this.config.id + '.dropbox').trigger('fileProgress', [file]);
            $('#table-' + table.instanceIndex + '-tr-' + file.uniqueIdentifier + ' .flow-file-progress .progress-bar').css('width', percent + '%').attr('aria-valuenow', percent).html(percent + '% ');
        }
    }, {
        key: 'uploadStart',
        value: function uploadStart() {
            $('#' + this.config.id + '.dropbox').trigger('uploadStart');
        }
    }, {
        key: 'catchAll',
        value: function catchAll() {}
    }]);

    return AsalaeUploader;
}();

/**
 * Fonctionnement générique d'une dropbox d'upload
 */


var genericConfiguration = {
    tableGenerator: undefined,
    urlDelete: undefined,
    dropBox: undefined,
    paginationElement: undefined,
    selectUrl: undefined,
    messages: {
        confirmDelete: "Êtes-vous sûr de vouloir supprimer ce fichier ?",
        confirmDeleteAll: "Êtes-vous sûr de vouloir supprimer les fichiers sélectionnés ?",
        deleteError: "Une erreur a eu lieu lors de la suppression",
        selectionFailed: "Une erreur inattendue a eu lieu"
    }
};
/**
 * Initialise une dropbox avec son tableau d'affichage pour une utilisation générique
 */

var GenericUploader = function () {
    function GenericUploader(config) {
        _classCallCheck(this, GenericUploader);

        genericConfiguration = $.extend(true, {}, genericConfiguration, config);

        $(config.tableGenerator.table).change(function () {
            if ($(this).find('tbody tr').length === 0) {
                $(this).addClass('hide').closest('section').find('button.btn-primary').parent().addClass('hide');
            }
        });
        $(config.tableGenerator.table).find('thead input.checkall').change(function () {
            $(this).closest('table').find('tbody .td-checkbox input').prop('checked', $(this).prop('checked'));
        });

        $('#' + config.dropBox.config.id + '.dropbox').off().on('fileAdded', function () {
            $(config.tableGenerator.table).removeClass('hide').closest('section').find('button.btn-primary').parent().removeClass('hide');
        });
    }

    /**
     * Supprime un fichier uploadé
     *
     * @param {string} id
     * @param {boolean} useConfirm
     */


    _createClass(GenericUploader, null, [{
        key: 'fileDelete',
        value: function fileDelete(id) {
            var useConfirm = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

            var trId = '#table-' + genericConfiguration.tableGenerator.instanceIndex + '-tr-' + id;
            if (useConfirm && !confirm(genericConfiguration.messages.confirmDelete)) {
                return;
            }
            $.ajax({
                url: genericConfiguration.urlDelete + '/' + id,
                success: function success(message) {
                    if (message.report === 'done') {
                        genericConfiguration.tableGenerator.removeDataId(id);
                        $(trId).fadeOut(400, function () {
                            $(this.remove());
                            $(genericConfiguration.tableGenerator.table).trigger('change');
                        });
                    } else {
                        this.error();
                    }
                },
                error: function error() {
                    var data;
                    $(trId).addClass('danger').find('td.transfert span.error-span').text(genericConfiguration.messages.deleteError);
                    data = genericConfiguration.tableGenerator.getDataId(id);
                    data.transfert = $(trId + ' td.transfert').html();
                    data.valid = false;
                    genericConfiguration.tableGenerator.replaceDataId(id, data);
                }
            });
        }

        /**
         * Modal d'informations supplémentaire sur le fichier
         *
         * @param {string} id
         */

    }, {
        key: 'loadDataModal',
        value: function loadDataModal(id) {
            var data = genericConfiguration.tableGenerator.getDataId(id),
                div = $('<div></div>'),
                schema = [{ separator: 'h3', value: 'Informations générales' }, { title: 'Nom de fichier', value: data.name }, { title: 'Taille', value: data.size, callback: TableHelper.readableBytes }],
                value;

            if (typeof data.mediainfo !== 'undefined') {
                schema = schema.concat({ separator: 'h3', value: 'Mediainfo' }, { title: 'Format de fichier', value: data.mediainfo.format }, { title: 'Durée', value: data.mediainfo.duration });

                for (var i in data.mediainfo.mediainfo_videos) {
                    schema = schema.concat({ separator: 'h4', value: 'Video' }, { title: 'Format', value: data.mediainfo.mediainfo_videos[i].format }, { title: 'Débit', value: data.mediainfo.mediainfo_videos[i].bit_rate }, { title: 'Largeur', value: data.mediainfo.mediainfo_videos[i].width }, { title: 'Hauteur', value: data.mediainfo.mediainfo_videos[i].height }, { title: 'Langue', value: data.mediainfo.mediainfo_videos[i].language });
                }

                for (var _i in data.mediainfo.mediainfo_audios) {
                    schema = schema.concat({ separator: 'h4', value: 'Audio' }, { title: 'Format', value: data.mediainfo.mediainfo_audios[_i].format }, { title: 'Débit', value: data.mediainfo.mediainfo_audios[_i].bit_rate }, { title: 'Langue', value: data.mediainfo.mediainfo_audios[_i].language });
                }

                for (var _i2 in data.mediainfo.mediainfo_texts) {
                    schema = schema.concat({ separator: 'h4', value: 'Sous-titres' }, { title: 'Titre', value: data.mediainfo.mediainfo_texts[_i2].title }, { title: 'Langue', value: data.mediainfo.mediainfo_texts[_i2].language });
                }

                for (var _i3 in data.mediainfo.mediainfo_images) {
                    schema = schema.concat({ separator: 'h4', value: 'Image' }, { title: 'Format', value: data.mediainfo.mediainfo_images[_i3].format });
                }
            }

            if (typeof data.siegfrieds !== 'undefined' && data.siegfrieds.length) {
                schema = schema.concat({ separator: 'h3', value: 'Siegfried' });

                for (var _i4 in data.siegfrieds) {
                    schema = schema.concat({ title: 'Pronom', value: data.siegfrieds[_i4].pronom }, { title: 'Format', value: data.siegfrieds[_i4].format }, { title: 'Mime', value: data.siegfrieds[_i4].mime });
                }
            }

            for (var _i5 in schema) {
                value = typeof schema[_i5].callback !== 'undefined' ? schema[_i5].callback(schema[_i5].value) : schema[_i5].value;

                if (typeof schema[_i5].separator !== 'undefined') {
                    div.append($('<' + schema[_i5].separator + '></' + schema[_i5].separator + '>').append(schema[_i5].value));
                } else {
                    div.append($('<span></span>').addClass('field').append(schema[_i5].title)).append($('<span></span>').addClass('value').append(value));
                }
            }

            $('#info-file').find('.modal-body').html('').append(div);
        }

        /**
         * Ajoute un fichier selectionné
         *
         * @param {string} id
         */

    }, {
        key: 'fileSelect',
        value: function fileSelect(id) {
            var trId = '#table-' + genericConfiguration.tableGenerator.instanceIndex + '-tr-' + id;
            $(trId).addClass('warning').find('td.transfert').append($('<div class="text-center"><i aria-hidden="true" class="fa fa-2x fa-spinner faa-spin animated"></i></div>'));
            $(trId + ' td.action a').disable();
            $.ajax({
                url: genericConfiguration.selectUrl + '/' + id,
                success: function success(content, textStatus, jqXHR) {
                    var data;
                    if (jqXHR.getResponseHeader('X-Asalae-Success') === 'true') {
                        genericConfiguration.tableGenerator.removeDataId(id);
                        $(trId).fadeOut(400, function () {
                            AsalaeGlobal.updatePaginationAjax($(genericConfiguration.paginationElement));
                            $(this).remove();
                            $(genericConfiguration.tableGenerator.table).trigger('change');
                        });
                    } else {
                        $(trId + ' td.action a').enable().closest('tr').addClass('danger').removeClass('warning').find('.fa-spinner').parent().remove();
                        if ((typeof content === 'undefined' ? 'undefined' : _typeof(content)) === 'object' && typeof content.report === 'string') {
                            $(trId + ' td.transfert span.error-span').text(content.report);
                        } else {
                            $(trId + ' td.transfert span.error-span').text(genericConfiguration.messages.selectionFailed);
                        }
                        data = genericConfiguration.tableGenerator.getDataId(id);
                        data.transfert = $(trId + ' td.transfert').html();
                        data.valid = false;
                        genericConfiguration.tableGenerator.replaceDataId(id, data);
                    }
                },
                error: function error() {
                    var data;
                    $(trId + ' td.action a').enable().closest('tr').addClass('danger').removeClass('warning').find('.fa-spinner').parent().remove();
                    data = genericConfiguration.tableGenerator.getDataId(id);
                    data.valid = false;
                    genericConfiguration.tableGenerator.replaceDataId(id, data);
                    alert("Une erreur a eu lieu");
                }
            });
        }

        /**
         * Ajoute les fichier selectionnés (table upload)
         */

    }, {
        key: 'addSelected',
        value: function addSelected() {
            $(genericConfiguration.tableGenerator.table).find('tbody > tr > td.td-checkbox input:enabled:checked').each(function () {
                $(this).disable();
                GenericUploader.fileSelect($(this).val());
            });
            $(genericConfiguration.tableGenerator.table).find('input:checked').prop('checked', false).enable();
        }
    }]);

    return GenericUploader;
}();

//# sourceMappingURL=asalae.upload-compiled.js.map